/********************************************************************************/
/*    H   H Y   Y TTTTT EEEEE  CCC       HYTEC ELECTRONICS LTD                  */
/*    H   H  Y Y    T   E     C          5 Cradock Road,                        */
/*    HHHHH   Y     T   EEE   C          Reading, Berks.    Tel: 0118 9757770	*/
/*    H   H   Y     T   E     C          RG2 0JT            Fax: 0118 9757566	*/
/*    H   H   Y     T   EEEEE  CCC       Web: www.hytec-electronics.co.uk       */
/********************************************************************************/
/*                                                                              */
/*				IOC9010 PROJECT	--- Generic Functions			 				*/
/*							for Linux Platform				    				*/
/*                                                                              */
/********************************************************************************/
/*  Source file name      :-      IOCGeneric.cpp								*/
/*  Definition header file:		  IOCGeneric.h									*/
/*								  IOCLinuxDriver.h								*/
/*  Platform			  :		  Linux               							*/
/*  Initial creation date :-      11-AUG-2006                                   */
/*                                                                              */
/*  Original Developers   :-      Jim S Chen                                    */
/********************************************************************************/
/*                                                                              */
/*  Description :-                                                              */
/*                                                                              */
/*  This library provides a series of functions that can be used to access 		*/
/*	IOC IP card's memory spaces and IOC9010 carrier board memory and I/O 		*/
/*	spaces. It wraps the device driver into a user friendly functions to 		*/
/*	provide	the user a clear and meanlingful interfaces. 						*/
/*																				*/
/*	It is called Generic because it can access IP card momery, I/O and 			*/
/*	register spaces of any kind that plugged into the IOC9010 carrier board 	*/
/*	slot, not restricted to Hytec IP cards.										*/
/*                                                                              */
/*	The industry pack memory area is 16 Mbytes, organised as 16-bit words. It 	*/
/*	has the following arragement:												*/
/*																				*/
/*		Offset				Contents											*/
/*	0000000h-01FFFFEh	Memory area of industry pack A, 2Mbytes					*/
/*	0200000h-03FFFFEh	Memory area of industry pack A, 2Mbytes					*/
/*	0400000h-05FFFFEh	Memory area of industry pack A, 2Mbytes					*/
/*	0600000h-07FFFFEh	Memory area of industry pack A, 2Mbytes					*/
/*	0800000h-09FFFFEh	Memory area of industry pack A, 2Mbytes					*/
/*	0A00000h-0BFFFFEh	Memory area of industry pack A, 2Mbytes					*/
/*	0C00000h-0DFFFFEh	Not used (spare)										*/
/*	0E00000h-0E00FFEh	Access to industry pack A-F I/O, ID and INT areas		*/
/*																				*/
/*	The last area is sub-divided as follows:									*/
/*																				*/
/*	Offset				Contents												*/
/*	000h-07Eh	Industry pack A I/O registers (64 bytes)						*/
/*	080h-0FEh	Industry pack A ID registers (64 bytes)							*/
/*	100h-17Eh	Industry pack B I/O registers (64 bytes)						*/
/*	180h-1FEh	Industry pack B ID registers (64 bytes)							*/
/*	200h-27Eh	Industry pack C I/O registers (64 bytes)						*/
/*	280h-2FEh	Industry pack C ID registers (64 bytes)							*/
/*	300h-37Eh	Industry pack D I/O registers (64 bytes)						*/
/*	380h-3FEh	Industry pack D ID registers (64 bytes)							*/
/*	400h-47Eh	Industry pack E I/O registers (64 bytes)						*/
/*	480h-4FEh	Industry pack E ID registers (64 bytes)							*/
/*	500h-57Eh	Industry pack F I/O registers (64 bytes)						*/
/*	580h-5FEh	Industry pack F ID registers (64 bytes)							*/
/*	600h-7FEh	Not used														*/
/*	800h-8FEh	Industry pack A INT registers (128 bytes, only 2 words used)	*/
/*	900h-97Eh	Industry pack B INT registers (128 bytes, only 2 words used)	*/
/*	A80h-AFEh	Industry pack C INT registers (128 bytes, only 2 words used)	*/
/*	B00h-B7Eh	Industry pack D INT registers (128 bytes, only 2 words used)	*/
/*	C80h-CFEh	Industry pack E INT registers (128 bytes, only 2 words used)	*/
/*	D00h-D7Eh	Industry pack F INT registers (128 bytes, only 2 words used)	*/
/*	E00h-FFEh	Not used														*/
/*																				*/
/*  The interfaces exposed by this generic library is common to all platforms, 	*/
/*	which means no matter what platform and its implementation of this library	*/
/*	is, the user would have exactly the same interfaces. This enables the user 	*/	
/*	layer application portable to any platforms.								*/
/*                                                                              */
/*  The language used is C++ but the final product -- the Dynamic Link library	*/
/*	(dll) or shared library can be used by either C or C++ applications.		*/
/*																				*/
/*  (C)2006 Hytec Electronics Ltd.												*/
/*                                                                              */
/********************************************************************************/
/*                                                                              */
/*  Modification record :-                                                      */
/*                                                                              */
/*  Date modified |     Person    |   Description of modification  |  Version   */
/* ---------------|---------------|-------------------------------------------- */
/*    11/08/06    |	  Jim S Chen  |	  Initial copy				   |  0.0.1		*/
/*                |               |                                             */
/*                                                                              */
/********************************************************************************/

#pragma once

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include "link.h"
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/ioctl.h>

#include <pthread.h>

#include "9010LinuxDriver.h"

#define STATUS_SUCCESS	0				// operation was successful
#define FAN_NOT_EXIST	-9010001			// fan number doesn't exist
#define SENSOR_NOT_EXIST	-9010002		// sensor doesn't exist
#define PSHUBUTTON_NOT_EXIST	-9010003	// Push button doesn't exist
#define IP_INDEX_OUT_OF_RANGE	-9010004	// IP index out of range
#define CARRIER_REG_NOT_EXIST	-9010005	// register doesn't exist in the carrier board
#define IP_NOT_PRESENT	-9010006			// IP card doesn't exist (not inserted)
#define CHANNEL_NOT_EXIST	-9010007			// IP card doesn't exist (not inserted)
#define C_ERR_NODEVICE -9010008				// Failed to open the IOC device 
#define WRONG_DEVICE -9010009				// It isn't an IOC9010 device 


#define MAX_IP_INDEX 5
#define MAX_IP_INDEX_VME 3
#define MAX_FAN_INDEX 6
#define MAX_SENSOR_INDEX 4
#define MAX_BUTTON_INDEX 3

#define LCD_LINE_1		0
#define LCD_LINE_2		64
#define LCD_CENTRE_ON	0x8000
#define LCD_LINE_LEN	40

typedef unsigned short int USHORT;
typedef unsigned char UCHAR;
typedef unsigned long int ULONG;

class CIOCGeneric
{
private:
	long m_ErrorStatus;			// global error variable
	long m_hndFile;			// Handle obtain from CreateFile
	void ccinit( void );
	void ccfini( void );
    void LCD_Ready( void );
    long LCD_Cursor_Reset(void);
	int probe( void );
public:
	CIOCGeneric(void);
	~CIOCGeneric(void);

	long GetError()
	{
		return m_ErrorStatus;
	}
	
	//************************************************************
	//***************************IP CARD**************************
	//************************************************************
	
	//-------------IP registers read/write operation--------------
/*----------------------------------------------------------------------------------------------*/
/*Function:		Read Single IP Register															*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Executes a single action of 16 bit read from IP card							*/
/*				registers.																		*/
/*Prototype:	long IOC9010IPRead(int vmeslot, int ipslot, USHORT add, USHORT *data)			*/
/*Arguements:	vmprobe( void )eslot = VME slot that carrier board is plugged in (ignored by IOC9010)      	*/
/*          	ipslot = IP card slot number 0 ~ 5												*/
/*				add  = register address															*/
/*				*dprobe( void )ata = pointer to hold the register value										*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010IPRead(int vmeslot, int ipslot, USHORT add, USHORT *data);
    long IOC9010IPRead32(int vmeslot, int ipslot, USHORT add, ULONG *data);
/*----------------------------------------------------------------------------------------------*/
/*Function:		Read Multiple IP Registers (overloaded function)								*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Executes a number of action of 16 bit read from IP card							*/
/*				registers.																		*/
/*Prototype:	int IOC9010IPRead(int vmeslot, int ipslot, USHORT startadd, int len, USHORT *data)*/
/*Arguements:	vmeslot = VME slot that carrier board is plugged in (ignored by IOC9010)      	*/
/*              ipslot = IP card slot number 0 ~ 5												*/
/*				startadd  = start register address												*/
/*				len  = number of the registers													*/
/*				*dprobe( void )ata  = pointer of first address of the returning result						*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010IPRead(int vmeslot, int ipslot, USHORT startadd, int len, USHORT *data);
	
/*----------------------------------------------------------------------------------------------*/
/*Function:		Write to IP card	registers													*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Executes a single action of 16 bit write to IP card								*/
/*				registers.																		*/
/*Subroutine	voprobe( void )id IOC9010IPWrite(int vmeslot, int ipslot, USHORT regAddress, USHORT data)	*/
/*Arguements:	vmeslot = VME slot that carrier board is plugged in (ignored by IOC9010)      	*/
/*				data = data buffer where stores the writing value								*/
/*				ipprobe( void )slot = IP card index from 0 -- 5												*/
/*				add  = register address															*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010IPWrite(int vmeslot, int ipslot, USHORT add, USHORT data);
    long IOC9010IPWrite32(int vmeslot, int ipslot, USHORT add, ULONG data);
	
/*----------------------------------------------------------------------------------------------*/
/*Function:		Write to a batch of IP card registers	(overload function)						*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Executes a number of action of 16 bit write to IP card							*/
/*				registers.																		*/
/*Subroutine	long IOC9010IPWrite(int vmeslot, int ipslot, USHORT startadd, int len, USHORT *data)*/
/*Arguements:	vmeslot = VME slot that carrier board is plugged in (ignored by IOC9010)      	*/
/*              ipslot = IP card slot number 0 ~ 5												*/
/*				startadd  = start register address												*/
/*				len  = number of the registers													*/
/*				*data  = pointer of first address of the data to write							*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010IPWrite(int vmeslot, int ipslot, USHORT startadd, int len, USHORT *data);

	//------------------IP ID read operation----------------------
/*----------------------------------------------------------------------------------------------*/
/*Function:		Batch Read IP card identity Registers											*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Executes multiple action of 16 bit read from IP card identity    				*/
/*				registers.																		*/
/*Prototype:	long IOC9010GetConfig(int vmeslot, int ipslot, int len, USHORT *data)			*/
/*Arguements:	vmeslot = VME slot that carrier board is plugged in (ignored by IOC9010)      	*/
/*              ipslot = IP card slot number 0 ~ 5												*/
/*				len = how many registers to read												*/
/*				*data = pointer to the ID array													*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
	long IOC9010GetConfig(int vmeslot, int ipslot, int len, USHORT *data);
	
/*----------------------------------------------------------------------------------------------*/
/*Function:		Read Carrier card identity Registers											*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Reads Carrier Card ID as an 16 bit Value Read from IP card identity    			*/
/*				registers.																		*/
/*Prototype:	long IOC9010GetCarrier(int vmeslot, USHORT *data)			                    */
/*Arguements:	vmeslot = VME slot that carrier board is plugged in (ignored by IOC9010)      	*/
/*				*data = pointer to the Carrier Card Type  									    */
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010GetCarrier(int vmeslot, USHORT *data);

	//-------------IP memory read/write operation-----------------
/*----------------------------------------------------------------------------------------------*/
/*Function:		Read a batch from IP memory area												*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Executes a number of action of 16 bit read from IP card							*/
/*				memory spaces.																	*/
/*Subroutine	long IOC9010IPMemoryRead(int vmeslot, int ipslot, USHORT offset, int len, USHORT *data)*/
/*Arguements:	vmeslot = VME slot that carrier board is plugged in (ignored by IOC9010)      	*/
/*          	ipslot = IP card slot number 0 ~ 5												*/
/*				offset  = start memory address													*/
/*				len  = number of the memorys 													*/
/*				*data  = pointer of first address of the data to write							*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
	long IOC9010IPMemoryRead(int vmeslot, int ipslot, ULONG offset, int len, USHORT *data);
	
/*----------------------------------------------------------------------------------------------*/
/*Function:		Write to a batch of IP card memory												*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Executes a number of action of 16 bit write to IP card							*/
/*				memory spaces.																	*/
/*Subroutine	long IOC9010IPMemoryWrite(int vmeslot, int ipslot, USHORT offset, int len, USHORT *data)*/
/*Arguements:	vmeslot = VME slot that carrier board is plugged in (ignored by IOC9010)      	*/
/*              ipslot = IP card slot number 0 ~ 5												*/
/*				offset  = start memory address													*/
/*				len  = number of the memorys													*/
/*				*data  = pointer of first address of the data to write							*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010IPMemoryWrite(int vmeslot, int ipslot, ULONG offset, int len, USHORT *data);

	
	//************************************************************
	//*********************CARRIER BOARD**************************
	//************************************************************
	
	//-------------carrier board functions------------------------
/*----------------probe( void )------------------------------------------------------------------------------*/
/*Function:		Read Carrier Board Single Register												*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Executes a single action of 16 bit read from carrier card						*/
/*				registers.																		*/
/*Prototype:	long IOC9010CarrierRead(USHORT add, USHORT *data)								*/
/*Arguements:	add  = register address															*/
/*				*data  = pointer to hold the register value										*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
	long IOC9010CarrierRead(USHORT add, USHORT *data);
	
/*----------------------------------------------------------------------------------------------*/
/*Function:		Read a batch Carrier Board Registers (overload function)						*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Executes an action of 16 bit read from a number of len from						*/
/*				carrier board registers.														*/
/*Prototype:	void IOC9010CarrierRead(USHORT startadd, int len, USHORT *data)					*/
/*Arguements:	startadd  = start register address												*/
/*				len  = number of the registers													*/
/*				*data  = pointer of first address of the returning result						*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010CarrierRead(USHORT startadd, int len, USHORT *data);

/*----------------------------------------------------------------------------------------------*/
/*Function:		Write to single carrier board register											*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Exprobe( void )ecutes a single action of 16 bit write to carrier card						*/
/*				registers.																		*/
/*Prototype:	long IOC9010CarrierWrite(USHORT add, USHORT data)								*/
/*Arguements:	data = data to write															*/
/*				add  = register address															*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
	long IOC9010CarrierWrite(USHORT add, USHORT data);
	
/*----------------------------------------------------------------------------------------------*/
/*Function:		Write to multiple carrier board registers (overload function)					*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Executes a number of action of 16 bit write to carrier card						*/
/*				registers.																		*/
/*Prototype:	long IOC9010CarrierWrite(USHORT startadd, USHORT len, USHORT *data)				*/
/*Arguements:	startadd  = start register														*/
/*				len  = number of the registers													*/
/*				*data  = pointer of first address of the writing data							*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010CarrierWrite(USHORT startadd, USHORT len, USHORT *data);
	
/*----------------probe( void )------------------------------------------------------------------------------*/
/*Function:		Set IP Slot Frequncy															*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Seprobe( void )t industry pack working frequency for selected slot. The frequency could		*/
/*				be set ot either 8MHz or 32MHz individually for each slot.						*/
/*Prototype:	long IOC9010SetIPFrequncy(int ipslot, int clock)								*/
/*Arguements:	ipslot  = industry pack slot, number 0 ~ 5										*/
/*				clock  = 0, 8MHz; = 1, 32MHz													*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
	long IOC9010SetIPClock(int ipslot, int clock);

/*----------------------------------------------------------------------------------------------*/
/*Function:		Geprobe( void )t carrier board Fan Speed														*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Read fan speed register from the carrier board and translate it 				*/
/*				to speed in RPM wise															*/
/*Prototype:	long IOC9010GetCarrierFanSpeed(int fan, int *data)								*/
/*Arguements:	fan  = fan number 0 ~ 5															*/
/*				*data  = pointer to the variable to hold the fan speed							*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010GetCarrierFanSpeed(int fan, int *data);
	
/*----------------------------------------------------------------------------------------------*/
/*Function:		Set carrier board Fan Speed														*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Write fan speed register to the carrier board register			 				*/
/*Prototype:	long IOC9010SetCarrierFanSpeed(int fan, int speed)								*/
/*Arguements:	fan  = fan number 0 ~ 5. If fan = -1, set auto control							*/
/*				speed  = 0, 1, 2 for stop, high and low respectively							*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010SetCarrierFanSpeed(int fan, int speed);
	
/*----------------------------------------------------------------------------------------------*/
/*Function:		Get carrier board sensor temperature											*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Read carrier board temperature sensors and convert them	to degree				*/
/*Prototype:	long IOC9010GetCarrierTemp(int sensor, int *data)								*/
/*Arguements:	sensor  = sendor number 0 ~ 4													*/
/*				*data = pointer to the variable to hold the result								*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010GetCarrierTemp(int sensor, int *data);
	
/*----------------------------------------------------------------------------------------------*/
/*Function:		Get carrier board pushbutton status												*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Read carrier board push button status											*/
/*Prototype:	long IOC9010GetCarrierSwitch(int pushButton, int *data)							*/
/*Arguements:	pushbutton  = pushbutton number 0 ~ 3											*/
/*					0:UP, 1:OK, 2:DOWN, 3:RESET													*/
/*				*data = 1, pushed down, = 0, lifted												*/
/*Return:		erprobe( void )ror code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010GetCarrierSwitch(int pushButton, int *data);

	//-----------------Carrier LCD functions----------------------
/*----------------------------------------------------------------------------------------------*/
/*Function:		Read carrier board LCD strings													*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Read carrier board push button status											*/
/*Prototype:	long IOC9010LCDRead(char *string)												*/
/*Arguements:	*string  = pointer to the char variable that holds the string. must be greater	*/
/*                      than 80 in size                                                         */
/*Return:		erprobe( void )ror code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
	long IOC9010LCDRead(char *string);
	
/*----------------------------------------------------------------------------------------------*/
/*Function:		Write a string to carrier board LCD												*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Write a string to carrier board LCD screen										*/
/*Prototype:	long IOC9010LCDWrite(USHORT position, char *string)								*/
/*Arguements:	positioin = use LCD_CENTRE_ON | LCD_LINE_1 to display on line 1 and centre;     */
/*                      user LCD_CENTRE_ON | LCD_LINE_2 to display on line 2 and centre         */
/*				*string  = pointer to the char variable that holds the string					*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				ifprobe( void ) successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010LCDWrite(USHORT position, char *string);
	
/*----------------------------------------------------------------------------------------------*/
/*Function:		Clear carrier board LCD display													*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	clear carrier board LCD screen													*/
/*Prototype:	long IOC9010LCDClear(void)														*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
    long IOC9010LCDClear(void);

/*----------------------------------------------------------------------------------------------*/
/*Function:		Enable/Disable LCD auto control													*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Enprobe( void )able/Disable LCD auto control by push buttons									*/
/*Prototype:	long IOC9010FPC(int iMMIOnOff)													*/
/*Arguements:	iMMIOnOff = 0, Disable auto; = 1, enable auto									*/
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
	long IOC9010FPC(int iMMIOnOff);


/*----------------------------------------------------------------------------------------------*/
/*Function:		Read 3331 CSR Register - used for debug purposes								*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Reads 3331 CSR register - useful for debug applications							*/
/*Prototype:	long Read3331CSR(USHORT *data)								                    */
/*Arguements:	*data = pointer to the CSR Value variable  									    */
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
	long Read5331CSR(USHORT *data);    /* For Compatibility with earlier version only */
	long Read3331CSR(USHORT *data);

/*----------------------------------------------------------------------------------------------*/
/*Function:		Reset 3331 - Software reset of the module										*/
/*----------------------------------------------------------------------------------------------*/
/*Description:	Resets the 3331 module															*/
/*Prototype:	long Reset3331(USHORT *data)								                    */
/*Arguements:	none 									    */
/*Return:		error code and also in m_ErrorStatus											*/
/*				if successful, return 0															*/
/*----------------------------------------------------------------------------------------------*/
	long Reset3331(void);


};

extern "C" CIOCGeneric* CreateIOCGeneric();
extern "C" void DeleteIOCGeneric( CIOCGeneric* instance );
