/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2000-2001 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: comtest.c
*
**********************************************************************/

/*
 * A program for testing serial communication on WinNT.
 * Author: Eugene Zharkov
 * Created: 13-Oct-2000
 */

#include <stdio.h>
#include <windows.h>
#include <vsys_version.h>
#include <vsys_errors.h>
#include <vsys_cli.h>

static char versionString[] = VSYS_VERSION_STRING ".1";
static char commandName[] = "comtest";

static HANDLE portHandle;
static char * portName;
static vcli_t * cli;

static struct {
  int state;
  int read;
  int write;
  int xchg;
  unsigned length;
  unsigned timeout;
  unsigned baudRate;
} q;

/*
 *********************************************************************
 */

#define RRR() rrr (__LINE__)
#define NT_ERROR() ntError (__LINE__)

static void rrr (int line) {
  fprintf (stderr, "\n*** error at %d\n", line);
  exit (EXIT_FAILURE);
}

static void ntError (int line) {
  printf ("GetLastError = %d\n", GetLastError ());
  rrr (line);
}


static void cliError (void) {
  vcliPerror (cli, NULL);
  exit (EXIT_FAILURE);
}

/*
 *********************************************************************
 */

static void hexdump (char *title, void *ptr, unsigned length) {
  unsigned char *c = ptr;
  unsigned i;
  printf ("DUMP %s", title);
  for (i = 0; i < length; i++) {
    if ((i % 16) == 0) printf ("\n%04X", i);
    printf (" %02X", c[i]);
  }
  printf ("\n");
}

/*
 *********************************************************************
 */

static void printState (void) {

  unsigned baudRate;
  COMMTIMEOUTS tmo;
  DCB dcb;

  if (! GetCommState (portHandle, &dcb)) NT_ERROR ();
  if (! GetCommTimeouts (portHandle, &tmo)) NT_ERROR ();

  switch (dcb.BaudRate) {
#define M(cbr) case CBR_##cbr: baudRate = cbr; break
    M (110);
    M (300);
    M (600);
    M (1200);
    M (2400);
    M (4800);
    M (9600);
    M (14400);
    M (19200);
    M (38400);
    M (56000);
    M (57600);
    M (115200);
    M (128000);
    M (256000);
#undef M
  default:
    baudRate = dcb.BaudRate;
  }
  printf ("BaudRate : %u\n", baudRate);

  if (dcb.fBinary != 1) RRR ();
#define M(flag, descr) printf (descr " : %s\n", flag ? "Yes" : "No");
  M (dcb.fParity, "Parity Checking Enabled");
  M (dcb.fOutxCtsFlow, "CTS output flow control");
  M (dcb.fOutxDsrFlow, "DSR output flow control");
  M (dcb.fDsrSensitivity, "DSR sensitivity");
  M (dcb.fTXContinueOnXoff, "XOFF continues Tx");
  M (dcb.fOutX, "XON/XOFF out flow control");
  M (dcb.fInX, "XON/XOFF in flow control");
  M (dcb.fErrorChar, "enable error replacement");
  M (dcb.fNull, "enable null stripping");
  M (dcb.fAbortOnError, "abort on error");
#undef M

  printf ("DTR flow control type : ");
  switch (dcb.fDtrControl) {
#define M(x) case x: printf (#x "\n"); break;
    M (DTR_CONTROL_DISABLE);
    M (DTR_CONTROL_ENABLE);
    M (DTR_CONTROL_HANDSHAKE);
#undef M
  default: RRR ();
  }

  printf ("RTS flow control : ");
  switch (dcb.fRtsControl) {
#define M(x) case x: printf (#x "\n"); break;
    M (RTS_CONTROL_DISABLE);
    M (RTS_CONTROL_ENABLE);
    M (RTS_CONTROL_HANDSHAKE);
    M (RTS_CONTROL_TOGGLE); 
#undef M
  default: RRR ();
  }

  printf ("transmit XON threshold : %d\n", dcb.XonLim);
  printf ("transmit XOFF threshold : %d\n", dcb.XoffLim);
  printf ("number of bits/byte, 4-8 : %d\n", dcb.ByteSize);

  printf ("Parity : ");
  switch (dcb.Parity) {
  case 0: printf ("NONE\n"); break;
  case 1: printf ("ODD\n"); break;
  case 2: printf ("EVEN\n"); break;
  case 3: printf ("MARK\n"); break;
  case 4: printf ("SPACE\n"); break;
  default: RRR ();
  }

  printf ("StopBits : ");
  switch (dcb.StopBits) {
  case 0: printf ("1\n"); break;
  case 1: printf ("1.5\n"); break;
  case 2: printf ("2\n"); break;
  default: RRR ();
  }

  printf ("Tx and Rx XON character : %d\n", dcb.XonChar);
  printf ("Tx and Rx XOFF character : %d\n", dcb.XoffChar);
  printf ("error replacement character : %d\n", dcb.ErrorChar);
  printf ("end of input character : %d\n", dcb.EofChar);
  printf ("received event character : %d\n", dcb.EvtChar);

#define M(x) printf (#x " = %d\n", tmo.x)
  M (ReadIntervalTimeout);
  M (ReadTotalTimeoutMultiplier);
  M (ReadTotalTimeoutConstant);
  M (WriteTotalTimeoutMultiplier);
  M (WriteTotalTimeoutConstant);
#undef M
}

/*
 *********************************************************************
 */

static void setTimeouts (void) {
  COMMTIMEOUTS t;
  t.ReadIntervalTimeout = 1000;
  t.ReadTotalTimeoutMultiplier = 100;
  t.ReadTotalTimeoutConstant = q.timeout;
  t.WriteTotalTimeoutMultiplier = 100;
  t.WriteTotalTimeoutConstant = q.timeout;
  if (! SetCommTimeouts(portHandle, &t)) RRR ();
}

static void setComm (void) {
  DCB dcb;
  setTimeouts ();
  if (! GetCommState (portHandle, &dcb)) RRR ();
  dcb.BaudRate = q.baudRate;
  dcb.Parity = ODDPARITY;      
  dcb.StopBits = ONESTOPBIT;
  if (! SetCommState (portHandle, &dcb)) RRR ();
}

/*
 *********************************************************************
 */

static void openPort (void) {
  portHandle = CreateFile (portName, GENERIC_READ | GENERIC_WRITE,
			   0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  if (portHandle == INVALID_HANDLE_VALUE) NT_ERROR ();
  if (q.state) printState ();
}

/*
 *********************************************************************
 */

static void stateCmd (void) {
  openPort ();
}

/*
 *********************************************************************
 */

static void readCmd (void) {

  char buff[1024];
  unsigned actualLen;
  unsigned len = q.length;

  openPort ();
  if (len == 0 || len > sizeof buff) RRR ();
  
  setComm ();
  if (! ReadFile (portHandle, buff, len, &actualLen, NULL)) NT_ERROR ();
  if (len != actualLen) {
    if (actualLen != 0)
      hexdump ("readCmd", buff, actualLen);
    printf ("actual len = %d\n", actualLen);
    RRR ();
  }
  hexdump ("readCmd", buff, actualLen);
}

/*
 *********************************************************************
 */

static void writeCmd (void) {

  char buff[1024];
  unsigned actualLen;
  unsigned len = q.length;
  unsigned i;

  openPort ();
  if (len == 0 || len > sizeof buff) RRR ();
  for (i = 0; i < len; i++) {
    buff[i] = (i % 20) + 'a';
    if (i == 70) buff[i] = '\n';
  }
  setComm ();
  if (! WriteFile (portHandle, buff, len, &actualLen, NULL)) NT_ERROR ();
  if (len != actualLen) RRR ();
}

/*
 *********************************************************************
 */

static void xchgCmd (void) {

  char out[] = "abc\ndef\nghi5uertiyrekghfdgihdfghdfg\n";
  char in[1024];
  unsigned actualLen;

  openPort ();
  setComm ();
  hexdump ("going to write", out, sizeof out);
  if (! WriteFile (portHandle, out, sizeof out, &actualLen, NULL)) NT_ERROR ();
  if (actualLen != sizeof out) RRR ();
  if (! ReadFile (portHandle, in, sizeof in, &actualLen, NULL)) NT_ERROR ();
  printf ("received %d\n", actualLen);
  hexdump ("xchg", in, actualLen);
}

/*
 *********************************************************************
 */

static void parseArguments (int argc, char **argv) {

  if ((cli = vcliInit ()) == NULL) RRR ();

  if (vcliSetTitle (cli, "COMTEST Utility") != VCLI_OK ||
      vcliSetVersion (cli, versionString) != VCLI_OK ||
      vcliSetCommandName (cli, commandName) != VCLI_OK ||
      vcliEnableSortedHelp (cli) != VCLI_OK) {
    cliError ();
  }

  if (vcliVparse (cli,
		  argc, argv,
		  "p port $string", &portName,
		  "q state", &q.state,
		  "q read", &q.read,
		  "q write", &q.write,
		  "q xchg", &q.xchg,
		  "disallow any2 (write, read)",
		  "q baudRate $uint default 9600", &q.baudRate,
		  "q length $uint default 8", &q.length,
		  "q timeout $uint default 1000", &q.timeout,
		  NULL) != VCLI_OK) {
    cliError ();
  }
}

/*
 *********************************************************************
 */

int main (int argc, char **argv) {
  parseArguments (argc, argv);
  if (q.read) readCmd ();
  else if (q.write) writeCmd ();
  else if (q.xchg) xchgCmd ();
  else if (q.state) stateCmd ();
  else RRR ();
  return EXIT_SUCCESS;
}
