/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2000 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec/conversion_routines.c
*
**********************************************************************/

/*
 * A conversion routines example.
 */

#include <vdb_type.h>
#include <vhytec.h>

typedef struct {
  int cparam1;
  int cparam2;
  int cparam3;
} myChannel_t;

typedef struct {
  int debug;
  myChannel_t *chan;
} myEnv_t;

/*
 *********************************************************************
 * Error handling
 *********************************************************************
 */

#define RRR() rrr (__LINE__)

static void rrr (int line) {
  printf ("fatal error at %d, %s\n", line, __FILE__);
  exit (EXIT_FAILURE);
}

/*
 *********************************************************************
 * An initialization routine example.
 *********************************************************************
 */

int init (vhytecEnv_t *env) {

  myEnv_t * my;
  unsigned i;

  /* Make sure that the versions of our module and the vhytec scanner
     are compatible. */

  VHYTEC_CHECK_VERSION (env);

  /* Allocate our own addition to the vhytecEnv_t. */

  if ((my = calloc (1, sizeof *my)) == NULL) RRR ();
  env->user = my;

  /* Parse the userParameter (if specified). In this example it is just
     an integer number indicating a debug level. */

  if (env->commandLineParameter != NULL) {
    my->debug = atoi (env->commandLineParameter);
    if (my->debug != 0)
      printf ("userParameter = <%s>\n", env->commandLineParameter);
  }

  if (my->debug)
    printf ("Number of channels = %d\n", env->numChannels);

  /* The number of channels parameter can be used to allocate and
     prefetch information needed to perform the conversion. */

  my->chan = calloc (env->numChannels, sizeof my->chan[0]);
  if (my->chan == NULL) RRR ();
  for (i = 0; i < env->numChannels; i++) {
    my->chan[i].cparam1 = 2; /* vdb_get (env->channel[i].chix, ...) */
    my->chan[i].cparam2 = 3;
    my->chan[i].cparam3 = 4;
  }

  return VHYTEC_OK;
}

/*
 *********************************************************************
 * An example of a conversion routine from external format to
 * internal hardware format.
 *********************************************************************
 */

int toInt (vhytecEnv_t *env, vhytecChannel_t *chan, vhytecValue_t *val) {

  myEnv_t *my = env->user;

  if (my->debug) {
    printf ("External to internal for channel ID: %d\n", chan->id);
    if (chan->hwDtype != VDB_DTYPE_INT16) RRR ();
    if (chan->dbDtype != VDB_DTYPE_FLOAT) RRR ();
  }

  val->int16 = (VINT16) val->float32;

  return VHYTEC_OK;
}

/*
 *********************************************************************
 * An example of a conversion routine from internal hardware format
 * to external database and/or logger format.
 *********************************************************************
 */

int toExt (vhytecEnv_t *env, vhytecChannel_t *chan, vhytecValue_t *val) {

  myEnv_t *my = env->user;
  unsigned id = chan->id;
  myChannel_t *myChan = &my->chan[id];

  if (my->debug) {
    printf ("Internal to external for channel ID: %d\n", chan->id);
    if (chan->hwDtype != VDB_DTYPE_INT16) RRR ();
  }

  /* The same routine can be used to convert different types of data. */

  switch (chan->dbDtype) {
  case VDB_DTYPE_INT:
    val->int32 = val->int16 * myChan->cparam1 + myChan->cparam2;
    break;
  case VDB_DTYPE_FLOAT:
    val->float32 = (float) val->int16 * myChan->cparam3;
    break;
  }

  return VHYTEC_OK;
}
