/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2000-2004 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec/server.c
*
**********************************************************************/

/*
 * A hytec server simulator
 * Author: Eugene Zharkov
 * Started: 22-Nov-2000
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <vsys_version.h>
#include <vsys_errors.h>
#include <vsys_time.h>
#include <vsys_cli.h>
#include <vthread.h>
#include <vsocket.h>

#include "version.h"
#include "internal.h"
#include "io.h"

static char programName[] = "Modbus Server Simulator";
static char commandName[] = "server";
static char versionString[] = VSYS_VERSION_STRING "." VERSION_STRING;

static struct {
  int port;		/* TCP port number. */
  bool banner;
  bool threads;
  bool once;		/* Exit after serving the first client. */
  bool tcpNoDelay;
  char * driver;
  char * device;
  vthreadMutex_t mutex;
} data;

/*
 *********************************************************************
 */

#define LOCK_ERROR() lockError (__LINE__)

static void lockError (int line) {
  (void) printf ("vhytec/server: error at %d", line);
  exit (EXIT_FAILURE);
}

/*
 *********************************************************************
 */

static void tsPrint (char *format, ...) {
  va_list args;
  char asctim[50];
  vtime bintim;
  if (data.threads && vthreadMutexLock (&data.mutex) != VTHREAD_OK)
    LOCK_ERROR ();
  (void) vsys_get_time (&bintim);
  asctim[0] = 0;
  (void) vsysc_asctime (asctim, sizeof (asctim), &bintim);
  (void) printf ("%s: ", asctim);
  va_start (args, format);
  (void) vprintf (format, args);
  (void) printf ("\n");
  (void) fflush (stdout);
  if (data.threads && vthreadMutexUnlock (&data.mutex) != VTHREAD_OK)
    LOCK_ERROR ();
  va_end (args);
}

/*
 *********************************************************************
 */

#define RRR() rrr (__LINE__)
#define RRR1(value) rrr1 (__LINE__, value)
#define FIXME() rrr (__LINE__)

static void rrr (int line) {
  tsPrint ("vhytec/server: error at %d", line);
  exit (EXIT_FAILURE);
}

static void rrr1 (int line, int value) {
  tsPrint ("vhytec/server: error %d at %d", value, line);
  exit (EXIT_FAILURE);
}

/*
 *********************************************************************
 */

static void runClientSession (vsocket_t *socket) {

  unsigned expectedResponseLen;
  unsigned responseBufferSize;
  unsigned requestBufferSize;
  unsigned byteCount;
  unsigned count;
  VUINT8* response;
  VUINT8* request;
  int hldr;
  
  Vmbport* vp = vmbAlloc (&request, &requestBufferSize,
			  &response, &responseBufferSize);
  struct header_s {
    VUINT8 tid[2];
    VUINT8 pid[2];
    VUINT8 zero;
    VUINT8 length;
    VUINT8 unit;
  } *header = (struct header_s*) (request - sizeof *header);

  if (vp == NULL) RRR ();

  if (vmbSet (vp, VMB_O_TYPE, (void*) VMB_TYPE_USER) != VMB_OK ||
      vmbSet (vp, VMB_O_DRIVER, data.driver) != VMB_OK ||
      vmbConnect (vp, data.device) != VMB_OK) {
    tsPrint ("vmb error: %s", vmbStrerror (vp));
    RRR ();
  }

  while (1) {

    VUINT8 *buff = (VUINT8*) header;
    unsigned buffSize = requestBufferSize + sizeof *header;
    unsigned totalLength = 0; /* we don't know it until header->length
				 is read. */
    unsigned bytesRead = 0;

    while (1) {
      unsigned n;
      if (vsocketRead (socket, buff, buffSize, &n) != VSOCKET_OK) {
	if (vsocketIsEof (socket)) {
	  if (bytesRead != 0) RRR ();
	  break;
	}
	RRR ();
      }
      bytesRead += n;
      if (totalLength == 0 &&
	  bytesRead > (unsigned) (&header->length - (VUINT8*) header)) {
	unsigned totalLength = sizeof *header - 1/*unit*/ + header->length;
	if (bytesRead == totalLength) break;
	if (bytesRead > totalLength) RRR ();
	buffSize = totalLength - bytesRead;
	buff += n;
	continue;
      }
      buff += n;
      buffSize -= n;
    }

    if (bytesRead == 0) break;

    switch (request[0]) {
    case 1:
    case 2:
      count = G16 (request + 3);
      byteCount = (count + 7) / 8;
      expectedResponseLen = 2 + byteCount; 
      break;
    case 3:
    case 4:
      count = G16 (request + 3);
      byteCount = count * 2;
      expectedResponseLen = 2 + byteCount; 
      break;
    default:
      RRR1 (request[0]);
    }

    if (vmbTransact (vp, header->unit, header->length - 1,
		     expectedResponseLen, &hldr) != VMB_OK) RRR ();

    header->length = expectedResponseLen + 1;
    (void) memcpy (response - sizeof *header, header, sizeof *header);
    if (vsocketWriteAll (socket, response - sizeof *header,
			 expectedResponseLen + sizeof *header) != VSOCKET_OK)
      RRR ();
  }

  if (vmbDisconnect (vp) != VMB_OK) {
    tsPrint ("vmb error: %s", vmbStrerror (vp));
    RRR ();
  }
}

/*
 *********************************************************************
 */

static void thread (vsocket_t *socket) {
  tsPrint ("thread: accepted connection");
  runClientSession (socket);
  if (vsocketClose (socket) != VSOCKET_OK) RRR ();
  tsPrint ("thread: closed connection");
}

/*
 *********************************************************************
 */

static void banner (void) {
  if (data.banner)
    (void) printf ("Vsystem %s Version %s\n", programName, versionString);
}

/*
 *********************************************************************
 */

static void parseCommandLine (int argc, char *argv[]) {

  char * outputFileName;
  vcli_t * cli;

  if ((cli = vcliInit ()) == NULL) RRR ();

  if (vcliSetTitle (cli, programName) != VCLI_OK ||
      vcliSetVersion (cli, versionString) != VCLI_OK ||
      vcliSetCommandName (cli, commandName) != VCLI_OK ||
      vcliEnableSortedHelp (cli) != VCLI_OK ||
      vcliVparse
      (cli,
       argc, argv,
       "q banner default 1", &data.banner,
       "q device $string disallow ! device", &data.device,
       "q driver $string disallow ! driver", &data.driver,
       "q threads", &data.threads,
       "q once default 1", &data.once,
       "q output $file", &outputFileName,
       "q port $int default 502", &data.port,
       "q tcpNoDelay", &data.tcpNoDelay,
       NULL) != VCLI_OK) {
    (void) vcliPerror (cli, NULL);
    RRR ();
  }

  if (outputFileName != NULL) {
    if (freopen (outputFileName, "w", stdout) == NULL) RRR ();
    banner ();
  }
}

/*
 *********************************************************************
 */

int main (int argc, char *argv[]) {

  vsocket_t *socket;
  vsocket_t *clientSocket;
  vsocklib_t *vsr;

  parseCommandLine (argc, argv);
  banner ();
  if ((vsr = vsocketInit ()) == NULL) RRR ();
  if (vsocketOpenServerSocket (vsr, data.port, &socket) != VSOCKET_OK) RRR ();

  if (data.tcpNoDelay) {
    tsPrint ("Setting TCP_NODELAY");
    if (vsocketEnableTcpNoDelay (socket) != VSOCKET_OK) RRR ();
  }

  if (data.threads)
    if (vthreadMutexInit (&data.mutex, NULL) != VTHREAD_OK) RRR ();

  while (1) {
    tsPrint ("Accepting connection");
    if (vsocketAccept (socket, &clientSocket) != VSOCKET_OK) RRR ();
    if (data.tcpNoDelay) {
      tsPrint ("Setting TCP_NODELAY");
      if (vsocketEnableTcpNoDelay (clientSocket) != VSOCKET_OK) RRR ();
    }
    if (data.threads) {
      VSYS_THREAD id;
      if (vsys_create_thread (0, (int(*)(void*)) thread,
			      clientSocket, &id) != VSYS_OK) RRR ();
    }
    else {
      tsPrint ("Accepted connection");
      runClientSession (clientSocket);
      if (vsocketClose (clientSocket) != VSOCKET_OK) RRR ();
      tsPrint ("Closed connection");
      if (data.once) {
	tsPrint ("Exiting");
	exit (EXIT_SUCCESS);
      }
    }
  }
}
