/******************************************************************************
*        ____
*      _-    -_    _------_
*     -        -_--        -_
*          _--------------_   -_
* 	_---              -_
*      _-                 ----___
*                         ---___
*      Copyright (c) 2000-2004 Vista Control Systems, Inc.
*      All rights reserved
*      176 Central Park Square * Los Alamos, N.M. 87544 * 505-662-2484
*
*      Filename: vhytec/sim2.c
*
**********************************************************************/

#include <vsys_internal.h>
#include <vsys_time.h>
#include "vhytec.h"
#include "internal.h"
#include "io.h"

typedef Vmbport vp_t;

typedef struct {
  union {
    VUINT8 *int8;
    VUINT16 *int16;
    VUINT32 *int32;
    float *real;
  } u;
  VUINT16 *discrete;
} slave_t;

typedef struct {
  int testNumber;
  int portNumber;
  int maxSlave;			/* Max slave number. */
  bool realRegisters;		/* Flag indicating that input registers
				   represent float data. */
  unsigned numInputDiscretes;	/* Number of input discretes per slave. */
  unsigned numInputRegisters;	/* Number of input registers per slave. */
  char * args;
  slave_t * slave;		/* Array of slaves. */
} sim_t;

/*
 *********************************************************************
 * Error handling
 *********************************************************************
 */

#define RRR() rrr (__LINE__, 0)
#define RRR1(pos) rrr (__LINE__, pos)

static int rrr (int line, int pos) {
  printf ("sim2: error at %d (%d)\n", line, pos);
  exit (EXIT_FAILURE);
  return -1;
}

/*
 *********************************************************************
 */

static void tsPrint (char* format, ...) {
  va_list args;
  char asctim[50];
  vtime bintim;
  (void) vsys_get_time (&bintim);
  asctim[0] = 0;
  (void) vsysc_asctime (asctim, sizeof (asctim), &bintim);
  (void) printf ("%s: ", asctim);
  va_start (args, format);
  (void) vprintf (format, args);
  (void) printf ("\n");
  (void) fflush (stdout);
  va_end (args);
}

/*
 *********************************************************************
 */

static void * fatalMalloc (unsigned size) {
  void * ptr = VMEM_MALLOC (size);
  if (ptr == NULL) RRR ();
  return ptr;
}

static void * fatalMalloc0 (unsigned size) {
  void * ptr = fatalMalloc (size);
  memset (ptr, 0, size);
  return ptr;
}

/*
 *********************************************************************
 */

int userConnect (vp_t *vp) {
  sim_t *sim = VMEM_CALLOC (1, sizeof *sim);
  char *p = vp->name;
  if (*p == 't') p++;
  sim->testNumber = strtol (p, &p, 10);
  if (*p == 'p') {
    p++;
    sim->portNumber = strtol (p, &p, 10);
  }
  if (*p == 'x') {
    p++;
    sim->maxSlave = strtol (p, &p, 10);
  }
  if (*p == 'x') {
    p++;
    sim->numInputDiscretes = strtol (p, &p, 10);
  }
  if (*p == 'x' || *p == 'f') {
    sim->realRegisters = 1;
    p++;
    sim->numInputRegisters = strtol (p, &p, 10);
  }
  if (*p == 'a') {
    p++;
    sim->args = fatalMalloc (strlen (p) + 1);
    strcpy (sim->args, p);
  }
  else {
    if (*p != 0) RRR ();
  }

  if (sim->maxSlave != 0) {
    int i;
    slave_t *slave = fatalMalloc0 (sim->maxSlave * sizeof *slave);
    if ((sim->numInputDiscretes & 15) != 0) RRR1 (sim->numInputDiscretes);
    for (i = 0; i < sim->maxSlave; i++) {
      if (sim->numInputDiscretes != 0) {
	slave[i].discrete = fatalMalloc0 (sim->numInputDiscretes / 8 *
					  sizeof slave[i].discrete[0]);
      }
      if (sim->numInputRegisters != 0) {
	slave[i].u.int16 = fatalMalloc0 (sim->numInputRegisters *
					 sizeof slave[i].u.int16[0]);
      }
    }
    sim->slave = slave;
  }

  vp->userData = sim;
  return 0;
}

/*
 *********************************************************************
 */

int userDisconnect (vp_t *vp) {
  sim_t *sim = vp->userData;
  if (sim != NULL) {
    if (sim->args != NULL) VMEM_FREE (sim->args);
    VMEM_FREE (sim);
    vp->userData = NULL;
  }
  return 0;
}

/*
 *********************************************************************
 */

static int test13 (vp_t *vp, int slave, unsigned requestLen,
		   unsigned expectedResponseLen) {

  VUINT8 *response = vmbGetPtr (vp, VMB_P_USER_RESPONSE_BUFFER);
  VUINT8 *request = vmbGetPtr (vp, VMB_P_USER_REQUEST_BUFFER);
  sim_t *sim = vp->userData;
  unsigned addr, count, value;
  bool in = 0;

  switch (request[0]) {
  case F_READ_INPUT_REGISTERS:
    addr = G16 (request + 1);
    value = sim->portNumber * 1000 + slave * 100 + addr * 10 + 9;
    count = G16 (request + 3);
    if (count != 1) RRR ();
    in = 1;
    break;
  case F_WRITE_OUTPUT_REGISTER:
    memcpy (response, request, 5);
    addr = G16 (request + 1);
    value = G16 (request + 3);
    in = 0;
    break;
  default: RRR ();
  }

#define M(_slave, _in, _addr, _value, _delay, _err)	\
  if (slave != _slave) RRR1 (1);			\
  if (in != _in) RRR1 (2);				\
  if (addr != _addr) RRR1 (3);				\
  if (_delay != 0) vsys_wait (_delay * 1000);		\
  if (in) {						\
    value = value / 10 * 10 + _value;			\
  }							\
  else {						\
    if (value != _value) RRR1 (4);			\
  }							\
  if (_err) {						\
    vmbSet (vp, VMB_O_ERROR, "RCVTMO");			\
    return -1;						\
  }							\
  else							\
    break

  if (0)
    if (sim->portNumber == 1)
      tsPrint ("port=%d, unit=%d, addr=%d, in=%d, value=%d",
	       sim->portNumber, slave, addr, in, value);

  switch (sim->portNumber) {
  case 1: {
    static int i = 0;
    /*				schedStart-Actual-Completed-Retry/Timer */
    switch (++i) {
    case 1: M (1, 0, 4, 111, 10, 1);	/* o1.000-000-010-011/o1 */
    case 2: M (2, 0, 4, 121, 5, 1);	/* o2.000-010-015-016/o2 */
    case 3: M (1, 1, 0, 1, 15, 1);	/* s0.000-015-030-031/r1 */
    case 4: M (2, 1, 0, 1, 5, 1);	/* ...000-030-035-036/r2 */
					/* next scan 035/s0 */
    case 5: M (1, 0, 4, 111, 10, 0);	/* o1.011-035-045 */
    case 6: M (1, 0, 6, 112, 0, 0);	/* ...011-045-... */
    case 7: M (2, 0, 4, 121, 10, 0);	/* o2.045-055 */
    case 8: M (2, 0, 6, 122, 0, 0);	/* ...055-... */
    case 9: M (1, 1, 0, 2, 5, 0);	/* r1.055-060 -> s0 */
    case 10: M (1, 1, 2, 2, 0, 0);	/* r1.060-... -> s0 */
    case 11: M (2, 1, 0, 2, 0, 0);
    case 12: M (2, 1, 2, 2, 0, 0);
    case 13: M (1, 1, 0, 9, 5, 0);
    case 14: M (1, 1, 2, 9, 0, 0);	/* 65 */
    case 15: M (2, 1, 0, 9, 0, 0);
    case 16: M (2, 1, 2, 9, 0, 0);
    }
    break;
  }
  case 2: {
    static int i = 0;
    switch (++i) {
					/* when started */
    case 1: M (1, 0, 4, 211, 40, 1);	/* 00 */
    case 2: M (2, 0, 4, 221, 10, 0);	/* 40 */
    case 3: M (2, 0, 6, 222, 0, 0);	/* 50 */
    case 4: M (1, 1, 0, 1, 30, 1);
    case 5: M (2, 1, 0, 1, 5, 0);	/* 80 */
    case 6: M (2, 1, 2, 1, 0, 0);	/* 85 */
    case 7: M (1, 0, 4, 211, 5, 0);
    case 8: M (1, 0, 6, 212, 0, 0);	/* 90 */
    case 9: M (1, 1, 0, 2, 5, 0);
    case 10: M (1, 1, 2, 2, 0, 0);	/* 95 */
    case 11: M (1, 1, 0, 3, 5, 0);
    case 12: M (1, 1, 2, 3, 0, 0);	/* 100 */
    case 13: M (2, 1, 0, 2, 0, 0);
    case 14: M (2, 1, 2, 2, 0, 0);
    case 15: M (1, 1, 0, 9, 5, 0);
    case 16: M (1, 1, 2, 9, 0, 0);	/* 105 */
    case 17: M (2, 1, 0, 9, 0, 0);
    case 18: M (2, 1, 2, 9, 0, 0);
    }
    break;
  }
  case 3: {
    static int i = 0;
    switch (++i) {
    case 1: M (1, 0, 4, 311, 0, 0);	/* 00 */
    case 2: M (1, 0, 6, 312, 0, 0);
    case 3: M (2, 0, 4, 321, 5, 1);
    case 4: M (1, 1, 0, 1, 15, 1);	/* 05 */
    case 5: M (2, 1, 0, 1, 5, 0);	/* 20 */
    case 6: M (2, 1, 2, 1, 0, 0);	/* 25 */
    case 7: M (2, 0, 4, 321, 45, 0);
    case 8: M (2, 0, 6, 322, 0, 0);	/* 70 */
    case 9: M (1, 1, 0, 2, 5, 0);
    case 10: M (1, 1, 2, 2, 0, 0);	/* 75 */
    case 11: M (1, 1, 0, 9, 35, 0);
    case 12: M (1, 1, 2, 9, 0, 0);	/* 110 */
    case 13: M (2, 1, 0, 9, 0, 0);
    case 14: M (2, 1, 2, 9, 0, 0);
    }
    break;
  }
  default: RRR ();
  }

#undef M

  if (in) {
    response[0] = request[0];
    response[1] = 2;
    P16 (response + 2, value);
  }

  return 0;
}

/*
 *********************************************************************
 */

static int test14 (vp_t *vp, int slave, unsigned requestLen,
		   unsigned expectedResponseLen) {

  VUINT8 *response = vmbGetPtr (vp, VMB_P_USER_RESPONSE_BUFFER);
  VUINT8 *request = vmbGetPtr (vp, VMB_P_USER_REQUEST_BUFFER);
  unsigned addr, count, byteCount;
  sim_t *sim = vp->userData;
  unsigned i;

  static struct {
    VINT16 discrete[16];
    VINT16 int16[30];
  } data[16];

  if (slave < 1 || slave > 16) RRR1 (slave);
  slave--;

  switch (request[0]) {
  case F_READ_INPUT_DISCRETES:
    addr = G16 (request + 1);
    count = G16 (request + 3);
    if (addr != 0) RRR ();
    if (count != 256) RRR ();
    byteCount = count / 8;
    response[0] = request[0];
    response[1] = byteCount;
    if (sim->args != NULL) {
      static unsigned callCount = 0;
      VINT16 *ptr = &data[slave].discrete[callCount / 16];
      VINT16 mask = 1 << (callCount & 15);
      if (*ptr & mask)
	*ptr &= ~mask;
      else
	*ptr |= mask;
      callCount++;
      if (callCount >= VSYS_ARRAY_SIZE (data[0].discrete) * 8)
	callCount = 0;
    }
    else {
      for (i = 0; i < VSYS_ARRAY_SIZE (data[0].discrete); i++)
	data[slave].discrete[i] = ~data[slave].discrete[i];
    }
    memcpy (response + 2, &data[slave].discrete, byteCount);
    break;

  case F_READ_INPUT_REGISTERS:
    addr = G16 (request + 1);
    count = G16 (request + 3);
    if (addr != 0) RRR ();
    if (count != 30) RRR1 (count);
    for (i = 0; i < VSYS_ARRAY_SIZE (data[0].int16); i++)
      data[slave].int16[i] += 256;
    byteCount = count * 2;
    response[0] = request[0];
    response[1] = byteCount;
    memcpy (response + 2, &data[slave].int16, byteCount);
    break;

  default: RRR ();
  }

  return 0;
}

/*
 *********************************************************************
 */

static int avtron16 (vp_t *vp, int slave, unsigned requestLen,
		     unsigned expectedResponseLen) {

  VUINT8 *response = vmbGetPtr (vp, VMB_P_USER_RESPONSE_BUFFER);
  VUINT8 *request = vmbGetPtr (vp, VMB_P_USER_REQUEST_BUFFER);
  unsigned addr, count, byteCount;
  sim_t *sim = vp->userData;
  unsigned i;

  if (slave < 1 || slave > sim->maxSlave) RRR1 (slave);
  slave--;

  switch (request[0]) {
  case F_READ_INPUT_DISCRETES:
    addr = G16 (request + 1);
    count = G16 (request + 3);
    if (addr != 0) RRR ();
    if (count != sim->numInputDiscretes) RRR ();
    byteCount = count / 8;
    response[0] = request[0];
    response[1] = byteCount;
    for (i = 0; i < sim->numInputDiscretes / 16; i++)
      sim->slave[slave].discrete[i] = ~sim->slave[slave].discrete[i];
    memcpy (response + 2, sim->slave[slave].discrete, byteCount);
    break;

  case F_READ_INPUT_REGISTERS:
    addr = G16 (request + 1);
    count = G16 (request + 3);
    if ((addr + count) > sim->numInputRegisters) RRR1 (count);
    if (sim->realRegisters) {
      union { float f; VUINT8 c[4]; VUINT32 i; } u1;
      union { float f; VUINT8 c[4]; VUINT32 i; } u2;
      for (i = 0; i < count / 2; i++) {
	u1.i = sim->slave[slave].u.int32[(addr / 2) + i];
	u2.c[0] = u1.c[1];
	u2.c[1] = u1.c[0];
	u2.c[2] = u1.c[3];
	u2.c[3] = u1.c[2];
	u2.f += 1.2F;
	if (i == 0) printf ("v = %d %d %g\n", addr, count, u2.f);
	u1.c[0] = u2.c[1];
	u1.c[1] = u2.c[0];
	u1.c[2] = u2.c[3];
	u1.c[3] = u2.c[2];
	sim->slave[slave].u.int32[(addr / 2) + i] = u1.i;
      }
    }
    else {
      for (i = 0; i < count; i++)
	sim->slave[slave].u.int16[addr + i] += 256;
    }
    byteCount = count * 2;
    response[0] = request[0];
    response[1] = byteCount;
    memcpy (response + 2, sim->slave[slave].u.int16 + addr, byteCount);
    break;

  default: RRR ();
  }

  return 0;
}

/*
 *********************************************************************
 */

int userTransact (vp_t *vp, int slave, unsigned requestLen,
		  unsigned expectedResponseLen) {
  sim_t *sim = vp->userData;
  switch (sim->testNumber) {
  case 13: return test13 (vp, slave, requestLen, expectedResponseLen);
  case 14: return test14 (vp, slave, requestLen, expectedResponseLen);
  case 16: return avtron16 (vp, slave, requestLen, expectedResponseLen);
  default: return RRR1 (sim->testNumber);
  }
}
