/*
 * Hytec scanner.
 * Author: Catherine Laughlin, based on Modbus by Eugene Zharkov
 * Started: 25-Jul-2008
 */

/*
 * DESIGN ISSUES
 *
 * Changes to everything affecting scan lists are processed by the scanner
 * threads, to simplify locking/unlocking of the lists.
 *
 * PLCK routines must be called with port mutex locked.
 *
 * LCK routines must be called with global mutex locked.
 */

//#define NO_HARDWARE_TEST 

#define VERR_FILE_ID "SC"

#include <ctype.h>
#include <math.h>

extern "C" {
#include "scanner_lib.h"
#include <vsys_version.h>
#include <vsys_internal.h>
#include <vlog_routines.h>
#include <vlog_record.h>
#include <vtimer.h>
#ifdef UNIX
#include <errno.h>
#include <fcntl.h>
#include <termios.h>
#include <stropts.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#endif
#include "io.h"
#include "slio.h"
#include "version.h"
#include "vhytec.h"
}

static struct {
  int baudSym;
  int baud;
} baudSymTable[] = {
#ifdef WINNT
  { CBR_110, 110 },
  { CBR_300, 300 },
  { CBR_600, 600 },
  { CBR_1200, 1200 },
  { CBR_2400, 2400 },
  { CBR_4800, 4800 },
  { CBR_9600, 9600 },
  { CBR_14400, 14400 },
  { CBR_19200, 19200 },
  { CBR_38400, 38400 },
  { CBR_56000, 56000 },
  { CBR_57600, 57600 },
  { CBR_115200, 115200 },
  { CBR_128000, 128000 },
  { CBR_256000, 256000 }
#elif defined VMS
  { TT$C_BAUD_50, 50 },
  { TT$C_BAUD_75, 75 },
  { TT$C_BAUD_110, 110 },
  { TT$C_BAUD_134, 134 },
  { TT$C_BAUD_150, 150 },
  { TT$C_BAUD_300, 300 },
  { TT$C_BAUD_600, 600 },
  { TT$C_BAUD_1200, 1200 },
  { TT$C_BAUD_1800, 1800 },
  { TT$C_BAUD_2000, 2000 },
  { TT$C_BAUD_2400, 2400 },
  { TT$C_BAUD_3600, 3600 },
  { TT$C_BAUD_4800, 4800 },
  { TT$C_BAUD_7200, 7200 },
  { TT$C_BAUD_9600, 9600 },
  { TT$C_BAUD_19200, 19200 },
  { TT$C_BAUD_38400, 38400 },
  { TT$C_BAUD_57600, 57600 },
  { TT$C_BAUD_76800, 76800 },
  { TT$C_BAUD_115200, 115200 }
#elif defined UNIX
  { B50, 50 },
  { B75, 75 },
  { B134, 134 },
  { B150, 150 },
  { B200, 200 },
  { B300, 300 },
  { B600, 600 },
  { B1200, 1200 },
  { B1800, 1800 },
  { B2400, 2400 },
  { B4800, 4800 },
  { B9600, 9600 },
  { B19200, 19200 },
  { B38400, 38400 },
#ifdef B57600
  { B57600, 57600 },
#endif
#ifdef B76800
  { B76800, 76800 },
#endif
#ifdef B115200
  { B115200, 115200 },
#endif
#ifdef B153600
  { B153600, 153600 },
#endif
#ifdef B230400
  { B230400, 230400 },
#endif
#ifdef B307200
  { B307200, 307200 },
#endif
#ifdef B460800
  { B460800, 460800 },
#endif
#ifdef B500000
  { B500000, 500000 },
#endif
#ifdef B576000
  { B576000, 576000 },
#endif
#ifdef B921600
  { B921600, 921600 },
#endif
#ifdef B1000000
  { B1000000, 1000000 },
#endif
#ifdef B1152000
  { B1152000, 1152000 },
#endif
#ifdef B1500000
  { B1500000, 1500000 },
#endif
#ifdef B2000000
  { B2000000, 2000000 },
#endif
#ifdef B2500000
  { B2500000, 2500000 },
#endif
#ifdef B3000000
  { B3000000, 3000000 },
#endif
#ifdef B3500000
  { B3500000, 3500000 },
#endif
#ifdef B4000000
  { B4000000, 4000000 },
#endif
  { B110, 110 } /* We need something without a comma at the end, i.e.
		   something that is defined on all platforms. */
#else
  what else ?;
#endif
};


static char programName[] = "Hytec Scanner";
static char commandName[] = "vhytec";
static char versionString[] = VSYS_VERSION_STRING "." VERSION_STRING;

#ifdef VAXFLOAT
error, should not yet be defined;
#endif
#if defined VMS && (defined VAX || defined AXP)
#define VAXFLOAT
#endif

#define vmbCONF_TRACE_SCAN 0
#define vmbCONF_TRACE_TRANSACTION 0

#define vmbCONF_LOG_MAX_TRANSACT_TIME 0

/* I added list puts when I was doing some performace research.
   I don't think that Mike Castle uses list puts.

   fixme0: vlogChannelList lists are currently build based on contiguous
   scanBlocks. Maybe should try to use multiple vlogList lists per
   noncontiguos scanBlock.

   fixme0: channelLists only work if scan blocks start on int16 boundary */

#define vmbCONF_LIST_PUTS 0

#if defined SOLARIS || defined POWERUX
#define vmbARCH_BIG_ENDIAN
#endif

/* Initialize variables to avoid "might be used uninitialized" warnings */

#ifdef LINUX
#define NIL0 = 0
#else
#define NIL0
#endif

/* Newly added internal routines should use this macros. Many old
   routines still return explicit 0/-1. */
#define SUCCESS VSCANNER_SUCCESS
#define FAILURE VSCANNER_FAILURE

/* ... */

#define CLASS_UNDEFINED 4294967295U
#define CLASS_UNDEFINED_STR "4294967295"

#define ENDIAN_UNDEFINED 0
#define ENDIAN_LITTLE 1
#define ENDIAN_BIG 2

#define READ 0
#define WRITE 1

/*
 *********************************************************************
 */

#ifdef WINNT
typedef unsigned __int64 VUINT64;
typedef __int64 VINT64;
#else
typedef unsigned long long VUINT64;
typedef long long VINT64;
#endif

typedef VhytecBinary Binary;

/*
 *********************************************************************
 */


typedef union {
  double float64;
  VUINT64 uint64;
  VINT64 int64;
  float float32;
  VUINT32 uint32;
  VINT32 int32;
  VUINT16 uint16;
  VINT16 int16;
  VUINT8 uint8;
  VINT8 int8;
  VhytecBinary binary;
} Value;

/*
 *********************************************************************
 */

#define VARC_LOCK() do {			\
  if (data.varc.mutexNeeded)			\
    vlogc_global_lock (data.varc.api);		\
} while (0)

#define VARC_UNLOCK() do {			\
  if (data.varc.mutexNeeded)			\
    vlogc_global_unlock (data.varc.api);	\
} while (0)

#define VARC_CHAN(chan) (chan)->flags.varc
#define DBPUT_CHAN(chan) (chan)->flags.dbPut
#define MAX_REFRESH_VARC_TIME 0xffffffffU

/*
 *********************************************************************
 */

struct mapEntry_s {
    struct mapEntry_s* next;
    unsigned count;
    unsigned offset;
}* mapEntry_s;


typedef struct {
  VhytecChannel pub;
  VhytecRoutine conversionRoutine;
  struct mapEntry_s* mapEntry;
  VlogChannel* varcChannel;
  unsigned int refreshRate;
  unsigned int refreshVarcTime;
  /* CML added the next four */
  unsigned int portBufferOffset;
  unsigned int blockListOffset;
  unsigned int clocksync;
  unsigned int SDIaddress;
  unsigned int rate;		/* This value comes from the scanRate field.
				   If scanRate field is 0, then this value
				   is one. This value is either rate in
				   rateUnits, or index of a schedule. Note
				   that if it is index of schedule, then it
				   should be decremented by 1 when accessing
				   the schedule. */
  int varcIOErrorField;
  struct {
    unsigned putInitialValue : 1;	/* Input channels only, put without
					   comparing with the prev value. */
    unsigned ioError : 1;		/* Flag indicating that we set the
					   IO_ERROR field of the channel (or
					   if IO_ERROR was 1 at startup). */
    unsigned dbPut : 1;			/* Channel needs to be updated in
					   the database. */
    unsigned varc : 1;			/* Channel needs to be archived. */
  } flags;
  VUINT16 databaseIndex;		/* Index of the database in the
					   data.databaseName list. */
} Channel;

/*
 *********************************************************************
 */

typedef struct {
  char* name;
  unsigned classNumber;
  unsigned fieldMask;
  unsigned extendedIndex;
  int dtype;
} FunctionField;

/*
 *********************************************************************
 * Offsets of different types of the in a PLC. BE CAREFUL CHANGING THIS
 * STRUCTURE: several routines assume that all members are offsets and
 * ints.
 *********************************************************************
 */

typedef struct {
  struct {
    int binary;	/* Offset of the first binary data point in the PLC.
		   For class0 PLCs this is the offset of the first
		   int16 register that contains that holds the first
		   bit. For class1 PLCs this is the offset of
		   the first bit itself. */
    int float32;
    int float64;
    int int16;
    int int32;
    int string;
    int sdi;
    int holding;
    int coils;
    int holding32;
    int holding64;
  } input, output;
} PlcAddrOffset;

/*
 *********************************************************************
 */

#define OFFSET_TYPE(name, ptr)						     \
	"define type " name,						     \
	"k binary $int range 0:65535 default -1", &ptr.input.binary,	     \
	"k double $int range 0:65535 default -1", &ptr.input.float64,	     \
	"k float $int range 0:65535 default -1", &ptr.input.float32,	     \
	"k int32 $int range 0:65535 default -1", &ptr.input.int32,	     \
	"k int16 $int range 0:65535 default -1", &ptr.input.int16,	     \
	"k string $int range 0:65535 default -1", &ptr.input.string,	     \
	"k sdi $int range 0:65535 default -1", &ptr.input.sdi,	     \
	"k holding $int range 0:65535 default -1", &ptr.input.holding,	     \
	"k holding32 $int range 0:65535 default -1", &ptr.input.holding32,	     \
	"k holding64 $int range 0:65535 default -1", &ptr.input.holding64,	     \
	"k coils $int range 0:65535 default -1", &ptr.input.coils,	     \
	"k outputBinary $int range 0:65535 default -1", &ptr.output.binary,  \
	"k outputDouble $int range 0:65535 default -1", &ptr.output.float64, \
	"k outputFloat $int range 0:65535 default -1", &ptr.output.float32,  \
	"k outputInt32 $int range 0:65535 default -1", &ptr.output.int32,    \
	"k outputInt16 $int range 0:65535 default -1", &ptr.output.int16,    \
	"k outputSdi $int range 0:65535 default -1", &ptr.output.sdi, \
	"k outputHolding $int range 0:65535 default -1", &ptr.output.holding, \
	"k outputHolding32 $int range 0:65535 default -1", &ptr.output.holding32, \
	"k outputHolding64 $int range 0:65535 default -1", &ptr.output.holding64 

/*
 *********************************************************************
 */

#define ENDIAN_TYPE()				\
	"define type endian",			\
	"k LITTLE", (void*) ENDIAN_LITTLE,	\
	"k BIG", (void*) ENDIAN_BIG

/*
 *********************************************************************
 * List of data points of a certain type ordered by address.
 *********************************************************************
 */

typedef struct {

  struct Plc* plc;		/* Pointer to the parent PLC this list
				   belongs to. */
  Channel* chan;		/* List of channels. */
  unsigned dsizeBits;		/* Size of the data point, in bits. */
  unsigned numChannels;		/* Number of channels in the list. */
  unsigned numRegisters;	/* Number of registers that could be requested
				   for this list. */
  unsigned numChannelsAllocated;
  char* descr;			/* Short description of the list. */
  bool isInput;			/* This is a list of input channels. */
  int type;			/* LIST_TYPE */
  int func;			/* read/write function */

  union {
    VUINT8* binary;
    VUINT16* int16;
    VUINT32* int32;
    VUINT64* int64;
    float* float32;
    double* float64;
    char** string;
  } value;

  struct {	/* List of values to output */

    /*
     * <----------------- numChannels ---------------------->
     * changed:  <0>  <1>  <0>  <0>  <1>  <0>  <1>       <0>
     * list:    [   ][   ][   ][   ][   ][   ][   ] ... [   ]
     *        	     ^ |	    ^ |       ^
     *		     | |	    | |       |
     * first:--------	------------   -------|
     * last: ---------------------------------
     *
     * numChannelsChanges: 3
     */

    unsigned numChanged; /* Number of changed channels. */
    void** list;	/* Array of size numChannels to hold the chain of
			   changed channels. */
    void** first;	/* Pointer to the first channel in the chain. */
    void** last;	/* Pointer to the last channel in the chain. */
    bool* changed;	/* Array of size numChannels of flags indicating
			   whether the channel has changed. */
  } o;

} AddrList;

/*
 *********************************************************************
 */

enum {
  LIST_TYPE_INPUT_R1 = 1,
  LIST_TYPE_OUTPUT_R1,
  LIST_TYPE_INPUT_R16,
  LIST_TYPE_OUTPUT_R16,
  LIST_TYPE_INPUT_R32,
  LIST_TYPE_OUTPUT_R32,
  LIST_TYPE_INPUT_R64,
  LIST_TYPE_OUTPUT_R64,
  LIST_TYPE_INPUT_STRING,
  LIST_TYPE_OUTPUT_STRING,
  LIST_TYPE_INPUT_SDI,
  LIST_TYPE_OUTPUT_SDI,
  LIST_TYPE_INPUT_HOLDING,
  LIST_TYPE_OUTPUT_HOLDING,
  LIST_TYPE_INPUT_COILS,
  LIST_TYPE_INPUT_HOLDING_R32,
  LIST_TYPE_OUTPUT_HOLDING_R32,
  LIST_TYPE_INPUT_HOLDING_R64,
  LIST_TYPE_OUTPUT_HOLDING_R64
};

/*
 *********************************************************************
 * macros for applying a function to several lists
 *********************************************************************
 */

#define DO_INPUT_LISTS(function) do {		\
  function (&plc->addrList.in.i1);		\
  function (&plc->addrList.in.o1);		\
  function (&plc->addrList.in.i16);		\
  function (&plc->addrList.in.o16);		\
  function (&plc->addrList.in.i32);		\
  function (&plc->addrList.in.o32);		\
  function (&plc->addrList.in.i64);		\
  function (&plc->addrList.in.o64);		\
  function (&plc->addrList.in.ostring);		\
  function (&plc->addrList.in.string);		\
  function (&plc->addrList.in.osdi);		\
  function (&plc->addrList.in.sdi);		\
  function (&plc->addrList.in.oholding);	\
  function (&plc->addrList.in.iholding);	\
function (&plc->addrList.in.oholding_o32);	\
  function (&plc->addrList.in.oholding_o64);	\
function (&plc->addrList.in.iholding_i32);	\
  function (&plc->addrList.in.iholding_i64);	\
  function (&plc->addrList.in.icoils);	\
} while (0)

#define DO_INPUT_LISTS_REG_FIRST(function) do {	\
  function (&plc->addrList.in.i16);		\
  function (&plc->addrList.in.i32);		\
  function (&plc->addrList.in.i64);		\
  function (&plc->addrList.in.iholding);	\
  function (&plc->addrList.in.iholding_i32);	\
  function (&plc->addrList.in.iholding_i64);	\
  function (&plc->addrList.in.icoils);		\
  function (&plc->addrList.in.o16);		\
  function (&plc->addrList.in.o32);		\
  function (&plc->addrList.in.o64);		\
  function (&plc->addrList.in.oholding);		\
  function (&plc->addrList.in.oholding_o32);		\
  function (&plc->addrList.in.oholding_o64);		\
  function (&plc->addrList.in.i1);		\
  function (&plc->addrList.in.o1);		\
  function (&plc->addrList.in.string);		\
  function (&plc->addrList.in.ostring);		\
  function (&plc->addrList.in.sdi);		\
  function (&plc->addrList.in.osdi);		\
} while (0)

#define DO_OUTPUT_LISTS(function) do {		\
  function (&plc->addrList.out.o1);		\
  function (&plc->addrList.out.o16);		\
  function (&plc->addrList.out.o32);		\
  function (&plc->addrList.out.o64);		\
  function (&plc->addrList.out.ostring);	\
  function (&plc->addrList.out.oholding);	\
  function (&plc->addrList.out.oholding_o32);	\
  function (&plc->addrList.out.oholding_o64);	\
} while (0)

#define DO_ALL_LISTS(function) do {	\
  DO_INPUT_LISTS (function);		\
  DO_OUTPUT_LISTS (function);		\
} while (0)

/*
 *********************************************************************
 */


typedef struct Plc {
  PlcAddrOffset addrOffset;
  struct Port* port;
  struct Plc* next;
  unsigned numInputFailedBlocks;
  unsigned numInputChannels;
  unsigned numOutputChannels;
  unsigned numChannelsChanged;	/* Number of changed output channels. */
  char* name;
  char* portName;
  char *staledata_channel;     /* Name of the channel to use as the staledata
				   channel for this RTU (an integer channel:
				   staledata = (# of scans since last successful 
				   scan. CML */
  char *throughput_channel;     /* Name of the channel to use as the throughput
				   channel for this RTU (a float channel:
				   throughput = (#successful scans / 
				   #attempted scans) * 100 CML */
  vista_chix throughput_chix;   /* Chix for the throughput channel - CML*/
  vista_chix staledata_chix;   /* Chix for the staledata channel - CML*/
  int01 d16;			/* See documentation. */
  vtime outputRetryTime;	/* When to retry after an output failure. */
  int01 enron;			/* This flag affects only 32-bit values.
				   The differences between the standard and
				   enron modes are:
				   - we do not subtract 1 from the address;
				   - the address is treated as the address
				   of the hole thing, not the first of the two
				   16-bit registers;
				   - the number of points in the request is
				   the number of 32-bit points, not the number
				   of 16-bit registers. */
  bool outputFailed;		/* Flag indicating that the last output attempt
				   has failed. */
  bool specifiedOnCmdline;
  int a32;			/* See documentation. */
  int endian;			/* Endianness of the PLC regarding the 32-bit
				   words (which of the 16-bit registers come
				   first). */
  int unit;
  float numScans;                 /* Number of total scans for this PLC - CML*/
  float numFailedScans;           /* Number of failed scans for this PLC - CML */
  int runningFailedScans;       /* Running total of failed scans since the 
				     last successful scan for this PLC - CML */

  struct PlcRetryTimer {
    unsigned index;		/* Index in the data.retry.delay array. */
    Vtimer* timer;
  } inputRetryTimer,		/* Timer for for retrying input channels. */
    outputTimer;		/* Timer for output channels and for retrying
				   output channels. */
  struct {
    unsigned offset;		/* "x" from "-map offset:x". */
    unsigned listSize;		/* Number of -map options. */
    char** list;		/* List of -map options. */
    struct {
      struct map_s {
	unsigned totalCount;
	struct mapEntry_s* entry;
      } binary,
	float32,
	float64,
	int16,
	int32,
	string,
	sdi,
	holding,
	coils,
	holding32,
	holding64;
    } input, output;
  } map;

  struct {
    struct {		/* Input address lists. */
      AddrList i1;	/* Input discretes. */
      AddrList i16;	/* Input registers. */
      AddrList i32;	/* 32-bit words in the input registers space. */
      AddrList i64;	/* 64-bit words in the input registers space. */
      AddrList string;	/* Strings in the input registers space */
      AddrList sdi;	/* SDI registers in the input registers space */
      AddrList iholding; /* Holding registers in the input registers space */
      AddrList iholding_i32; /* Holding 32 registers in the input registers space */
      AddrList iholding_i64; /* Holding 64 registers in the input registers space */
      AddrList icoils; /* Input coils in the input registers space */
      AddrList o1;	/* Output discretes. */
      AddrList o16;	/* Output registers. */
      AddrList o32;	/* 32-bit words in the output registers space. */
      AddrList o64;	/* 64-bit words in the output registers space. */
      AddrList ostring;	/* 64-bit words in the output registers space. */
      AddrList osdi;	/* SDI registers in the output registers space. */
      AddrList oholding; /* Holding registers in the output registers space */
      AddrList oholding_o32; /* Holding 32 registers in the output registers space */
      AddrList oholding_o64; /* Holding 64 registers in the output registers space */
    } in;
    struct {		/* Output address lists. */
      AddrList o1;	/* Output discretes. */
      AddrList o16;	/* Output registers. */
      AddrList o32;	/* 32-bit words in the output registers space. */
      AddrList o64;	/* 64-bit words in the output registers space. */
      AddrList ostring;	/* 64-bit words in the output registers space. */
      AddrList osdi;	/* 64-bit words in the output registers space. */
      AddrList oholding; /* Holding registers in the output registers space. */
      AddrList oholding_o32; /* Holding 32 registers in the output registers space. */
      AddrList oholding_o64; /* Holding 64 registers in the output registers space. */
    } out;
  } addrList;
} Plc;

struct map_s {
  unsigned totalCount;
  struct mapEntry_s {
    struct mapEntry_s* next;
    unsigned count;
    unsigned offset;
  }* entry;
} map_s;


struct PlcRetryTimer {
    unsigned index;		/* Index in the data.retry.delay array. */
    Vtimer* timer;
} PlcRetryTimer;

/*
 *********************************************************************
 * Contiguous block of input PLC channels of a certain type and with
 * the same scan rate.
 *********************************************************************
 */

typedef struct ScanBlock {
  struct ScanBlock* next;
#if vmbCONF_LIST_PUTS
  VlogList* varcList;
#endif
  AddrList* addrList;
  unsigned count;	/* Number of data points to read, including
			   gaps. For binaries counting starts from
			   16-bit boundary. */
  unsigned numChannels;
  unsigned offset;	/* Offset of first channel of the block in the
			   addrList->chan & addList->value arrays. */
  Plc* plc;
  bool conversionRequired; /* There are channels that require conversion. */
  unsigned  cur_retries;
  bool inputFailed;	/* Flag indicating that the last input attempt
			   has failed. */
  bool ioError;		/* There are channels with I/O error flag set. */
  unsigned rate;		/* This is the same as chan->rate. See
				   comment there. */
} ScanBlock;

/*
 *********************************************************************
 * List of scan blocks with the same scan rate.
 *********************************************************************
 */

typedef struct ScanList {
  struct ScanBlock* block;
  struct ScanList* next;
  struct Port* port;
  VSYS_THREAD delay_thread;
  unsigned rate;		/* This is the same as chan->rate. See
				   comment there. */
  /* CML added this to allow for delay before scanning */
  unsigned clocksync;		/* This is the clock sync value for this scan 
				   list, in minutes. See below. */
  Vtimer* timer;
} ScanList;

/* Clock sync:
If this value is n, we wait until the next n-minute boundary to begin scanning 
this list. For example, if the current time is 3:17, scan time for this list 
is 30 minutes, and clocksync is 15 minutes, we'll wait 28 minutes (15 + 30 - 17).

See scanwait() for more details.

*/

/*
 *********************************************************************
 */

typedef struct Port {
  VthreadMutex mutex;
  PlcAddrOffset addrOffset;
  struct Port* next;
  VlogChannel* startScanChannel;	/* See startScanChannelName. */
  VlogChannel* endScanChannel;
  VlogChannel* maxChannel;
  VtimerBank* timerBank;
  ScanList* scanList;
  Vmbport* vp;
  unsigned baud;
  unsigned numFailedPlcs;	/* # of PLCs in plc->outputFailed and/or
				   plc->inputFailed states. */
  unsigned numInputChannels;
  unsigned numOutputChannels;
  unsigned numPlcs;
  unsigned numTypes;
  unsigned requestBufferSize;	/* Size of the port.request buffer. */
  unsigned responseLen;
  unsigned responseBufferSize;	/* Size of the port.response buffer. */
  unsigned rtsDelay;
  unsigned stop;
  unsigned timeout;
  unsigned turnaroundDelay;
  VUINT8* request;
  VUINT8* response;
  int01 d16;			/* See documentation. */
  Plc** plc;
  char* name;
  char* device;
  char* driver;
  char* startScanChannelName;	/* Channel to be logged at the
				   beginning of each scan. */
  char* endScanChannelName;
  char* maxChannelName;		/* Max transact time channel. */
  int01 enron;			/* The default for all PLCs connected to
				   this port. */
  bool newInterval;		/* Flag indicating that new varchive interval
				   has been created and that putInitialValue
				   flag should be set on all input channels
				   associated with the given port. */
  bool scanFlag;		/* Scan triggered via control channel. */
  bool freeFlag;		/* Free scan list for scans via control channel. */
  int PLCs_to_scan[100];	/* List of PLCs to scan via control channel. */
  bool thereIsSomethingToOutput;
  int* typeList;
  int a32;			/* See documentation. */
  int endian;			/* Endianness of the PLC regarding the 32-bit
				   words (which of the 16-bit registers come
				   first). */
  int parity;
  int tcpPort;
  int type;
  int varcScanId;		/* Input scan ID, which gets incremented
				   at the beginning of each new scan. */
  /* Settings for the Hytec cards */
  /* Hy85156 */
  int delay;
  int halfduplex;
  int delay485;
  int databits;
  char flowctl;
  char *ipslot;
  /* Hy8601 */
  int axis;
  int startspeed;
  int maxspeed;
  int ramp;
/* Hy8402 */
  int vectornum;
  int doram;
  int clocksource;
  int clockrate;
  int inhibit;
/* Hy8403 */
  int inputtype;
  int gain;
 /* Hy85056 */
  int vmeslot;     
  int debrate;
  USHORT debmask;
  int scanrate;
  int dir;
  USHORT intmask;
  int pwidth;
  USHORT pmask;
  int clock;
  int portnum;
  /* Hy8512 */
  USHORT scaler;
  USHORT overflow;
  /* Hy8413 */
  int itrstate;
  int aitype;
  int samples;
  int spacing;
  int trigger;
  /* Hy8404 */
  int range;
  /* Pointer to the Hytec structure for this PLC */
  void *hytec_card;

  union {
    VUINT8* u8;
    VUINT16* u16;
    VUINT32* u32;
    VUINT64* u64;
    float *float32;
    /*     char **ustring; */
  } buffer;			/* Buffer for reading from HYTEC, large
				   enough to read any scan block. */
} Port;

/*
 *********************************************************************
 */

  struct varc_s {
    FunctionField* onceField;
    VlogApi* api;
    VlogLog* log;
    unsigned bufferSize;
    unsigned intervalSec;
    unsigned intervalKb;
    unsigned latency;
    unsigned lengthKb;
    unsigned lengthSec;
    unsigned onceListSize;
    char** onceList;		/* List of fields to be logged in once mode. */
    int01 fsync;
    int01 iv;			/* Log initial values at the beginning of
				   each interval. */
    int01 restart;
    int01 shared;
    int01 circular;
    char* name;
    bool mutexNeeded;		/* 1 if multiple ports or if shared
				   or circular archive. */
#if vmbCONF_LIST_PUTS
    bool useChannelLists;	/* If possible, use VlogList lists. */
#endif
  } varc_s;

static struct {
  VthreadMutex globalMutex;	/* Mutex for serializing access to
				   everything. */
  VthreadMutex logMutex;	/* Mutex for serializing access to the
				   log file/status channel. */
  VtimerBank* timerBank;	/* Watchdog and timeSpan timers. */
  VhytecEnv env;
  vista_chix controlChannelChix;
  Vtimer* timeSpanTimer;
  unsigned debug;
  unsigned group_timestamps;    /* Whether or not to set database timestamps
				   to the initial scan time */
  unsigned startOutput;         /* Whether or not to make the initial output
				   (at startup) for output channels */
  unsigned control_design;      /* Whether or not to use custom options for 
				   Control Design PLCs */
  unsigned setup;      /* Whether or not to start the configuration editor */
  unsigned numDatabases;	/* Number of databases. */
  unsigned numPlcFailedPorts;	/* Number of ports with at least on PLCs
				   in plc->inputFailed or plc->outputFailed
				   states. */
  unsigned numPortsWithChannels;
  unsigned scanRateUnitMs;	/* Number of milliseconds per rate unit. */
  unsigned scheduleSize;
  unsigned timeSpan;		/* Life time of the scanner, in seconds. */
  char** databaseName;		/* List of names of databases */
  char** schedule;
  char** userFiles;		/* List of user files. */
  Port* timerLoopPort;		/* Port whose timer bank is run in
				   vtimerLoop. */
  Port* portList;		/* List sorted by name. */
  Port* defaultPort;
  char* controlChannelName;
  char* logChannelName;
  char* userInitRoutine;
  vtime wakeupScannersTime;
  int01 disconnect;
  int01 noline;			/* Do not print __LINE__ number (for
				   autotests). */
  int01 showDecimal;		/* -show decimal - print addresses in decimal
				   form. */
  int01 showPlcs;		/* -show plcs */
  int01 showPorts;		/* -show ports */
  int01 showScanLists;		/* -show scans */
  int01 traceControlChannel;
  int01 traceInitComplete;
  Plc* plcList;		/* List sorted by name. */
  bool exitFlag;		/* Flag set in AST on the control channel. */
  bool timeSpanPresent;		/* If present and 0, exit without doing
				   any I/O. */
  char channelName[512];	/* Buffer for channel name. */
  char fullChannelName[1024];	/* Buffer for dbName::channelName. */
  int disposition;		/* disposition qualifier */
  int endian;			/* Endianness of the computer the scanner
				   is running on. */

  struct varc_s {
    FunctionField* onceField;
    VlogApi* api;
    VlogLog* log;
    unsigned bufferSize;
    unsigned intervalSec;
    unsigned intervalKb;
    unsigned latency;
    unsigned lengthKb;
    unsigned lengthSec;
    unsigned onceListSize;
    char** onceList;		/* List of fields to be logged in once mode. */
    int01 fsync;
    int01 iv;			/* Log initial values at the beginning of
				   each interval. */
    int01 restart;
    int01 shared;
    int01 circular;
    char* name;
    bool mutexNeeded;		/* 1 if multiple ports or if shared
				   or circular archive. */
#if vmbCONF_LIST_PUTS
    bool useChannelLists;	/* If possible, use VlogList lists. */
#endif
  } varc;

  struct {
    FunctionField address;
    FunctionField conversionRoutine;
    FunctionField dataPointType;
    FunctionField appRegBit;
    FunctionField thermocoupleType;
    FunctionField disposition;
    FunctionField refreshRate;
    FunctionField plcName;
    FunctionField scanRate;
    FunctionField clockSync;
    FunctionField SDIaddress;
  } fields;

  struct /* Watchdog channel stuff. */ {
    vista_chix chix;
    unsigned rate;		/* Watchdog rate, in seconds. */
    Vtimer* timer;		/* Watchdog timer. */
    char* channelName;		/* Channel name. */
    bool valid;			/* Chix is valid. */
    int value;			/* Value to be put into watchdog channel. */
    int valueMax;		/* Max value. */
    int valueMin;		/* Min value. */
  } wd;

  struct /* Retry stuff. */ {
    unsigned* delay;
    unsigned size;		        /* Size of the retry array. */
    unsigned num_retries;		/* Total number of retries */
    unsigned cur_retries;		/* Current number of retries */
    unsigned cur_retries_output;	/* Current number of retries */
  } retry;

} data;

/*
 *********************************************************************
 */

#define logError vscannerLogError
#define logFatal vscannerLogFatal
#define logMsg vscannerLogMsg
#define fatalINTERR vscannerFatalInterr

/* fixme: for now all VDBCHIX errors are fatal. Eventually I may need
   to review that and make some of the errors non-fatal. */
#define errorVDBCHIX(c) vscannerFatalVdbChixError (c)

#define errorWDPUT(c) logError ("Error writing to watchdog channel")
#define errorINVCONF(l) logError ("Error in configuration file in line %d", l)

#define fatalMB(p) logFatal ("%s", vmbStrerror (p))
#define fatalCLI() logFatal ("Non specific CLI error %s", VERR__FILE__LINE__)
#define fatalNODB() logFatal ("No database(s) specified")
#define fatalNOSLOT() logFatal ("No slot specified")
#define fatalVARC() fatalVarcError (VERR__FILE__LINE__)
#define fatalNOMEM() logFatal ("Insufficient virtual memory");
#define fatalNOPLC() logFatal ("No PLC specified")
#define fatalNOTIMPL fatalINTERR
#define fatalNOCONF() logFatal ("No configuration file specified")
#define fatalNOTYPE() logFatal ("Communication type is not specified")
#define fatalINVOPT(o) logFatal ("Invalid option\n \\%s\\", o)
#define fatalINVSTOP() logFatal ("Invalid number of stop bits specified")
#define fatalINVTYPE() logFatal ("Invalid communication type specification")
#define fatalNODEVICE() logFatal ("No communication device specified")
#define fatalNOWDCHAN() logFatal ("Watchdog channel not found")
#define fatalCONFOPEN(e) logFatal ("Unable to open configuration file\n \\%s\\", e)
#define fatalCONFREAD(e) logFatal ("Error reading from configuration file\n \\%s\\", e)
#define fatalINVPARITY() logFatal ("Invalid parity specified")
#define fatalINVWDTYPE() logFatal ("Invalid watchdog channel type")
#define fatalNOCTRCHAN() logFatal ("Control channel not found")
#define fatalADDR0(name) logFatal ("Address is not specified\n \\%s\\", name)
#define fatalINVCOPT(l,o) logFatal ("Invalid option in line %d\n \\%s\\", l, o)
#define fatalNOCHANNELS() logFatal ("No channels found")
#define fatalVTIMER(bank) logFatal ("%s: %s", VERR__FILE__LINE__, vtimerStrerror (bank))
#define fatalADDRMAP(name) logFatal ("Address is not found in map\n \\%s\\", name)
#define fatalCONFLICT(a,b) logFatal ("Illegal combination of command elements\n \\%s\\, \\%s\\", a, b)
#define fatalINVADDR(name) logFatal ("Invalid address\n \\%s\\", name)
#define fatalINVUNIT(name) logFatal ("Invalid unit idenfifier\n \\%s\\", name)
#define fatalNOSCHED(name) logFatal ("Scan schedule is not defined\n \\%s\\", name)
#define fatalUSERSTATUS(s) logFatal ("User init routine returned %d", s)
#define fatalUSERVERSION() logFatal ("User file version mismatch")
#define fatalDUPPORT(pname) logFatal ("Duplicate port name\n \\%s\\", pname)
#define fatalINVCHANTYPE(t) logFatal ("Channel type %d not supported", t)
#define fatalINVINOUT(name) logFatal ("Incompatible IN and dataPointType fields\n \\%s\\", name)
#define fatalDUPPLC(plcName) logFatal ("Duplicate PLC name\n \\%s\\", plcName)
#define fatalINVHWTYPE(name) logFatal ("Invalid dataPointType\n \\%s\\", name)
#define fatalNOROUTINE(name) logFatal ("Routine not found\n \\%s\\", name)
#define fatalMAPOPT(plc,opt) logFatal ("Invalid map option\n \\%s\\ (PLC=%s)", opt, plc)
#define fatalINVPLCNAM(pname) logFatal ("Invalid PLC name\n \\%s\\", pname)
#define fatalNOSUCHPORT(name) logFatal ("No such port\n \\%s\\", name)
#define fatalONCESTRING(name) logFatal ("Non-string once field\n \\%s\\", name)
#define fatalDISPOSITION(name) logFatal ("Invalid disposition\n \\%s\\", name)
#define fatalNOSUCHFIELD(name) logFatal ("No such field\n \\%s\\", name)
#define fatalCARDINIT(name) logFatal ("Error: Could not initialize card\n \\%s\\", name)
#define fatalPORTINIT(name) logFatal ("Error: Could not initialize com port\n \\%s\\", name)

#define mberrorPROTOCOL(p) logError ("Protocol error %s", VERR__FILE__LINE__)
#define mberrorTRANSACT(p) logError ("%s", vmbStrerror (p));
#define mberrorWRONGCARD(p) logError ("%s", p);
#define mberrorCONNECT(p) logError ("Failed to initialize: %s", p);
#define mberrorAXIS(p) logError ("Failed to initialize axis for motor card: %s", p);
#define mberrorDISCONNECT(p) logError ("%s", vmbStrerror (p));
/*#define mberrorEXCEPTION(x,q) logError ("Exception %d at %s (%02X %02X %02X %02X %02X)", x, VERR__FILE__LINE__, q[0], q[1], q[2], q[3], q[4]); */

/* CML changed this to give better error output. Also, macros == bad. */ 
int mberrorEXCEPTION(int x, int linenum, VUINT8 *q) {
  int err = 0;
  if (q == NULL) {
    return 0;
  }
  switch(x) {
  case 1:
    err = logError ("Exception %d (Illegal Function) (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    vsys_msg_printf (0, "Exception %d (Illegal Function) (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  case 2:
    err = logError ("Exception %d (Illegal Data Address)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    vsys_msg_printf (0, "Exception %d (Illegal Data Address)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  case 3:
    err = logError ("Exception %d (Illegal Data Value)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
     vsys_msg_printf (0,"Exception %d (Illegal Data Value)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  case 4:
    err = logError ("Exception %d (Slave Device Failure)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    vsys_msg_printf (0,"Exception %d (Slave Device Failure)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  case 5:
    err = logError ("Exception %d (Acknowledge)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    vsys_msg_printf (0,"Exception %d (Acknowledge)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  case 6:
    err = logError ("Exception %d (Slave Device Busy)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    vsys_msg_printf (0,"Exception %d (Slave Device Busy)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  case 7:
    err = logError ("Exception %d (Negative Acknowledge)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    vsys_msg_printf (0,"Exception %d (Negative Acknowledge)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  case 8:
    err = logError ("Exception %d (Memory Parity Error)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    vsys_msg_printf (0,"Exception %d (Memory Parity Error)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  case 9:
    err = logError ("Exception %d (Undefined Error)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    vsys_msg_printf (0,"Exception %d (Undefined Error)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  case 10:
    err = logError ("Exception %d (Gateway Path Unavailable)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    vsys_msg_printf (0,"Exception %d (Gateway Path Unavailable)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  case 11:
    err = logError ("Exception %d (Gateway Target Device Failed to Respond)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    vsys_msg_printf (0,"Exception %d (Gateway Target Device Failed to Respond)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  default:
    err = logError ("Exception %d (Unknown Error)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    vsys_msg_printf (0,"Exception %d (Unknown Error)  (%02X %02X %02X %02X %02X)", x,  q[0], q[1], q[2], q[3], q[4]);
    break;
  }
    return err;
}

static int mymbI16put (Plc*, unsigned, unsigned);
static void reallocValueBuffer (AddrList*);
extern void set_control_design(int);
extern void set_show_decimal(int);
/*
 *********************************************************************
 */



#ifdef DEBUG
#define fatalMalloc(a) fatalMalloc_LINE (a, __LINE__)
static void* fatalMalloc_LINE (unsigned size, int line) {
  void* ptr = vmemMallocDbg (NULL, size, __FILE__, line);
  if (ptr == NULL) fatalNOMEM ();
  return ptr;
}
#else
static void* fatalMalloc (unsigned size) {
  void* ptr = VMEM_MALLOC (size);
  if (ptr == NULL) fatalNOMEM ();
  return ptr;
}
#endif

/*
 *********************************************************************
 */

#ifdef DEBUG
#define fatalMalloc0(a) fatalMalloc0_LINE (a, __LINE__)
static void* fatalMalloc0_LINE (unsigned size, int line) {
  void* ptr = fatalMalloc_LINE (size, line);
  (void) memset (ptr, 0, size);
  return ptr;
}
#else
static void* fatalMalloc0 (unsigned size) {
  void* ptr = fatalMalloc (size);
  (void) memset (ptr, 0, size);
  return ptr;
}
#endif

/*
 *********************************************************************
 */

#ifdef DEBUG
#define fatalRealloc(a,b) fatalRealloc_LINE (a,b,__LINE__)
static void* fatalRealloc_LINE (void* ptr, unsigned size, int line) {
  ptr = vmemReallocDbg (NULL, ptr, size, __FILE__, line);
  if (ptr == NULL) fatalNOMEM ();
  return ptr;
}
#else
static void* fatalRealloc (void* ptr, unsigned size) {
  ptr = VMEM_REALLOC (ptr, size);
  if (ptr == NULL) fatalNOMEM ();
  return ptr;
}
#endif

/*
 *********************************************************************
 */

void hytec_error(int hy_err) {
  if (hy_err != 0 && hy_err != -1) {
    printf("Hytec error code %d: ", hy_err);
    switch(hy_err) {
    case -1:
      break;
    case -9010001:	printf("Carrier board fan number invalid"); break;
    case -9010002:	printf("Carrier board temperature sensor number invalid");	 break;		
    case -9010003:	printf("Carrier board push button number invalid");   break; 
    case -9010004:	printf("IP slot invalid	"); break;
  case -9010005:	printf("Carrier register address invalid"); break;	
    case -9010006:	printf("IP card not present");	 break;	     
    case -9010007:	printf("Channel number does not exist"); break;	      
    case -9010008:	printf("Device driver error"); break;
    case -8402001:	printf("IP card in the specified slot is not 8402"); break;
    case -8402010:	printf("VME setting invalid"); break;
    case -8402012:	printf("Clock rate setting invalid"); break;
    case -8402013:	printf("Channel number invalid"); break;
    case -8402014:	printf("8402 not configured OR waveform memory setting does not match 32k"); break;
    case -8403001:	printf("IP card is not 8403"); break;
    case -8403010:	printf("VME slot setting invalid"); break;
    case -8403003:	printf("Type setting invalid"); break;
    case -8403004:	printf("Gain setting invalid"); break;
    case -8403005:	printf("Auto-calibration setting invalid"); break;
    case -8403002:	printf("8403 channel number invalid"); break;
    case -8403014:	printf("8403 Not configured"); break;
    case -8411001:	printf("IP card is not 8411"); break;
    case -8411010:	printf("VME slot setting invalid"); break;
    case -8411012:	printf("Clock rate setting invalid"); break;
    case -8411013:	printf("8411 channel number invalid"); break;
    case -8411014:	printf("8411 Not configured"); break;
    case -8413001:	printf("IP card is not 8413"); break;
    case -8413010:	printf("VME slot setting invalid"); break;
    case -8413012:	printf("Clock rate setting invalid"); break;
    case -8413013:	printf("8413 channel number invalid"); break;
    case -8413014:	printf("8413 Not configured"); break;
    case -8005010:	printf("IP is not 8005 card"); break;
    case -8005012:	printf("8005 not configured"); break;
    case -8005013:	printf("Debounce rate setting invalid"); break;
    case -8005014:	printf("Scan rate setting invalid"); break;
    case -8005015:	printf("IO direction setting invalid"); break;
    case -8005016:	printf("Pause width setting invalid"); break;
    case -8005017:	printf("Clock setting invalid"); break;
    case -8005018:	printf("Port number invalid"); break;
    case -8005019:	printf("Write violation – attempt to write to i/o port configured as inputs"); break;
    case -8005020:	printf("Bit number invalid"); break;
    case -8505010:	printf("IP is not 8505 card"); break;
    case -8506010:	printf("IP is not 8506 card"); break;
    case -85056012:	printf("8505 or 8506 IP not configured"); break;
    case -85056013:	printf("Debounce rate setting invalid"); break;
    case -85056014:	printf("Scan rate setting invalid"); break;
    case -85056015:	printf("IO direction setting invalid"); break;
    case -85056016:	printf("Pause width setting invalid"); break;
    case -85056017:	printf("Clock setting invalid"); break;
    case -85056018:	printf("Port number invalid (for 8506 only)"); break;
    case -85056019:	printf("Write violation. Try to write to an input port."); break;
    case -85056020:	printf("Bit number invalid"); break;
    case -8512010:	printf("Not 8512 IP card"); break;
    case -8512012:	printf("Invalid register number"); break;
    case -8512014:	printf("8512 IP not configured"); break;
    case -85156011:	printf("85156 IP not configured"); break;
    case -85156012:	printf("Port number invalid"); break;
    case -85156013:	printf("Baud rate setting invalid"); break;
    case -85156014:	printf("Parity setting invalid"); break;
    case -85156015:	printf("Data bit setting invalid"); break;
    case -85156016:	printf("Stop bit setting invalid"); break;
    case -85156017:	printf("Flow control setting invalid"); break;
    case -8601010:	printf("IP card is not 8601"); break;
    case -8601011:	printf("8601 IP not configured"); break;
    case -8601012:	printf("axis number invalid"); break;
    case -8601013:	printf("start/stop speed setting invalid"); break;
    case -8601014:	printf("maximum speed setting invalid"); break;
    case -8601015:	printf("ramp rate setting invalid"); break;
    case -8404001:      printf("IP card is not 8404"); break;
    case -8404012:      printf("Clock setting invalid"); break;
    case -8404013:      printf("8404 channel number invalid"); break;
    case -8404014:      printf("8404 IP not configured"); break;
    case -8408001:      printf("IP card is not 8408"); break;
    case -8408012:      printf("Clock setting invalid"); break;
    case -8408013:      printf("8408 channel number invalid"); break;
    case -8408014:      printf("8408 IP not configured"); break;

    default: printf("Unknown hytec error code: %d", hy_err); break;
    }
    printf("\n");
  } else if (hy_err == 0){
    if (data.debug)
      printf("Success!\n");
  }
}

void setup_dac(CHy8403 *DAC03) {

/* DAC03.PGAMuxSettings[16][6] ={ */
  /* ADC A - Channel 0 */
  DAC03->PGAMuxSettings[0][0] = 0x01;
  DAC03->PGAMuxSettings[0][1] = 0x08;
  DAC03->PGAMuxSettings[0][2] = 0x01;
  DAC03->PGAMuxSettings[0][3] = 0x08;
  DAC03->PGAMuxSettings[0][4] = 0x01;
  DAC03->PGAMuxSettings[0][5] = 0x01;
  /* ADC B - Channel 0 */
  DAC03->PGAMuxSettings[1][0] = 0x01;
  DAC03->PGAMuxSettings[1][1] = 0x08;
  DAC03->PGAMuxSettings[1][2] = 0x01;
  DAC03->PGAMuxSettings[1][3] = 0x08;
  DAC03->PGAMuxSettings[1][4] = 0x01;
  DAC03->PGAMuxSettings[1][5] = 0x01;
  /* ADC A - Channel 1 */
  DAC03->PGAMuxSettings[2][0] = 0x23;
  DAC03->PGAMuxSettings[2][1] = 0x18;
  DAC03->PGAMuxSettings[2][2] = 0x12;
  DAC03->PGAMuxSettings[2][3] = 0x18;
  DAC03->PGAMuxSettings[2][4] = 0x23;
  DAC03->PGAMuxSettings[2][5] = 0x23;
  /* ADC B - Channel 1 */
  DAC03->PGAMuxSettings[3][0] = 0x23;
  DAC03->PGAMuxSettings[3][1] = 0x18;
  DAC03->PGAMuxSettings[3][2] = 0x12;
  DAC03->PGAMuxSettings[3][3] = 0x18;
  DAC03->PGAMuxSettings[3][4] = 0x23;
  DAC03->PGAMuxSettings[3][5] = 0x23;
  /* ADC A - Channel 2 */
  DAC03->PGAMuxSettings[4][0] = 0x45;
  DAC03->PGAMuxSettings[4][1] = 0x28;
  DAC03->PGAMuxSettings[4][2] = 0x23;
  DAC03->PGAMuxSettings[4][3] = 0x28;
  DAC03->PGAMuxSettings[4][4] = 0x45;
  DAC03->PGAMuxSettings[4][5] = 0x45;
  /* ADC B - Channel 2 */
  DAC03->PGAMuxSettings[5][0] = 0x45;
  DAC03->PGAMuxSettings[5][1] = 0x28;
  DAC03->PGAMuxSettings[5][2] = 0x23;
  DAC03->PGAMuxSettings[5][3] = 0x28;
  DAC03->PGAMuxSettings[5][4] = 0x45;
  DAC03->PGAMuxSettings[5][5] = 0x45;
  /* ADC A - Channel 3 */
  DAC03->PGAMuxSettings[6][0] = 0x67;
  DAC03->PGAMuxSettings[6][1] = 0x38;
  DAC03->PGAMuxSettings[6][2] = 0x34;
  DAC03->PGAMuxSettings[6][3] = 0x38;
  DAC03->PGAMuxSettings[6][4] = 0x67;
  DAC03->PGAMuxSettings[6][5] = 0x67;
/* ADC B - Channel 3 */
  DAC03->PGAMuxSettings[7][0] = 0x67;
  DAC03->PGAMuxSettings[7][1] = 0x38;
  DAC03->PGAMuxSettings[7][2] = 0x34;
  DAC03->PGAMuxSettings[7][3] = 0x38;
  DAC03->PGAMuxSettings[7][4] = 0x67;
  DAC03->PGAMuxSettings[7][5] = 0x67;
/* ADC A - Channel 4 */
  DAC03->PGAMuxSettings[8][0] = 0x00;
  DAC03->PGAMuxSettings[8][1] = 0x48;
  DAC03->PGAMuxSettings[8][2] = 0x45;
  DAC03->PGAMuxSettings[8][3] = 0x48;
  DAC03->PGAMuxSettings[8][4] = 0x01;
  DAC03->PGAMuxSettings[8][5] = 0x01;
/* ADC B - Channel 4 */
  DAC03->PGAMuxSettings[9][0] = 0x00;
  DAC03->PGAMuxSettings[9][1] = 0x48;
  DAC03->PGAMuxSettings[9][2] = 0x45;
  DAC03->PGAMuxSettings[9][3] = 0x48;
  DAC03->PGAMuxSettings[9][4] = 0x01;
  DAC03->PGAMuxSettings[9][5] = 0x01;
/* ADC A - Channel 5 */
  DAC03->PGAMuxSettings[10][0] = 0x00;
  DAC03->PGAMuxSettings[10][1] = 0x58;
  DAC03->PGAMuxSettings[10][2] = 0x56;
  DAC03->PGAMuxSettings[10][3] = 0x58;
  DAC03->PGAMuxSettings[10][4] = 0x23;
  DAC03->PGAMuxSettings[10][5] = 0x23;
/* ADC B - Channel 5 */
  DAC03->PGAMuxSettings[11][0] = 0x00;
  DAC03->PGAMuxSettings[11][1] = 0x58;
  DAC03->PGAMuxSettings[11][2] = 0x56;
  DAC03->PGAMuxSettings[11][3] = 0x58;
  DAC03->PGAMuxSettings[11][4] = 0x23;
  DAC03->PGAMuxSettings[11][5] = 0x23;
/* ADC A - Channel 6*/
  DAC03->PGAMuxSettings[12][0] = 0x00;
  DAC03->PGAMuxSettings[12][1] = 0x68;
  DAC03->PGAMuxSettings[12][2] = 0x67;
  DAC03->PGAMuxSettings[12][3] = 0x68;
  DAC03->PGAMuxSettings[12][4] = 0x45;
  DAC03->PGAMuxSettings[12][5] = 0x45;
/* ADC B - Channel 6*/
  DAC03->PGAMuxSettings[13][0] = 0x00;
  DAC03->PGAMuxSettings[13][1] = 0x68;
  DAC03->PGAMuxSettings[13][2] = 0x67;
  DAC03->PGAMuxSettings[13][3] = 0x68;
  DAC03->PGAMuxSettings[13][4] = 0x45;
  DAC03->PGAMuxSettings[13][5] = 0x45;
/* ADC A - Channel 7*/
  DAC03->PGAMuxSettings[14][0] = 0x00;
  DAC03->PGAMuxSettings[14][1] = 0x78;
  DAC03->PGAMuxSettings[14][2] = 0x78;
  DAC03->PGAMuxSettings[14][3] = 0x78;
  DAC03->PGAMuxSettings[14][4] = 0x67;
  DAC03->PGAMuxSettings[14][5] = 0x67;
/* ADC A - Channel 7*/
  DAC03->PGAMuxSettings[15][0] = 0x00;
  DAC03->PGAMuxSettings[15][1] = 0x78;
  DAC03->PGAMuxSettings[15][2] = 0x78;
  DAC03->PGAMuxSettings[15][3] = 0x78;
  DAC03->PGAMuxSettings[15][4] = 0x67;
  DAC03->PGAMuxSettings[15][5] = 0x67;

#if 0
/* Dif   S/E  PT100 TC    PTB   TCB */
{ 0x01, 0x08, 0x01, 0x08, 0x01, 0x01 },
{ 0x01, 0x08, 0x01, 0x08, 0x01, 0x01 },	/* ADC B - Channel 0 */
{ 0x23, 0x18, 0x12, 0x18, 0x23, 0x23 },	/* ADC A - Channel 1 */
{ 0x23, 0x18, 0x12, 0x18, 0x23, 0x23 },	/* ADC B - Channel 1 */
{ 0x45, 0x28, 0x23, 0x28, 0x45, 0x45 },	/* ADC A - Channel 2 */
{ 0x45, 0x28, 0x23, 0x28, 0x45, 0x45 },	/* ADC B - Channel 2 */
{ 0x67, 0x38, 0x34, 0x38, 0x67, 0x67 },	/* ADC A - Channel 3 */
{ 0x67, 0x38, 0x34, 0x38, 0x67, 0x67 },	/* ADC B - Channel 3 */
{ 0x00, 0x48, 0x45, 0x48, 0x01, 0x01 },	/* ADC A - Channel 4 */
{ 0x00, 0x48, 0x45, 0x48, 0x01, 0x01 },	/* ADC B - Channel 4 */
{ 0x00, 0x58, 0x56, 0x58, 0x23, 0x23 },	/* ADC A - Channel 5 */
{ 0x00, 0x58, 0x56, 0x58, 0x23, 0x23 },	/* ADC B - Channel 5 */
{ 0x00, 0x68, 0x67, 0x68, 0x45, 0x45 },	/* ADC A - Channel 6 */
{ 0x00, 0x68, 0x67, 0x68, 0x45, 0x45 },	/* ADC B - Channel 6 */
{ 0x00, 0x78, 0x78, 0x78, 0x67, 0x67 },	/* ADC A - Channel 7 */
{ 0x00, 0x78, 0x78, 0x78, 0x67, 0x67}};	/* ADC B - Channel 7 */
#endif

/* ADC A - Channel 0 */
DAC03->ADCChannelMapping[0][0] = 0x40;
DAC03->ADCChannelMapping[0][1] = 0x40;
DAC03->ADCChannelMapping[0][2] = 0x40;
DAC03->ADCChannelMapping[0][3] = 0x40;
DAC03->ADCChannelMapping[0][4] = 0x40;
DAC03->ADCChannelMapping[0][5] = 0x40;
/* ADC A - Channel 1 */
  DAC03->ADCChannelMapping[1][0] = 0x48;
  DAC03->ADCChannelMapping[1][1] = 0x48;
  DAC03->ADCChannelMapping[1][2] = 0x48;
  DAC03->ADCChannelMapping[1][3] = 0x48;
  DAC03->ADCChannelMapping[1][4] = 0x44;
  DAC03->ADCChannelMapping[1][5] = 0x44;
/* ADC A - Channel 2 */
  DAC03->ADCChannelMapping[2][0] = 0x50;
  DAC03->ADCChannelMapping[2][1] = 0x50;
  DAC03->ADCChannelMapping[2][2] = 0x50;
  DAC03->ADCChannelMapping[2][3] = 0x50;
  DAC03->ADCChannelMapping[2][4] = 0x48;
  DAC03->ADCChannelMapping[2][5] = 0x48;
/* ADC A - Channel 3 */
  DAC03->ADCChannelMapping[3][0] = 0x58;
  DAC03->ADCChannelMapping[3][1] = 0x58;
  DAC03->ADCChannelMapping[3][2] = 0x58;
  DAC03->ADCChannelMapping[3][3] = 0x58;
  DAC03->ADCChannelMapping[3][4] = 0x4C;
  DAC03->ADCChannelMapping[3][5] = 0x4C;
/* ADC A - Channel 4 */
  DAC03->ADCChannelMapping[4][0] = 0x60;
  DAC03->ADCChannelMapping[4][1] = 0x60;
  DAC03->ADCChannelMapping[4][2] = 0x60;
  DAC03->ADCChannelMapping[4][3] = 0x60;
  DAC03->ADCChannelMapping[4][4] = 0x50;
  DAC03->ADCChannelMapping[4][5] = 0x50;
 /* ADC A - Channel 5 */
  DAC03->ADCChannelMapping[5][0] = 0x68;
  DAC03->ADCChannelMapping[5][1] = 0x68;
  DAC03->ADCChannelMapping[5][2] = 0x68;
  DAC03->ADCChannelMapping[5][3] = 0x68;
  DAC03->ADCChannelMapping[5][4] = 0x54;
  DAC03->ADCChannelMapping[5][5] = 0x54;
 /* ADC A - Channel 6 */
  DAC03->ADCChannelMapping[6][0] = 0x70;
  DAC03->ADCChannelMapping[6][1] = 0x70;
  DAC03->ADCChannelMapping[6][2] = 0x70;
  DAC03->ADCChannelMapping[6][3] = 0x70;
  DAC03->ADCChannelMapping[6][4] = 0x58;
  DAC03->ADCChannelMapping[6][5] = 0x58;
/* ADC A - Channel 7 */
  DAC03->ADCChannelMapping[7][0] = 0x78;
  DAC03->ADCChannelMapping[7][1] = 0x78;
  DAC03->ADCChannelMapping[7][2] = 0x78;
  DAC03->ADCChannelMapping[7][3] = 0x78;
  DAC03->ADCChannelMapping[7][4] = 0x58;
DAC03->ADCChannelMapping[7][5] = 0x58;
 /* ADC B - Channel 0 */
DAC03->ADCChannelMapping[8][0] = 0x44;
  DAC03->ADCChannelMapping[8][1] = 0x44;
  DAC03->ADCChannelMapping[8][2] = 0x44;
  DAC03->ADCChannelMapping[8][3] = 0x44;
  DAC03->ADCChannelMapping[8][4] = 0x5C;
  DAC03->ADCChannelMapping[8][5] = 0x5C;
 /* ADC B - Channel 1 */
DAC03->ADCChannelMapping[9][0] = 0x4C;
  DAC03->ADCChannelMapping[9][1] = 0x4C;
  DAC03->ADCChannelMapping[9][2] = 0x4C;
  DAC03->ADCChannelMapping[9][3] = 0x4C;
  DAC03->ADCChannelMapping[9][4] = 0x40;
  DAC03->ADCChannelMapping[9][5] = 0x40;
/* ADC B - Channel 2 */
DAC03->ADCChannelMapping[10][0] = 0x54;
  DAC03->ADCChannelMapping[10][1] = 0x54;
  DAC03->ADCChannelMapping[10][2] = 0x54;
  DAC03->ADCChannelMapping[10][3] = 0x54;
  DAC03->ADCChannelMapping[10][4] = 0x44;
  DAC03->ADCChannelMapping[10][5] = 0x44;
/* ADC B - Channel 3 */
DAC03->ADCChannelMapping[11][0] = 0x5C;
  DAC03->ADCChannelMapping[11][1] = 0x5C;
  DAC03->ADCChannelMapping[11][2] = 0x5C;
  DAC03->ADCChannelMapping[11][3] = 0x5C;
  DAC03->ADCChannelMapping[11][4] = 0x48;
  DAC03->ADCChannelMapping[11][5] = 0x48;
/* ADC B - Channel 4 */
DAC03->ADCChannelMapping[12][0] = 0x64;
  DAC03->ADCChannelMapping[12][1] = 0x64;
  DAC03->ADCChannelMapping[12][2] = 0x64;
  DAC03->ADCChannelMapping[12][3] = 0x64;
  DAC03->ADCChannelMapping[12][4] = 0x4C;
  DAC03->ADCChannelMapping[12][5] = 0x4C;
/* ADC B - Channel 5 */
DAC03->ADCChannelMapping[13][0] = 0x6C;
  DAC03->ADCChannelMapping[13][1] = 0x6C;
  DAC03->ADCChannelMapping[13][2] = 0x6C;
  DAC03->ADCChannelMapping[13][3] = 0x6C;
  DAC03->ADCChannelMapping[13][4] = 0x50;
  DAC03->ADCChannelMapping[13][5] = 0x50;
/* ADC B - Channel 6 */
DAC03->ADCChannelMapping[14][0] = 0x74;
  DAC03->ADCChannelMapping[14][1] = 0x74;
  DAC03->ADCChannelMapping[14][2] = 0x74;
  DAC03->ADCChannelMapping[14][3] = 0x74;
  DAC03->ADCChannelMapping[14][4] = 0x54;
  DAC03->ADCChannelMapping[14][5] = 0x54;
/* ADC B - Channel 7 */
DAC03->ADCChannelMapping[15][0] = 0x7C;
  DAC03->ADCChannelMapping[15][1] = 0x7C;
  DAC03->ADCChannelMapping[15][2] = 0x7C;
  DAC03->ADCChannelMapping[15][3] = 0x7C;
  DAC03->ADCChannelMapping[15][4] = 0x5C;
DAC03->ADCChannelMapping[15][5] = 0x5C;

#if 0

 DAC03.ADCChannelMapping[16][6];={
/* Dif   S/E  PT100 TC    PTB   TCB */
{ 0x40, 0x40, 0x40, 0x40, 0x40, 0x40 },	/* ADC A - Channel 0 */
{ 0x48, 0x48, 0x48, 0x48, 0x44, 0x44 },	/* ADC A - Channel 1 */
{ 0x50, 0x50, 0x50, 0x50, 0x48, 0x48 },	/* ADC A - Channel 2 */
{ 0x58, 0x58, 0x58, 0x58, 0x4C, 0x4C },	/* ADC A - Channel 3 */
{ 0x60, 0x60, 0x60, 0x60, 0x50, 0x50 },	/* ADC A - Channel 4 */
{ 0x68, 0x68, 0x68, 0x68, 0x54, 0x54 },	/* ADC A - Channel 5 */
{ 0x70, 0x70, 0x70, 0x70, 0x58, 0x58 },	/* ADC A - Channel 6 */
{ 0x78, 0x78, 0x78, 0x78, 0x58, 0x58 },	/* ADC A - Channel 7 */
{ 0x44, 0x44, 0x44, 0x44, 0x5C, 0x5C },	/* ADC B - Channel 0 */
{ 0x4C, 0x4C, 0x4C, 0x4C, 0x40, 0x40 },	/* ADC B - Channel 1 */
{ 0x54, 0x54, 0x54, 0x54, 0x44, 0x44 },	/* ADC B - Channel 2 */
{ 0x5C, 0x5C, 0x5C, 0x5C, 0x48, 0x48 },	/* ADC B - Channel 3 */

{ 0x64, 0x64, 0x64, 0x64, 0x4C, 0x4C },	/* ADC B - Channel 4 */
{ 0x6C, 0x6C, 0x6C, 0x6C, 0x50, 0x50 },	/* ADC B - Channel 5 */
{ 0x74, 0x74, 0x74, 0x74, 0x54, 0x54 },	/* ADC B - Channel 6 */
{ 0x7C, 0x7C, 0x7C, 0x7C, 0x5C, 0x5C}}; /* ADC B - Channel 7 */
#endif
}

void wrongcard(int curcard, char *type) {
  char buff[1028];
  char buff2[1028];
  char buff3[1028];
  int i = 0;
  sprintf(buff, "Wrong card");
    switch(curcard) {
  case(VMB_TYPE_85156):
    sprintf(buff2, "%s 85156\n", buff);
  case(VMB_TYPE_8505): 
    sprintf(buff2, "%s 8505\n", buff);
  case(VMB_TYPE_8506): 
    sprintf(buff2, "%s 8506\n", buff);
  case(VMB_TYPE_8601):
      sprintf(buff2, "%s 8601\n", buff);
  case (VMB_TYPE_8402):
    sprintf(buff2, "%s 8402\n", buff);
  case (VMB_TYPE_8403):
    sprintf(buff2, "%s 8403\n", buff);
    case(VMB_TYPE_8413): 
      sprintf(buff2, "%s 8413\n", buff); 
case(VMB_TYPE_8404): 
      sprintf(buff2, "%s 8404\n", buff); 
  case(VMB_TYPE_8005):
    sprintf(buff2, "%s 8005\n", buff);
  case(VMB_TYPE_8411):
    sprintf(buff2, "%s 8411\n", buff);
  case(VMB_TYPE_8512):
    sprintf(buff2, "%s 8412\n", buff);
    default:
      sprintf(buff2, "UNKNOWN CARD TYPE\n");
  }
    sprintf(buff3, "%s %s for %s data type", buff, buff2, type);
  /*  switch(curcard) {
  case(VMB_TYPE_85156):
    sprintf(buff4, "%s 85156\n", buff3);
  case(VMB_TYPE_85056): 
    sprintf(buff4, "%s 85056\n", buff3);
  case(VMB_TYPE_8601):
      sprintf(buff4, "%s 8601\n", buff3);
  case (VMB_TYPE_8402):
    sprintf(buff4, "%s 8402\n", buff3);
  case (VMB_TYPE_8403):
    sprintf(buff4, "%s 8403\n", buff3);
  case(VMB_TYPE_8413): 
    sprintf(buff4, "%s 8413\n", buff3); 
  case(VMB_TYPE_8005):
    sprintf(buff4, "%s 8005\n", buff3);
  case(VMB_TYPE_8411):
    sprintf(buff4, "%s 8411\n", buff3);
  case(VMB_TYPE_8512):
    sprintf(buff4, "%s 8412\n", buff3);
    }*/
  mberrorWRONGCARD(buff3);
}

static ScanBlock * copy_scanblock(ScanBlock *sb) {
  ScanBlock *newsb;
  newsb = (ScanBlock *)fatalMalloc0 (sizeof (ScanBlock));
  newsb->next = NULL;
 #if vmbCONF_LIST_PUTS
  newsb->varcList = sb->varcList;
#endif 
  newsb->addrList = sb->addrList;
  newsb->count = sb->count;
  newsb->numChannels = sb->numChannels;
  newsb->offset = sb->offset;
  newsb->plc = sb->plc;
  newsb->conversionRequired = sb->conversionRequired;
  newsb->cur_retries = sb->cur_retries;
  newsb->inputFailed = sb->inputFailed;
  newsb->ioError = sb->ioError;
  newsb->rate = sb->rate;
  return(newsb);
}

#ifdef DEBUG
#define fatalStrdup(a) fatalStrdup_LINE (a,__LINE__)
static char* fatalStrdup_LINE (char* string, int line) {
  if (string == NULL)
    return NULL;
  else {
    unsigned l = strlen (string) + 1;
    char* ptr = (char *)fatalMalloc_LINE (l, line);
    (void) memcpy (ptr, string, l);
    return ptr;
  }
}
#else
static char* fatalStrdup (char* string) {
  if (string == NULL)
    return NULL;
  else {
    unsigned l = strlen (string) + 1;
    char* ptr = (char *)fatalMalloc (l);
    (void) memcpy (ptr, string, l);
    return ptr;
  }
}
#endif

/*
 *********************************************************************
 */

#ifdef DEBUG
#define myFree(a) myFree_LINE (a,__LINE__)
static void myFree_LINE (void* ptr, int line) {
  if (ptr != NULL) vmemFreeDbg (NULL, ptr, __FILE__, line);
}
#else
static void myFree (void* ptr) {
  if (ptr != NULL) VMEM_FREE (ptr);
}
#endif

/*
 *********************************************************************
 */

static void initMutexes (void) {
  Port* port;
  if (vthreadMutexInit (&data.globalMutex, NULL) != VTHREAD_OK)
    fatalINTERR ();	/*NCV*/
  for (port = data.portList; port != NULL; port = port->next) {
    if (port->numInputChannels != 0 || port->numOutputChannels != 0) {
      if (vthreadMutexInit (&port->mutex, NULL) != VTHREAD_OK)
	fatalINTERR ();	/*NCV*/
    }
  }
}

/*
 *********************************************************************
 */

static void lockGlobal (void) {
  if (vthreadMutexLock (&data.globalMutex) != VTHREAD_OK)
    fatalINTERR ();	/*NCV*/
}

/*
 *********************************************************************
 */

static void unlockGlobal (void) {
  if (vthreadMutexUnlock (&data.globalMutex) != VTHREAD_OK)
    fatalINTERR ();	/*NCV*/
}

/*
 *********************************************************************
 */

static void lockPort (Port* port) {
  if (vthreadMutexLock (&port->mutex) != VTHREAD_OK)
    fatalINTERR ();	/*NCV*/
}

/*
 *********************************************************************
 */

static void unlockPort (Port* port) {
  if (vthreadMutexUnlock (&port->mutex) != VTHREAD_OK)
    fatalINTERR ();	/*NCV*/
}

/*
 *********************************************************************
 */

static void setExitFlagAndWakeupScanners_LCK (void) {

  Port* port = data.timerLoopPort;

  (void) vsys_get_time (&data.wakeupScannersTime);
  data.exitFlag = 1;
  
  unlockGlobal ();

  if (port != NULL) {
    lockPort (port);
    if (port->timerBank != NULL)
      if (vtimerAlarm (port->timerBank) != VTIMER_OK)
	fatalVTIMER (port->timerBank);
    unlockPort (port);
  }

  lockGlobal ();
}

/*
 *********************************************************************
 */

/* Stuff from io.c and slio.c; don't need that file because there is no serial or TCP
   connection to be managed */

vp_t * vmbAlloc (VUINT8 **requestBuf, unsigned *requestBufSize,
		 VUINT8 **responseBuf, unsigned *responseBufSize) {
  vp_t *h = (vp_t *)VMEM_MALLOC0 (sizeof *h);
  if (h != NULL) {
    if ((h->sl = slInit ()) == NULL) {
      VMEM_FREE (h);
      h = NULL;
    }
  }
  if (h != NULL) {
   /*  h->tcp.port = VMB_HYTEC_PORT; */
/*     h->type = VMB_TYPE_RTU; */
    h->debug = 0;
    h->tcp.timeout.t_sec = 5;
    h->tcp.timeout.t_nsec = 0;
    *requestBuf = USER_REQUEST_BUFFER (h);
    *responseBuf = USER_RESPONSE_BUFFER (h);
    *requestBufSize = USER_BUFFER_SIZE;
    *responseBufSize = USER_BUFFER_SIZE;
  }
  return h;
}

int vmbConnect (Port *p, char *name) {
  Plc *plc;
  int status = 0;
  int finalstat = 0;

#ifndef NO_HARDWARE_TEST 

  for (plc = data.plcList; plc != NULL; plc = plc->next) {
    //    printf("looping: fnalstate %d\n", finalstat);
    switch(plc->port->type) { 
      /************************************************************/
      /* Hytec 85156 (COM1)                                       */
      /************************************************************/
    case(VMB_TYPE_85156): {
      plc->port->hytec_card = (CHy85156 *)new CHy85156;
      int i = 0;
      //  COMPRT = (CHy85156 *)fatalMalloc (sizeof(CHy85156));
      if(data.debug)
      vsys_msg_printf(0,"Initializing 85156 \"%s\"", plc->port->name);
      status = 0;
      if (plc->port->ipslot != NULL) {
	char ipslot;
	ipslot = plc->port->ipslot[0];
if(data.debug)
	vsys_msg_printf(0, "vmeslot %d, IPslot %c, vector %d, delay %d, halfduplex %d, delay485 %d", plc->port->vmeslot, ipslot, plc->port->vectornum, plc->port->delay, plc->port->halfduplex, plc->port->delay485);
	
	status = ((CHy85156 *)plc->port->hytec_card)->Hy85156_Configure(plc->port->vmeslot, ipslot, plc->port->vectornum, plc->port->delay, plc->port->halfduplex, plc->port->delay485);
	if (status != 0) {
/* 	  plc->port->hytec_card = (void *)&COMPRT; */
/* 	else { */
	  char buff[2000];
	  plc->port->hytec_card = NULL;
	  sprintf(buff, "85156: \"%s\", code %d", plc->port->name, status);
	  hytec_error(status);
	  (void) fatalCARDINIT(buff);
	}
      }      else
	fatalNOSLOT();
      //  for(i=0; i<8; i++) {	
	status = ((CHy85156 *)plc->port->hytec_card)->Hy85156_ComsSetup(plc->port->portnum, plc->port->baud, plc->port->parity, plc->port->stop, plc->port->databits, plc->port->flowctl);
	if (status != 0) {
	  char buff[2000];
	  plc->port->hytec_card = NULL;
	  sprintf(buff, "85156: \"%s\", port %d, code %d", plc->port->name, i, status);	  
	  hytec_error(status);
	  (void) fatalPORTINIT(buff);
	}
	//  }
      break;
    }
      /************************************************************/
      /* Hytec 8505 (DIO)                                       */
      /************************************************************/
    case(VMB_TYPE_8505): {
      plc->port->hytec_card = (CHy85056 *)new CHy85056;
      int i = 0;
      // DIO = (CHy85056 *)fatalMalloc (sizeof(CHy85056));
      if(data.debug)
      vsys_msg_printf(0,"Initializing 8505 \"%s\"", plc->port->name);
      status = 0;
      if (plc->port->ipslot != NULL) {
	char ipslot;
	ipslot = plc->port->ipslot[0];
if(data.debug)
	vsys_msg_printf(0, "ipslot %c, %x, finally %d", ipslot, ipslot, ipslot - 0x41);
if(data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c,debrate %d, debmask %d, scanrate %d, dir %d, vectornum %d, intmask %d, pwidth %d, pmask %d, clock %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->debrate, 
			plc->port->debmask, 
			plc->port->scanrate, 
			plc->port->dir, 
			plc->port->vectornum, 
			plc->port->intmask, 
			plc->port->pwidth,
			plc->port->pmask,
			plc->port->clock);
	status = ((CHy85056 *)plc->port->hytec_card)->Hy85056_Configure(plc->port->vmeslot, ipslot, plc->port->debrate, plc->port->debmask, 
					  plc->port->scanrate, plc->port->dir, plc->port->vectornum, plc->port->intmask, 
					  plc->port->pwidth, plc->port->pmask, plc->port->clock);	
      }      else
	fatalNOSLOT();
if(data.debug)
      vsys_msg_printf(0, "status: %d", status);
      if (status != 0) {
/* 	plc->port->hytec_card = (void *)&DIO; */
/*       else { */
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8505: \"%s\", code %d", plc->port->name, status);
	hytec_error(status);
	(void) fatalCARDINIT(buff);
      }
      break;
    }

      /************************************************************/
      /* Hytec 8506 (DIO)                                       */
      /************************************************************/
    case(VMB_TYPE_8506): {
      plc->port->hytec_card = (CHy85056 *)new CHy85056;
      int i = 0;
      // DIO = (CHy85056 *)fatalMalloc (sizeof(CHy85056));
      if(data.debug)
      vsys_msg_printf(0,"Initializing 8506 \"%s\"", plc->port->name);
      status = 0;
      if (plc->port->ipslot != NULL) {
	char ipslot;
	ipslot = plc->port->ipslot[0];
if(data.debug)
	vsys_msg_printf(0, "ipslot %c, %x, finally %d", ipslot, ipslot, ipslot - 0x41);
if(data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c,debrate %d, debmask %d, scanrate %d, dir %d, vectornum %d, intmask %d, pwidth %d, pmask %d, clock %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->debrate, 
			plc->port->debmask, 
			plc->port->scanrate, 
			plc->port->dir, 
			plc->port->vectornum, 
			plc->port->intmask, 
			plc->port->pwidth,
			plc->port->pmask,
			plc->port->clock);
	status = ((CHy85056 *)plc->port->hytec_card)->Hy85056_Configure(plc->port->vmeslot, ipslot, plc->port->portnum, plc->port->debrate, plc->port->debmask, 
					  plc->port->scanrate, plc->port->dir, plc->port->vectornum, plc->port->intmask, 
					  plc->port->pwidth, plc->port->pmask, plc->port->clock);	
      }      else
	fatalNOSLOT();
if(data.debug)
      vsys_msg_printf(0, "status: %d", status);
      if (status != 0) {
/* 	plc->port->hytec_card = (void *)&DIO; */
/*       else { */
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8506: \"%s\", code %d", plc->port->name, status);
	hytec_error(status);
	(void) fatalCARDINIT(buff);
      }
      break;
    }

      /************************************************************/
      /* Hytec 8601 (Motor)                                       */
      /************************************************************/
    case(VMB_TYPE_8601): {
      plc->port->hytec_card = (CHy8601 *)new CHy8601;
      status = 0;
      int i = 0;
      int axis = 0;
      // motor1 = (CHy8601 *)fatalMalloc (sizeof(CHy8601));
if(data.debug)
      vsys_msg_printf(0,"Initializing 8601 \"%s\"", plc->port->name);
      if (plc->port->ipslot != NULL) {
	char ipslot;
	int cardstat = 0;
	ipslot = plc->port->ipslot[0];
if(data.debug)
	vsys_msg_printf(0, "ipslot %c\n", ipslot);
	//	for(i=0; i<4; i++)  {                  
		  //initialise 4 motor channels
	  
	//  axis = i;
	    status = ((CHy8601 *)plc->port->hytec_card)->Hy8601_Configure(plc->port->vmeslot, ipslot);
	    if (status != 0) {
	      cardstat = 1;
	      char buff[2000];
	      plc->port->hytec_card = NULL;
	      sprintf(buff, "8601: \"%s\", code %d", plc->port->name, i, status);
	  hytec_error(status);
	      (void) fatalCARDINIT(buff);
	    }

	    status = ((CHy8601 *)plc->port->hytec_card)->Hy8601_MotorSetup(plc->port->axis, plc->port->startspeed, plc->port->maxspeed, plc->port->ramp);
	    if (status != 0) {
	      char buff[2000];
	      plc->port->hytec_card = NULL;
	      sprintf(buff, "8601: \"%s\", axis %d, code %d", plc->port->name, i, status);
	  hytec_error(status);
	      (void) mberrorAXIS(buff);
	      //  }
	 }
/* 	if (cardstat != 0) */
/* 	  plc->port->hytec_card = (void *)&motor1; */
	  
       }      else 
	fatalNOSLOT();
      break;
    }
      /************************************************************/
      /* Hytec 8402 (DAC)                                       */
      /************************************************************/
    case(VMB_TYPE_8402): {
      plc->port->hytec_card = (CHy8402 *)new CHy8402;

      status = 0;
      if (data.debug)
      vsys_msg_printf(0,"Initializing 8402 \"%s\"", plc->port->name);
      if (plc->port->ipslot != NULL) {
	char ipslot;
	ipslot = plc->port->ipslot[0];
if (data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c, vectornum %d, doram %d, clocksource %d, clockrate %d, inhibit %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->vectornum, 
			plc->port->doram, 
			plc->port->clocksource, 
			plc->port->clockrate, 
			plc->port->inhibit);

	status = ((CHy8402 *)plc->port->hytec_card)->Hy8402_Configure(plc->port->vmeslot, 
				      plc->port->ipslot[0], 
				      plc->port->vectornum, 
				      plc->port->doram, 
				      plc->port->clocksource, 
				      plc->port->clockrate, 
				      plc->port->inhibit); 


/* 	status = (DAC).Hy8402_Configure(vmeslot, dacIPslot1, vectornum2, doram1, clockSource1, dacClockRate1, inhibit1); */

      }      else
	fatalNOSLOT();
      if (data.debug)
	vsys_msg_printf(0, "status: %d", status);
      if (status != 0) {
/* 	plc->port->hytec_card = (void *)&DAC; */
/* 	vsys_msg_printf(0, "8402: %x\n", &DAC); */
/*       } else { */
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8402: \"%s\", code %d", plc->port->name, status);
	  hytec_error(status);
	(void) fatalCARDINIT(buff);
      }
      break;
    }
      /************************************************************/
      /* Hytec 8403 (DAC)                                       */
      /************************************************************/
    case(VMB_TYPE_8403): {
      //CHy8403 *DAC03 = new CHy8403;
      plc->port->hytec_card = (CHy8403 *)new CHy8403;

      status = 0;   
      if (data.debug)
      vsys_msg_printf(0,"Initializing 8403 \"%s\"", plc->port->name);
      
      //  DAC03 = (CHy8403 *)fatalMalloc (sizeof(CHy8403));

      setup_dac((CHy8403 *)plc->port->hytec_card);

      if (plc->port->ipslot) {
	char ipslot;
	ipslot = plc->port->ipslot[0];
if (data.debug)
	vsys_msg_printf(0, "vmeslot %d, ipslot %c, inputtype %d, gain %d", plc->port->vmeslot, plc->port->ipslot[0], plc->port->inputtype, plc->port->gain );	
	status = ((CHy8403 *)plc->port->hytec_card)->Hy8403_Configure(plc->port->vmeslot, plc->port->ipslot[0], plc->port->inputtype, plc->port->gain);
      } else
	fatalNOSLOT();

      if (status != 0) {
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8403: \"%s\", code %d", plc->port->name, status);
	  hytec_error(status);
	(void) fatalCARDINIT(buff);
      } else {
	/* Set up auto-calibration for 6 hour interval */
	status = ((CHy8403 *)plc->port->hytec_card)->Hy8403_EnableAutoCal(12);
	if (status != 0) {
	  char buff[2000];
	  sprintf(buff, "8403: \"%s\", code %d", plc->port->name, status);
	  hytec_error(status);
	}
      }
      break; }
      /************************************************************/
      /* Hytec 8408 ()                                       */
      /************************************************************/
      case(VMB_TYPE_8408): {
      plc->port->hytec_card = (CHy8408 *)new CHy8408;

      status = 0;
      if (data.debug)
      vsys_msg_printf(0,"Initializing 8408 \"%s\"", plc->port->name);
      if (plc->port->ipslot != NULL) {
	char ipslot;
	ipslot = plc->port->ipslot[0];
if (data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c, clockrate %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->clockrate);

	status = ((CHy8408 *)plc->port->hytec_card)->Hy8408_Configure(plc->port->vmeslot, 
				      plc->port->ipslot[0], 
				      plc->port->clockrate); 


/* 	status = (DAC).Hy8402_Configure(vmeslot, dacIPslot1, vectornum2, doram1, clockSource1, dacClockRate1, inhibit1); */

      }      else
	fatalNOSLOT();
if (data.debug)
      vsys_msg_printf(0, "status: %d", status);
      if (status != 0) {
/* 	plc->port->hytec_card = (void *)&DAC; */
/* 	vsys_msg_printf(0, "8402: %x\n", &DAC); */
/*       } else { */
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8408: \"%s\", code %d", plc->port->name, status);
	  hytec_error(status);
	(void) fatalCARDINIT(buff);
      }
      break;
    }

      /************************************************************/
      /* Hytec 8404 ()                                       */
      /************************************************************/

    case(VMB_TYPE_8404): {
      plc->port->hytec_card = (CHy8404 *)new CHy8404;

      status = 0;
if (data.debug)
      vsys_msg_printf(0,"Initializing 8404 \"%s\"", plc->port->name);
      if (plc->port->ipslot != NULL) {
	char ipslot;
	ipslot = plc->port->ipslot[0];
if (data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c, vectornum %d, doram %d, clocksource %d, clockrate %d, inhibit %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->vectornum, 
			plc->port->doram, 
			plc->port->clocksource, 
			plc->port->clockrate, 
			plc->port->inhibit);

	status = ((CHy8404 *)plc->port->hytec_card)->Hy8404_Configure(plc->port->vmeslot, 
				      plc->port->ipslot[0], 
				      plc->port->vectornum, 
				      plc->port->doram, 
				      plc->port->clocksource, 
				      plc->port->clockrate, 
				      plc->port->inhibit); 


/* 	status = (DAC).Hy8402_Configure(vmeslot, dacIPslot1, vectornum2, doram1, clockSource1, dacClockRate1, inhibit1); */

      }      else
	fatalNOSLOT();
if (data.debug)
      vsys_msg_printf(0, "status: %d", status);
      if (status != 0) {
/* 	plc->port->hytec_card = (void *)&DAC; */
/* 	vsys_msg_printf(0, "8402: %x\n", &DAC); */
/*       } else { */
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8404: \"%s\", code %d", plc->port->name, status);
	  hytec_error(status);
	(void) fatalCARDINIT(buff);
      }
      break;
    }

      /************************************************************/
      /* Hytec 8413 ()                                       */
      /************************************************************/

     case(VMB_TYPE_8413): {
       plc->port->hytec_card = (CHy8413 *)new CHy8413;
       status = 0;
if (data.debug)
       vsys_msg_printf(0,"Initializing 8413 \"%s\"", plc->port->name);
       //   DAC13 = (CHy8413 *)fatalMalloc (sizeof(CHy8413));
       if (plc->port->ipslot) {
if (data.debug)
	 vsys_msg_printf(0, "vmeslot %d, ipslot %c, vectornum %d itrstate %d aitype %d clocksource %d clockrate %d inhibit %d samples %d spacing %d trigger %d", plc->port->vmeslot, plc->port->ipslot[0], plc->port->vectornum, plc->port->itrstate, plc->port->aitype, plc->port->clocksource, plc->port->clockrate, plc->port->inhibit, plc->port->samples, plc->port->spacing, plc->port->trigger);

	 	 status = ((CHy8413 *)plc->port->hytec_card)->Hy8413_Configure(plc->port->vmeslot, plc->port->ipslot[0], plc->port->vectornum, plc->port->itrstate, plc->port->aitype, plc->port->clocksource, plc->port->clockrate, plc->port->inhibit, plc->port->samples, plc->port->spacing, plc->port->trigger);
		 //status = hytec_8413_card.Hy8413_Configure(plc->port->vmeslot, plc->port->ipslot[0], plc->port->vectornum, plc->port->itrstate, plc->port->aitype, plc->port->clocksource, plc->port->clockrate, plc->port->inhibit, plc->port->samples, plc->port->spacing, plc->port->trigger);
       } else
	 fatalNOSLOT();
       
      if (status != 0) {
/* 	plc->port->hytec_card = (void *)&DAC13; */
/*        else { */
	 char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8413: \"%s\", code %d", plc->port->name, status);
	  hytec_error(status);
	(void) fatalCARDINIT(buff);
       }
	break; }

/* case(VMB_TYPE_8413): { */
/*   vsys_msg_printf(0,"******* 8413 not yet implemented ******"); */
/* break; } */
      /************************************************************/
      /* Hytec 8005 ()                                       */
      /************************************************************/
    case(VMB_TYPE_8005): {
     plc->port->hytec_card = (CHy8005 *)new CHy8005;
      status = 0;
if (data.debug)
      vsys_msg_printf(0,"Initializing 8005 \"%s\"", plc->port->name);
      //  HY = (CHy8005 *)fatalMalloc (sizeof(CHy8005));
      if (plc->port->ipslot) {
	char ipslot;
	ipslot = plc->port->ipslot[0];
	if (data.debug)
	vsys_msg_printf(0, "vmeslot %d, ipslot %c, ", plc->port->vmeslot, plc->port->ipslot[0]);
	if (data.debug)
	vsys_msg_printf(0, "portnum %d, debrate %d, debmask %d, scanrate %d, dir %d, vectornum %d, intmask %d, pwidth %d, pmask %d, inhibit %d", plc->port->portnum, plc->port->debrate, 
					plc->port->debmask, plc->port->scanrate,
					plc->port->dir, plc->port->vectornum, 
					plc->port->intmask, plc->port->pwidth, 
					plc->port->pmask, plc->port->inhibit);
	status = ((CHy8005 *)plc->port->hytec_card)->Hy8005_Configure(plc->port->vmeslot, ipslot, 
					plc->port->portnum, plc->port->debrate, 
					plc->port->debmask, plc->port->scanrate,
					plc->port->dir, plc->port->vectornum, 
					plc->port->intmask, plc->port->pwidth, 
					plc->port->pmask, plc->port->inhibit);
  
      }      else
	fatalNOSLOT();
      if (data.debug)
      vsys_msg_printf(0, "status: %d", status);
      if (status != 0) {
/* 	plc->port->hytec_card = (void *)&HY; */
/*       else { */
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8005: \"%s\", code %d", plc->port->name, status);
	  hytec_error(status);
	(void) fatalCARDINIT(buff);
      }
      break;
    }
      /************************************************************/
      /* Hytec 8411 ()                                       */
      /************************************************************/
    case(VMB_TYPE_8411): {
      plc->port->hytec_card = (CHy8411 *)new CHy8411;
      status = 0;
      //  HY = (CHy8411 *)fatalMalloc (sizeof(CHy8411));
      if (data.debug)
      vsys_msg_printf(0,"Initializing 8411 \"%s\"", plc->port->name);
      if (plc->port->ipslot != NULL) {
	char ipslot;
	ipslot = plc->port->ipslot[0];
	if (data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c, clockrate %d,",
			plc->port->vmeslot, 
			ipslot,
			plc->port->clockrate);
	status = ((CHy8411 *)plc->port->hytec_card)->Hy8411_Configure(plc->port->vmeslot, ipslot, plc->port->clockrate);
      }      else
	fatalNOSLOT();
      if (data.debug)
      vsys_msg_printf(0, "status: %d", status);
      if (status != 0) {
/* 	plc->port->hytec_card = (void *)&HY; */
/*       else { */
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8411: \"%s\", code %d", plc->port->name, status);
  hytec_error(status);
	(void) fatalCARDINIT(buff);
      }
      break;
    }
      /************************************************************/
      /* Hytec 8512()                                       */
      /************************************************************/
          case(VMB_TYPE_8512): {
	    plc->port->hytec_card = (CHy8512 *)new CHy8512;
      status = 0;
      if (data.debug)
      vsys_msg_printf(0,"Initializing 8512 \"%s\"", plc->port->name);
      if (plc->port->ipslot != NULL) {
	char ipslot;
	ipslot = plc->port->ipslot[0];
	if (data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c, scaler %d, clock %d, vectornum %d, overflow %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->scaler,
			plc->port->clock,
			plc->port->vectornum, 
			plc->port->overflow);
	status = ((CHy8512 *)plc->port->hytec_card)->Hy8512_Configure(plc->port->vmeslot, ipslot, plc->port->scaler, plc->port->clock, plc->port->vectornum, plc->port->overflow);
      }      else
	fatalNOSLOT();
if (data.debug)
      vsys_msg_printf(0, "status: %d", status);
      if (status != 0) {
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8512: \"%s\", code %d", plc->port->name, status);
  hytec_error(status);
	(void) fatalCARDINIT(buff);
	}
      break;
    }
      default: {
	fatalINVTYPE ();
	break;
      }
    }
    finalstat += status;
  }

#endif /* NO_HARDWARE_TEST */

  if(finalstat == 0)  
    return SUCCESS;
  else
    return FAILURE;
}

char * vmbStrerror (vp_t *h) {
  return h->errstr;
}

#define errINVARG(h) errInvarg (h, __LINE__)

static int errInvarg (vp_t *h, int line) {
  (void) sprintf (h->errstr, "VMBIO_INVARG, Invalid argument (%d)", line);
  return FAILURE;
}

#define errINVARG_sl(h) errInvarg_sl (h, __LINE__)

static int errInvarg_sl (sl_t *h, int line) {
  (void) sprintf (h->errstr, "VMBIO_INVARG, Invalid argument (%d)", line);
  return FAILURE;
}



static int stringError (vp_t *h, char *string) {
  (void) vsys_strncpy (h->errstr, string, sizeof h->errstr);
  return FAILURE;
}

static int slError (vp_t *h) {
  return stringError (h, slStrerror (h->sl));
}


char * slStrerror (sl_t *h) {
  return h->errstr;
}

sl_t* slInit (void) {

  sl_t* h = (sl_t*)VMEM_MALLOC0 (sizeof *h);

  /* Timeouts used to be hardcoded. When I made them configurable, I kept
     the defaults. Where do these defaults come from - I have no idea!
     (EZ 7-Oct-2004). */

#ifdef VMS
  h->timeout = 2;
#elif defined WINNT
  h->timeout = 4;
#else
  h->timeout = 5;
#endif

  return h;
}

static int baudToBaudSym (sl_t *h, int baud, int *baudSym) {
  unsigned i;
  for (i = 0; i < VSYS_ARRAY_SIZE (baudSymTable); i++) {
    if (baudSymTable[i].baud == baud) {
      *baudSym = baudSymTable[i].baudSym;
      return SUCCESS;
    }
  }
  return errINVARG_sl (h);
}

int slSetBaud (sl_t *h, int baud) {
  int baudSym;
  if (baudToBaudSym (h, baud, &baudSym) != SUCCESS)
    return FAILURE;
  else {
    h->baud = baud;
    h->baudSym = baudSym;
    return SUCCESS;
  }
}

/*
 *********************************************************************
 */

int slSetDebug (sl_t *h, int value) {
  h->debug = value;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slSetParity (sl_t *h, int value) {
  h->parity = value;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slSetRts (sl_t *h, int value) {
  h->delays.rts = value;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slSetStop (sl_t *h, int value) {
  h->stop = value;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slSetTurnaround (sl_t *h, int value) {
  h->delays.turnaround = value;
  return SUCCESS;
}

/*
 *********************************************************************
 */

int slSetTimeout (sl_t *h, unsigned value) {
  h->timeout = value;
  return SUCCESS;
}

int vmbSet (vp_t *h, int option, void *value) {

  bool connected = h->connected;
  sl_t *sl = h->sl;
  
  switch (option) {
  case VMB_O_BAUD:
    if (connected) goto err;
    if  (h->sl != NULL && slSetBaud (sl, (int) value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_DEBUG:
    h->debug = (int) value;
    if (h->sl != NULL && slSetDebug (h->sl, (int) value) != SL_OK)
      return slError (h);
    break;
    /*  case VMB_O_DRIVER:
    if (connected) goto err;
    if (h->user.driver != NULL) VMEM_FREE (h->user.driver);
    if (value != NULL) {
      h->user.driver = (char *)VMEM_STRDUP (value);
      if (h->user.driver == NULL) return errNOMEM (h);
    }
    break;*/
  case VMB_O_ERROR:
    (void) stringError (h, (char*)value);
    break;
  case VMB_O_PARITY:
    if (connected) goto err;
    if  (h->sl != NULL && slSetParity (sl, (int) value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_PORT:
    /* normally it is VMB_HYTEC_PORT, but for debugging purpose
       we may use another one (to avoid running our debugging server
       from root, for example) */
    if (connected) goto err;
    h->tcp.port = (int) value;
    break;
  case VMB_O_STOP:
    if (connected) goto err;
    if  (h->sl != NULL && slSetStop (sl, (int) value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_TYPE:
    if (connected) goto err;
    h->type = (int) value;
    break;
  case VMB_O_RTS:
    if (connected) goto err;
    if  (h->sl != NULL && slSetRts (sl, (int) value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_TURNAROUND:
    if (connected) goto err;
    if  (h->sl != NULL && slSetTurnaround (sl, (int) value) != SL_OK)
      return slError (h);
    break;
  case VMB_O_TIMEOUT:
    if (connected) goto err;
    h->tcp.timeout.t_sec = (unsigned) value;
    h->tcp.timeout.t_nsec = 0;
    if (h->sl != NULL && slSetTimeout (sl, (unsigned) value) != SL_OK)
      return slError (h);
    break;
  default:
    return errINVARG (h);
  }

  return SUCCESS;

 err:
  return errINVARG (h); /* h->connected != 0 */
}

static void timeSpanTimerCallback (void* arg) {
  if (! data.exitFlag)
    setExitFlagAndWakeupScanners_LCK ();
}


/*****************************************************************
 * It would probably be better to do everything with mutex unlocked,
 * but vtimer does not allow the mutex to be unlocked in the callback.
 *********************************************************************
 */

static void wdTimerCallback (void* arg) {
  if (! data.exitFlag) {
    if (vdb_iput (&data.wd.chix, &data.wd.value) != VDB_ERR_OK)
      (void) errorWDPUT (&data.wdChannelChix);		/*NYCV*/
    if (data.wd.value == data.wd.valueMax)
      data.wd.value = data.wd.valueMin;		/*NYCV*/
    else					/*NYCV*/
      data.wd.value++;
  }
}

/*
 *********************************************************************
 */

static void initWdAndTimeSpanTimers (void) {

  bool wdTimer = data.wd.channelName && data.wd.rate != 0;

  if (data.timeSpan != 0 || wdTimer)
    if ((data.timerBank = vtimerBankCreate (&data.globalMutex)) == NULL)
      fatalNOMEM ();	/*NCV*/

  if (data.timeSpan != 0) {
    Vtimer* timer;
    if ((timer = data.timeSpanTimer = vtimerCreate (data.timerBank)) == NULL ||
	vtimerSetCallback (timer, timeSpanTimerCallback, NULL) != VTIMER_OK ||
	vtimerSet (timer, data.timeSpan) != VTIMER_OK)
      fatalVTIMER (data.timerBank);	/*NCV*/
  }

  if (wdTimer) {
    Vtimer* timer;
    if ((timer = data.wd.timer = vtimerCreate (data.timerBank)) == NULL ||
	vtimerSetCallback (timer, wdTimerCallback, NULL) != VTIMER_OK ||
	vtimerEnableAutomaticMode (timer) != VTIMER_OK ||
	vtimerSet (timer, data.wd.rate) != VTIMER_OK)
      fatalVTIMER (data.timerBank);	/*NCV*/
  }
}

/*
 *********************************************************************
 */

static void deleteTimerBank (void) {
  lockGlobal ();
  if (data.timerBank != NULL) {
    if (vtimerBankDelete (data.timerBank) != VTIMER_OK)
      fatalINTERR ();	/*NCV*/
    data.timerBank = NULL;
  }
  unlockGlobal ();
}

/*
 *********************************************************************
 */

static char* mydbChannelName (vista_chix* chix) {
  if (vdbc_channel_name (chix, data.channelName, sizeof data.channelName)
      != VDB_ERR_OK)
    vscannerFatalVdbChixError (chix); /*NCV*/
  return data.channelName;
}

/*
 *********************************************************************
 * This function gets the classNumber, fieldMask and extendedIndex
 * of the specified field.
 *********************************************************************
 */

static void mydbFindField (FunctionField* field) {

  VDB_FIELDS_INFO out[4];
  VDB_FIELDS_INFO in[1];
  unsigned classNumber;
  unsigned fieldMask;
  unsigned extendedIndex;
  int dtype;

  in[0].field_type = FIELD_INFO_FLDNAME;
  in[0].data_len = strlen (field->name);
  in[0].data_ptr = field->name;

  out[0].field_type = FIELD_INFO_CLASSNUM;
  out[0].data_len = sizeof (classNumber);
  out[0].data_ptr = &classNumber;

  out[1].field_type = FIELD_INFO_FIELDMASK;
  out[1].data_len = sizeof (fieldMask);
  out[1].data_ptr = &fieldMask;

  out[2].field_type = FIELD_INFO_INDEX;
  out[2].data_len = sizeof (extendedIndex);
  out[2].data_ptr = &extendedIndex;

  out[3].field_type = FIELD_INFO_DATATYPEX;
  out[3].data_len = sizeof (dtype);
  out[3].data_ptr = &dtype;

  if (vdb_fields_info (1, in, 4, out, NULL, 0) != VDB_ERR_OK)
    fatalNOSUCHFIELD (field->name);

  field->classNumber = classNumber;
  field->fieldMask = fieldMask;
  field->extendedIndex = extendedIndex;
  field->dtype = dtype;
}

/*
 *********************************************************************
 */

static char* mydbGetStringField (vista_chix* chix, FunctionField* field,
				 char* buff, unsigned size) {
  VDB_DATA_ITEM_LIST item[2];
  if (field->fieldMask == 0)
    mydbFindField (field);
  item[0].chix = *chix;
  item[0].event_class_number = field->classNumber;
  item[0].event_field_mask = field->fieldMask;
  item[0].index = field->extendedIndex;
  item[0].data_addr = buff;
  item[0].dtype = VDB_DTYPE_STRING;
  item[0].size = size;
  item[1].chix.chix = 0;
  (void) vdb_get_item (item);
  if (item[0].status == VDB_ERR_OK)
    return buff;
  else if (item[0].status != VDB_ERR_NOTALLOC)
    vscannerFatalVdbStatusError (chix, item[0].status);	/*NYCV*/
  return NULL;
}

/*
 *********************************************************************
 * This function retrieves an unsigned int value of the specified
 * field. Returns 1 on success, 0 if the field is not allocated,
 * forces the process to exit if none of the above.
 *********************************************************************
 */

static int mydbGetUintField (vista_chix* chix, FunctionField* field,
			     unsigned* value) {
  VDB_DATA_ITEM_LIST item[2];
  *value = 0;
  if (field->fieldMask == 0)
    mydbFindField (field);
  item[0].chix = *chix;
  item[0].event_class_number = field->classNumber;
  item[0].event_field_mask = field->fieldMask;
  item[0].index = field->extendedIndex;
  item[0].data_addr = value;
  item[0].dtype = VDB_DTYPE_UINT;
  item[0].size = 4;
  item[1].chix.chix = 0;
  (void) vdb_get_item (item);
  if (item[0].status == VDB_ERR_OK) return 1;
  if (item[0].status != VDB_ERR_NOTALLOC)
    vscannerFatalVdbStatusError (chix, item[0].status);	/*NYCV*/
  return 0;
}

/*
 *********************************************************************
 */

static int mydbGetIOError (vista_chix* chix) {
  struct vdb_descrip_v desc;
  int value;
  VSYS_INTEGER_FILL (desc, value, sizeof value);
  if (vdb_get (chix, VCLS_M_ENTLVE, VFLD_M_IO_ERROR, &desc) != VDB_ERR_OK)
    vscannerFatalVdbChixError (chix); /*NCV*/
  return value == 0 ? 0 : 1;
}

/*
 *********************************************************************
 */

#if vmbCONF_TRACE_TRANSACTION
static void traceTransactRequest (Plc* plc,
				  unsigned requestLen,
				  unsigned expectedResponseLen) {
  Port* port = plc->port;
  if (strcmp (port->name, "port1") == 0) {
    VUINT8* request = port->request;
    unsigned function = request[0];
    switch (function) {
    case 6: {
      unsigned addr = request[1] * 256 + request[2];
      unsigned value = request[3] * 256 + request[4];
      logMsg ("req %s:%s set %u = %u", port->name, plc->name, addr, value);
      break;
    }
    case 4: {
      unsigned addr = request[1] * 256 + request[2];
      unsigned count = request[4];
      logMsg ("req %s:%s get %u %u", port->name, plc->name, addr, count);
      break;
    }
    case 2: {
      unsigned addr = request[1] * 256 + request[2];
      unsigned count = request[4];
      logMsg ("req %s:%s bget %u %u", port->name, plc->name, addr, count);
      break;
    }
    default:
      logMsg ("req %s:%s function %u", port->name, plc->name, function);
    }
  }
}
#endif

/*
 *********************************************************************
 */

#if vmbCONF_TRACE_TRANSACTION
static void traceTransactReply (Plc* plc, int status) {
  Port* port = plc->port;
  if (strcmp (port->name, "port1") == 0) {
    if (status == VMB_OK)
      logMsg ("rep %s:%s OK", port->name, plc->name);
    else
      logMsg ("rep %s:%s ERROR", port->name, plc->name);
  }
}
#endif

/*
 *********************************************************************
 */

#if vmbCONF_LOG_MAX_TRANSACT_TIME
static void fatalVarcError (char* line);
#endif

/*
 *********************************************************************
 */

static int hytecTransact (Plc* plc, void* buffer, Channel* chan, int read) {
  Port* port = plc->port;
 float floaty, *fltptr;
 int *intptr;
 long *longptr;
 unsigned long *ulongptr;
  unsigned long longy;
  unsigned short int val = 0;
  char hldr[1000];
  int dtype, status = 0;
  char charfloat[4];
  int bit1 = 0x1B, bit2 = 0x58;
  int bit3 = 0x27, bit4 = 0x88;
/* READ_AI Offsets for Reading Calibration Values */
#define ADC_A_ZERO_CAL  16  /* ADC A - Zero Raw Calibration */
#define ADC_A_POS_CAL   17  /* ADC A - Pos  Raw Calibration */
#define ADC_A_NEG_CAL   18  /* ADC A - Neg  Raw Calibration */
#define ADC_B_ZERO_CAL  19  /* ADC B - Zero Raw Calibration */
#define ADC_B_POS_CAL   20  /* ADC B - Pos  Raw Calibration */
#define ADC_B_NEG_CAL   21  /* ADC B - Neg  Raw Calibration */
    short           iChannelCount;
    unsigned long   ADC_Value;

	/* PT-100 / Cold Junction Temperature Variables */
    double			Vpt100;		/* PT100 Voltage */
    double			Ipt100;		/* PT100 Current */
    double			Rpt100;		/* PT100 Resistance */
    double			Tpt100;		/* PT100 Temperature */
    double           Rref = 100.0; /* Reference/Vish Resistor is 100R */
    double           A = 0.00390802;
    double           B = -0.000000580195;

	double           Voltage;
    double           Temperature;
    double           TV;
	  ULONG value1;
	  ULONG value2;

    fltptr = (float *)buffer;
    longptr = (long *)buffer;
    ulongptr = (unsigned long *)buffer;
    intptr = (int *)buffer;
    
    if (read == WRITE) {
      switch(port->type) {
      case(VMB_TYPE_8505): {
	if(data.debug) 
	  vsys_msg_printf(0,"8505: \"%s\", putting value %d", mydbChannelName (&chan->pub.chix), *intptr);
	//DIO = (CHy85056 *)port->hytec_card;
#ifndef NO_HARDWARE_TEST 
	if (*intptr != 0)
	  status = ((CHy85056*)port->hytec_card)->Hy85056_Write_Bit(chan->pub.addr, 1);
	else
	  status = ((CHy85056*)port->hytec_card)->Hy85056_Write_Bit(chan->pub.addr, 0);
#endif
	if(data.debug) {
	  vsys_msg_printf(0, "8505: \"%s\", code %d", plc->port->name, status);
	   
      }
	break;}
      case(VMB_TYPE_8506): {
	//DIO = (CHy85056 *)port->hytec_card;
	vsys_msg_printf(0,"8506: \"%s\", putting value %d", mydbChannelName (&chan->pub.chix), *intptr);
#ifndef NO_HARDWARE_TEST 
	if (*intptr != 0)
	  status = ((CHy85056*)port->hytec_card)->Hy85056_Write_Bit(port->portnum, chan->pub.addr, 1);
	else
	  status = ((CHy85056*)port->hytec_card)->Hy85056_Write_Bit(port->portnum, chan->pub.addr, 0);
#endif
	if(data.debug) {
	  vsys_msg_printf(0, "8506: \"%s\", code %d", plc->port->name, status);
	   
      }
	break;}
      case(VMB_TYPE_8005): {
	if(data.debug)
	vsys_msg_printf(0,"8005: \"%s\", putting value %d", mydbChannelName (&chan->pub.chix), *intptr);
#ifndef NO_HARDWARE_TEST 
	if (*intptr != 0)
	  status = ((CHy8005*)port->hytec_card)->Hy8005_Write_Bit(port->portnum, chan->pub.addr, 1);
	else
	  status = ((CHy8005*)port->hytec_card)->Hy8005_Write_Bit(port->portnum, chan->pub.addr, 0);
#endif
	if(data.debug) {
	vsys_msg_printf(0, "8005: \"%s\", code %d", plc->port->name, status);
	   
      }
	break;}
      case(VMB_TYPE_8402):
	//  DAC = (CHy8402 *)port->hytec_card; 
	if  (chan->pub.dbDtype == VDB_DTYPE_INT ||
	     chan->pub.dbDtype == VDB_DTYPE_UINT) {
	  if(data.debug)
	    vsys_msg_printf(0,"8402: \"%s\", putting value %li", mydbChannelName (&chan->pub.chix), *longptr);
#ifndef NO_HARDWARE_TEST
	  status = ((CHy8402*)port->hytec_card)->Hy8402_Write(chan->pub.addr, (USHORT)*longptr);
#endif
	} else {
	  if(data.debug)
	    vsys_msg_printf(0,"8402: \"%s\", putting value %f", mydbChannelName (&chan->pub.chix), *fltptr);
#ifndef NO_HARDWARE_TEST 
	  status = ((CHy8402*)port->hytec_card)->Hy8402_Write(chan->pub.addr, (USHORT) *fltptr);
#endif
	}
	if(data.debug) {
	  vsys_msg_printf(0, "8402: \"%s\", code %d", plc->port->name, status);
	  
	}
	break;
      case(VMB_TYPE_8404):{
	//  DAC = (CHy8402 *)port->hytec_card; 
	if  (chan->pub.dbDtype == VDB_DTYPE_INT ||
	     chan->pub.dbDtype == VDB_DTYPE_UINT) {
	  if(data.debug)
	    vsys_msg_printf(0,"8404: \"%s\", putting value %li", mydbChannelName (&chan->pub.chix), *longptr);
#ifndef NO_HARDWARE_TEST
	  status = ((CHy8404*)port->hytec_card)->Hy8404_Write(chan->pub.addr, (USHORT)*longptr, port->range);
#endif
	} else {
	if(data.debug)
	  vsys_msg_printf(0,"8404: \"%s\", putting value %f", mydbChannelName (&chan->pub.chix), *fltptr);
#ifndef NO_HARDWARE_TEST 
	status = ((CHy8404*)port->hytec_card)->Hy8404_Write(chan->pub.addr, (USHORT) *fltptr, port->range);
#endif
	}
	if(data.debug) {
	vsys_msg_printf(0, "8404: \"%s\", code %d", plc->port->name, status);
	   
      }
	break;
      }
      case(VMB_TYPE_8601):
	//  DAC = (CHy8402 *)port->hytec_card;
	if  (chan->pub.dbDtype == VDB_DTYPE_INT ||
	     chan->pub.dbDtype == VDB_DTYPE_UINT) {
	  if(data.debug)
	    vsys_msg_printf(0,"8601: \"%s\", putting value %li", mydbChannelName (&chan->pub.chix), *longptr);
#ifndef NO_HARDWARE_TEST
	  status = ((CHy8601*)port->hytec_card)->Hy8601_Move(chan->pub.addr, (long)*longptr);
#endif
	} else {
	if(data.debug)
	   vsys_msg_printf(0,"8601: \"%s\", putting value %f", mydbChannelName (&chan->pub.chix), *fltptr);
#ifndef NO_HARDWARE_TEST 
	status = ((CHy8601*)port->hytec_card)->Hy8601_Move(chan->pub.addr, (long) *fltptr);
#endif
	}
	if(data.debug) {
	vsys_msg_printf(0, "8601: \"%s\", code %d", plc->port->name, status);
	   
      }
	break;
      case (VMB_TYPE_8512): {
	if (chan->pub.appregType == VHYTEC_TYPE_OUTPUT_APPREG) {
	  USHORT hldr = 0;
	  int reg = 0;
	  if  (chan->pub.dbDtype == VDB_DTYPE_INT16 ||
	       chan->pub.dbDtype == VDB_DTYPE_UINT16 ||
	       chan->pub.dbDtype == VDB_DTYPE_INT) {
	  if(data.debug)
	    vsys_msg_printf(0,"8512: \"%s\", putting value %d to application register %d", mydbChannelName (&chan->pub.chix), *intptr, chan->pub.addr);
#ifndef NO_HARDWARE_TEST 
	  status = ((CHy8512*)port->hytec_card)->Hy8512_Write_Reg((int)chan->pub.addr, (USHORT)*intptr);
#endif
	  } else {
	     if(data.debug)
	    vsys_msg_printf(0,"8512: \"%s\", putting value %d to application register %d, bit # %d", mydbChannelName (&chan->pub.chix), *intptr, chan->pub.addr , chan->pub.appreg);
#ifndef NO_HARDWARE_TEST 
	  status = ((CHy8512*)port->hytec_card)->Hy8512_Read_Reg((int)chan->pub.addr, (USHORT*)&hldr);
#endif
	  if(data.debug)
	    vsys_msg_printf(0,"8512: Now reading original value of application register %d: %d", chan->pub.addr, hldr);
	  if (*intptr == 0)
	    hldr = hldr & ~(1<<(chan->pub.appreg)); // flip bit off 
	  else 
	    hldr = hldr | 1<<(chan->pub.appreg); // flip bit on
	  if(data.debug)
	  vsys_msg_printf(0,"8512: Now putting new value to application register %d: %d", chan->pub.addr, hldr);
#ifndef NO_HARDWARE_TEST 
	   status = ((CHy8512*)port->hytec_card)->Hy8512_Write_Reg((int)chan->pub.addr, (USHORT)hldr);
#endif
	  }
	} else {
	  if  (chan->pub.dbDtype == VDB_DTYPE_INT ||
	       chan->pub.dbDtype == VDB_DTYPE_UINT) {
	  if(data.debug)
	    vsys_msg_printf(0,"8512: \"%s\", putting long value %li", mydbChannelName (&chan->pub.chix), *ulongptr);
#ifndef NO_HARDWARE_TEST
	  status = ((CHy8512*)port->hytec_card)->Hy8512_Read((int)chan->pub.addr, &value1, 0);
	  status = ((CHy8512*)port->hytec_card)->Hy8512_Read((int)chan->pub.addr + 1, &value2, 0);
#endif
	  vsys_msg_printf(0,"8512: \"%s\", read this channel %li, next channel %li", mydbChannelName (&chan->pub.chix), value1, value2);

#ifndef NO_HARDWARE_TEST
	  status = ((CHy8512*)port->hytec_card)->Hy8512_Write((int)chan->pub.addr, (ULONG)*ulongptr);
	  status = ((CHy8512*)port->hytec_card)->Hy8512_Read((int)chan->pub.addr, &value1, 0);
	  status = ((CHy8512*)port->hytec_card)->Hy8512_Read((int)chan->pub.addr + 1, &value2, 0);
#endif
	  vsys_msg_printf(0,"8512: \"%s\", post-put readback; this channel %li, next channel %li", mydbChannelName (&chan->pub.chix), value1, value2);
	  
	  } else {
if(data.debug)
	    vsys_msg_printf(0,"8512: \"%s\", putting float value %f", mydbChannelName (&chan->pub.chix), *fltptr);
#ifndef NO_HARDWARE_TEST
	  status = ((CHy8512*)port->hytec_card)->Hy8512_Write((int)chan->pub.addr, (ULONG)*fltptr);
#endif
	  }
	}
	if(data.debug) {
	vsys_msg_printf(0, "8512: \"%s\", code %d", plc->port->name, status);
	   
      }

	break;
      }
      default: {
	wrongcard(port->type, "int, binary, or float");
return -1;
break; 
      }
      }
    } else {
      /* READ */
      switch(port->type) {
      case (VMB_TYPE_8403): {
	VDB_DATA_ITEM_LIST dlist[40]; 
	ULONG refvalueA = 0;
	ULONG refvalueB = 0;
	double			Gain;
	vtime returntime;

	Gain = port->gain;
	if(data.debug)
	  vsys_msg_printf(0, "8403!, ADC %x, channel %d, vmeslot %d ipslot %c gain %f!", (CHy8403*)port->hytec_card, chan->pub.addr, port->vmeslot, (char) port->ipslot[0], Gain);
#ifndef NO_HARDWARE_TEST
	/* Read the reference channel, store in Cparam 1 for conversion*/

	if(port->inputtype == INPUTTYPE_TC || port->inputtype == INPUTTYPE_TCB)
	  ((CHy8403 *)(plc->port->hytec_card))->Hy8403_Read(15, (ULONG *)&refvalueB);
	if(port->inputtype == INPUTTYPE_PT100) {
	  ((CHy8403 *)(plc->port->hytec_card))->Hy8403_Read(7, (ULONG *)&refvalueA);
	  ((CHy8403 *)(plc->port->hytec_card))->Hy8403_Read(15, (ULONG *)&refvalueB);
	}
	if(data.debug)
	  vsys_msg_printf(0, "refvalueA %d refvalue B %d", refvalueA, refvalueB); 

	dlist[0].chix = chan->pub.chix; 
	dlist[0].event_class_number = VCLS_N_CONVERSION; 
	dlist[0].event_field_mask = VFLD_M_CPARAM1; 
	dlist[0].dtype = VDB_DTYPE_UINT; 
	dlist[0].size = sizeof(unsigned int); 
	dlist[0].data_addr = (unsigned int *)&refvalueA;

	dlist[1].chix = chan->pub.chix; 
	dlist[1].event_class_number = VCLS_N_CONVERSION; 
	dlist[1].event_field_mask = VFLD_M_CPARAM2; 
	dlist[1].dtype = VDB_DTYPE_FLOAT; 
	dlist[1].size = sizeof(float); 
	dlist[1].data_addr = (float *)&refvalueB;

	dlist[2].chix = chan->pub.chix; 
	dlist[2].event_class_number = VCLS_N_CONVERSION; 
	dlist[2].event_field_mask = VFLD_M_CPARAM3; 
	dlist[2].dtype = VDB_DTYPE_DOUBLE; 
	dlist[2].size = sizeof(double); 
	dlist[2].data_addr = (double *)&Gain; 
		
	vdb_zero_chix(&(dlist[3].chix)); 
  
	vdb_put_item (dlist); 
#endif

	/* Read the channel value */
#ifndef NO_HARDWARE_TEST
	status = ((CHy8403 *)(plc->port->hytec_card))->Hy8403_Read(chan->pub.addr,(ULONG *)&ADC_Value);
#endif
	if (data.debug)
	  vsys_msg_printf(0, "channel value %d 0x%08x", (int)ADC_Value, (int)ADC_Value); 
	if (chan->pub.hwDtype == VDB_DTYPE_FLOAT)
	  *fltptr = (float)ADC_Value;
	else
	  *longptr = ADC_Value;
	
	//  status = ((CHy8403*)port->hytec_card)->Hy8403_Read(chan->pub.addr, (ULONG *)&longy);
	if (data.debug) {
	vsys_msg_printf(0, "8403: \"%s\", code %d", plc->port->name, status);
      }
	
	break; }
	
      case (VMB_TYPE_8411): {
	USHORT value;
	if(data.debug)
	  vsys_msg_printf(0,"8411: \"%s\", channel %d, getting value", mydbChannelName (&chan->pub.chix), chan->pub.addr, *fltptr);
#ifndef NO_HARDWARE_TEST
	status = ((CHy8411*)port->hytec_card)->Hy8411_Read(chan->pub.addr, &value);
#endif
	*fltptr = (float)value;
	if(data.debug) {
	vsys_msg_printf(0, "8411: \"%s\", code %d", plc->port->name, status);
	 
      }
	 
	break;
      }
      case (VMB_TYPE_8413): {
	USHORT value;
if(data.debug)
   vsys_msg_printf(0,"8413: \"%s\", channel %d, getting value", mydbChannelName (&chan->pub.chix), chan->pub.addr, *fltptr);
#ifndef NO_HARDWARE_TEST
	status = ((CHy8413*)port->hytec_card)->Hy8413_Read(chan->pub.addr, &value);
#endif
	*fltptr = (float)value;
	if(data.debug) {
	vsys_msg_printf(0, "8413: \"%s\", code %d", plc->port->name, status);
	 
      }

	break;
      }
      case (VMB_TYPE_8601): {
	ULONG value;
if(data.debug)
  	vsys_msg_printf(0, "8601, channel %d, vmeslot %d, ipslot %c", chan->pub.addr, port->vmeslot, (char) port->ipslot[0]);

#ifndef NO_HARDWARE_TEST
	status = ((CHy8601*)port->hytec_card)->Hy8601_ReadEncoder(chan->pub.addr, &value);
#endif
	*fltptr = (float)value;
	if(data.debug) {
	vsys_msg_printf(0, "8601: \"%s\", code %d", plc->port->name, status);
	 
      }

	break;
      }
      case (VMB_TYPE_8512): {
	ULONG value;
	int hldr;
	int reg;
	 if(data.debug)
	vsys_msg_printf(0, "8512, channel %d, vmeslot %d, ipslot %c", chan->pub.addr, port->vmeslot, (char) port->ipslot[0]);
	 if (chan->pub.appregType == VHYTEC_TYPE_INPUT_APPREG) {
	   USHORT appreg_value;
	   if  (chan->pub.dbDtype == VDB_DTYPE_INT16 ||
		chan->pub.dbDtype == VDB_DTYPE_UINT16 ||
		chan->pub.dbDtype == VDB_DTYPE_INT) {
	     if(data.debug)
	       vsys_msg_printf(0,"8512: \"%s\", reading from application register %d", mydbChannelName (&chan->pub.chix), chan->pub.addr);
	     // ADC = (CHy8413 *)port->hytec_card;
#ifndef NO_HARDWARE_TEST	 
    status = ((CHy8512*)port->hytec_card)->Hy8512_Read_Reg((int)chan->pub.addr, &appreg_value);
#endif
	     *intptr = appreg_value;
	     if(data.debug)
	       vsys_msg_printf(0,"8512: Value of application register %d: %d", chan->pub.addr, *intptr);
	   } else {
	     if(data.debug)
	       vsys_msg_printf(0,"8512: \"%s\", reading from application register %d, bit # %d", mydbChannelName (&chan->pub.chix), chan->pub.addr, chan->pub.appreg);
#ifndef NO_HARDWARE_TEST
    status = ((CHy8512*)port->hytec_card)->Hy8512_Read_Reg((int)chan->pub.addr, (USHORT*)&hldr);
#endif
	     if(data.debug)
	       vsys_msg_printf(0,"8512: Read value of application register %d: %d", chan->pub.addr, (int)hldr);
	     if (hldr & (1 << (chan->pub.appreg)))
	       *intptr = (int)1;
	     else 
	       *intptr = (int)0;
	     if(data.debug)
	       vsys_msg_printf(0,"8512: Value of application register %d bit %d: %d", chan->pub.addr, chan->pub.appreg, *intptr);
	   }
	 } else {
#ifndef NO_HARDWARE_TEST
	   status = ((CHy8512*)port->hytec_card)->Hy8512_Read((int)chan->pub.addr, &value, 0);
#endif
 vsys_msg_printf(0,"8512: Value of 8512 register %d: %li", chan->pub.addr, value);
 if  (chan->pub.dbDtype == VDB_DTYPE_INT) {
   *longptr = (long)value;
   vsys_msg_printf(0,"8512: Value of 8512 pointer: %li", *longptr);
 } else if (chan->pub.dbDtype == VDB_DTYPE_UINT) {
   *longptr = (unsigned long)value;
   vsys_msg_printf(0,"8512: Value of 8512 pointer: %li", *longptr);
 }  else {
   *fltptr = (float)value;
   vsys_msg_printf(0,"8512: Value of 8512 pointer: %f", *fltptr);
 }	   
	 }
	 
	 if(data.debug) {
	  vsys_msg_printf(0, "8512: \"%s\", code %d", plc->port->name, status);
	   
	}
	break;
      }
      case (VMB_TYPE_8408): {
	USHORT value;
	if(data.debug)
  	vsys_msg_printf(0, "8408, channel %d, vmeslot %d, ipslot %c", chan->pub.addr, port->vmeslot, (char) port->ipslot[0]);
#ifndef NO_HARDWARE_TEST
	status = ((CHy8408*)port->hytec_card)->Hy8408_Read(chan->pub.addr, &value);
#endif
	*fltptr = (float)value;
	if(data.debug) {
	  vsys_msg_printf(0, "8408: \"%s\", code %d", plc->port->name, status);
	   
	}

	break;
      }
      case (VMB_TYPE_8505): {
	if(data.debug)
	  vsys_msg_printf(0, "8505, channel %d, vmeslot %d, ipslot %c", chan->pub.addr, port->vmeslot, (char) port->ipslot[0]);
#ifndef NO_HARDWARE_TEST
	*intptr =((CHy85056*)port->hytec_card)->Hy85056_Read_Bit(chan->pub.addr);
#endif
	if(data.debug){
	  vsys_msg_printf(0, "8505: \"%s\", code %d", plc->port->name, status);
	   
	}

	break; }

      case (VMB_TYPE_8506): {
	if(data.debug)
	  vsys_msg_printf(0, "8506, channel %d, vmeslot %d, ipslot %c", chan->pub.addr, port->vmeslot, (char) port->ipslot[0]);
#ifndef NO_HARDWARE_TEST
	*intptr =((CHy85056*)port->hytec_card)->Hy85056_Read_Bit(chan->pub.addr);
#endif
	break; }
      case (VMB_TYPE_8005): {
	if(data.debug)
	  vsys_msg_printf(0, "8005, channel %d, vmeslot %d, ipslot %c", chan->pub.addr, port->vmeslot, (char) port->ipslot[0]);
#ifndef NO_HARDWARE_TEST
	 *intptr = ((CHy8005*)port->hytec_card)->Hy8005_Read_Bit(port->portnum, chan->pub.addr);
#endif

	break; }
      default: {
	wrongcard(port->type, "int, binary, or float");
return -1;
break;
      }
      }
    }
    return status;
}

/*
 *********************************************************************
 */

static int mymbTransact (Plc* plc, 
			 unsigned requestLen,
			 unsigned expectedResponseLen) {

  Port* port = plc->port;
  VUINT8* response = port->response;
  int function = port->request[0];
  int status;
  int ret_slave = 0;

  if (data.control_design) {
    /* 4 and 2 are not used in Control Design PLCs. */
    if (port->request[0] == 4) {  
/*       vsys_msg_printf(0, "changing function %d\n", port->request[0]); */
      port->request[0] = function = 3; 
    }
    
    if (port->request[0] == 2) { 
/*       vsys_msg_printf(0, "changing function %d\n", port->request[0]);    */
      port->request[0] = function = 3;    
    }   
  }

#if vmbCONF_LOG_MAX_TRANSACT_TIME
  vtime t1, t2, t3;
  unsigned diff;
#endif

#if vmbCONF_LOG_MAX_TRANSACT_TIME
  if (port->maxChannel != NULL)
    vsys_get_time (&t1);
#endif

#if vmbCONF_TRACE_TRANSACTION
  traceTransactRequest (plc, requestLen, expectedResponseLen);
#endif

  //  status = vmbTransact (port->vp, plc->unit, requestLen, expectedResponseLen, &ret_slave);

/*   vsys_msg_printf(0, "Ret slave %d", ret_slave); */

#if vmbCONF_TRACE_TRANSACTION
  traceTransactReply (plc, status);
#endif 

#if vmbCONF_LOG_MAX_TRANSACT_TIME
  if (port->maxChannel != NULL) {
    vsys_get_time (&t2);
    timSUB (t2, t1, t3);
    diff = t3.t_sec * 1000 + t3.t_nsec / 1000000;
    if (diff > 100) {
      VARC_LOCK ();
      if (vlogc_log_channel (data.varc.log, &t2,
			     port->maxChannel, &diff) != 1)
	fatalVARC ();
      VARC_UNLOCK ();
    }
  }
#endif

  if (status != VMB_OK)
    return mberrorTRANSACT (port->vp);

  if (*response != function) {
    vsys_msg_printf(0, "function %d response %d\n", function, *response);  
    if (*response == 0x80 + function) {			
      int hldr = 0;
      VUINT8* request = port->request;
      hldr = atoi(VERR__FILE__LINE__);
      return mberrorEXCEPTION (response[1], hldr, request);
    }							
    return mberrorPROTOCOL (port);			   
  }
  
/*   vsys_msg_printf(0, "unit %d slave %d", plc->unit, ret_slave); */
  
/*  if (port->type != VMB_TYPE_TCP && port->type != VMB_TYPE_USER) {
    if (plc->unit != ret_slave && ret_slave != 0) {
      vsys_msg_printf(0, "Error, sent message for station %d, got reply from station %d", plc->unit, ret_slave);
      return mberrorPROTOCOL (port);			    
    }
  }
*/
  return 0;
}

/*
 *********************************************************************
 */

static int mymbBget (Plc* plc, VUINT8* buffer, Channel* chan,
		     unsigned bitCount, int function) {

  Port* port = plc->port;
  unsigned short int val = 0;
  int status = 0;
 
  if(data.debug)
    vsys_msg_printf(0, "Getting binary value for %s", mydbChannelName (&chan->pub.chix));

  status = hytecTransact (plc, (void*) buffer, chan, READ);
  if (status != 0) {
    hytec_error(status);
    return -1;
  }
    if(data.debug)
     vsys_msg_printf(0, "Got value: %d", (int)*buffer);

  return 0;
  }

/*
 *********************************************************************
 */

static int mymbI16get (Plc* plc, VUINT16* buffer, Channel* chan,
		       unsigned numRegisters, int function) {
  
  Port* port = plc->port;
  int status;
  
   if(data.debug)
    vsys_msg_printf(0, "Getting int16 value for %s", mydbChannelName (&chan->pub.chix));
  status = hytecTransact (plc, (void*) buffer, chan, READ);
  if (status != 0) {
    hytec_error(status);
    return -1;
  }
if(data.debug)
     vsys_msg_printf(0, "Got value: %d", (int)*buffer);

  return 0;
}


static int mymbStringget (Plc* plc, char* buffer, Channel *chan,
			  unsigned numRegisters, int function) {

  Port* port = plc->port;
  VUINT16* holder;
  int numBytes = 0;
  int numTotalRegisters = 0;
  int numTimes = 0;
  int curRegister = 0;
  int x = 0;
  int stringpos = 0;
  //  int initaddr = addr;
  int stringsize = 5000;
  int status;
 if(data.debug)
    vsys_msg_printf(0, "Getting string value for %s", mydbChannelName (&chan->pub.chix));
  switch(port->type) {
  case (VMB_TYPE_85156): {
    //  COM = (CHy85156 *)port->hytec_card;
    status = ((CHy85156*)port->hytec_card)->Hy85156_Read(chan->pub.addr, (UCHAR*)buffer);
    if(data.debug)
    vsys_msg_printf(0, "status %d\n", status);
    break; }
  default: {
    wrongcard(port->type, "string");
  }
  }

 if (status != 0) {
    hytec_error(status);
    return -1;
  }
if(data.debug)
      vsys_msg_printf(0, "got value: %s", buffer);

  /*  holder = calloc(60, sizeof(VUINT16));
  
  while (1) {
    unsigned n = numRegisters > 60 ? 60 : numRegisters;
    VUINT8* response = port->response + 1;
    VUINT8* request = port->request;
    unsigned i;

    *request++ = function;
    *request++ = (addr >> 8) & 0xff;
    *request++ = addr & 0xff;
    *request++ = 0;
    *request++ = n;

    if (mymbTransact (plc, 5, n * 2 + 2)) return -1;

    
    if (*response++ != n * 2) return mberrorPROTOCOL (port);   

#ifdef vmbARCH_BIG_ENDIAN
    memcpy (holder, response, n * 2);
    holder += n;
#else
    for (i = 0; i < n; i++) {

      numBytes = (response[0] << 8) + response[1];
      *holder++ = (response[0] << 8) + response[1];
      response += 2;
    }
#endif

    if (n == numRegisters)
      break;
    numRegisters -= n;
    addr += n;
  }
  
  if (numBytes == 0) {
    buffer[stringpos] = '\0';
    return 666;
  }

  numRegisters = (int)ceil(numBytes / 2);
  vsys_msg_printf(0, "Numreg %d numbytes %d, now getting string\n",numRegisters, numBytes);  
  addr = 3332;

    while (1) {
      unsigned n = numRegisters > 60 ? 60 : numRegisters;
      VUINT8* response = port->response + 1;
      VUINT8* request = port->request;
      unsigned i;
      
      *request++ = function;
      *request++ = (addr >> 8) & 0xff;
      *request++ = addr & 0xff;
      *request++ = 0;
      *request++ = n;
   

      if (mymbTransact (plc, 5, n * 2 + 2)) return -1;
      
      
      if (*response++ != n * 2) return mberrorPROTOCOL (port);   
      
      if (stringpos + ((int)n*2) > stringsize - 100) {
	stringsize += (5000 + (n*2));
	buffer = fatalRealloc (buffer, sizeof (char) * stringsize);
      }

      for (i = 0; i < n * 2; i++) {
	buffer[stringpos] = response[i];
	stringpos++;
      } 
      
      if (n == numRegisters) {
	vsys_msg_printf(0, "Break!\n");
	break;
      } else 
	vsys_msg_printf(0, "Go around again!\n");
      numRegisters -= n;
      addr += n;
    }
    
    buffer[stringpos] = '\0';

    vsys_msg_printf(0, "Starting to clear buffer %d", initaddr);
    mymbI16put (plc, initaddr, 24929);
    vsys_msg_printf(0, "Done clearing buffer");
*/
    return 0;
}


static int mymbSDIget (Plc* plc, float* value, unsigned addr,
			  unsigned numRegisters, int function) {

  Port* port = plc->port;
  int numBytes = 0;
  int numTotalRegisters = 0;
  int numTimes = 0;
  int curRegister = 0;
  int x = 0;
  int stringpos = 0;
  int initaddr = addr;
  char buffer[100];
  int numbytes = 0;

  //Disabled
  vsys_msg_printf(0, "SDI type not supported");
  return 0;

  while (1) {
    unsigned n = numRegisters > 60 ? 60 : numRegisters;
    VUINT8* response = port->response + 1;
    VUINT8* request = port->request;
    int i;

    *request++ = function;
    *request++ = (addr >> 8) & 0xff;
    *request++ = addr & 0xff;
    *request++ = 0;
    *request++ = n;

     vsys_msg_printf(0, "Start of SDI"); 

    if (mymbTransact (plc, 5, 12)) return -1;

     vsys_msg_printf(0, "After SDI send"); 

     for (i = 0; i < 7; i++) { 
       vsys_msg_printf(0, "Total Reponse %d\n", response[i]); 
     } 

    numbytes = *response++; /* Number of bytes returned */
    vsys_msg_printf(0, "Number of bytes returned %d\n", numbytes);     
    if (numbytes != 10) return mberrorPROTOCOL (port);    

      vsys_msg_printf(0, "ASCII ID %c", *response++);  

    /* And now the data should start */
    for (i = 0; i < (numbytes); i++) {
       	vsys_msg_printf(0, "N * 2 is %d, the char is %c\n", n * 2, (char)response[i]); 
      if (response[i] == 13) /* If you see the carriage return, break */
	break;
      buffer[stringpos] = response[i];
      stringpos++;
    }
    
    if (n == numRegisters)
      break;
    numRegisters -= n;
    addr += n;
  }
  
  buffer[stringpos] = '\0';
  *value = (float)atof(buffer);
   vsys_msg_printf(0, "The string was %s, value %f\n", buffer, *value);   
   vsys_msg_printf(0, "The string was %s\n", buffer);  
  return 0;
}

/*
 *********************************************************************
 */

static int mymbI32get (Plc* plc, VUINT32* buffer32, Channel* chan,
		       unsigned numChannels, int function) {

  Port* port = plc->port;
  VUINT8* buffer8 = (VUINT8*) buffer32;
  int bit1 = 0x1B, bit2 = 0x58;
  int bit3 = 0x27, bit4 = 0x88;
  int status;
    if(data.debug)
      vsys_msg_printf(0, "Getting long int value for %s", mydbChannelName (&chan->pub.chix));

   status = hytecTransact (plc, (void*) buffer32, chan, READ);
   if (status != 0) {
     hytec_error(status);
     return -1;
   }
   if(data.debug)
     vsys_msg_printf(0, "Got value: %li", (long)*buffer32);

  return 0;
}

static int mymbI32get_float (Plc* plc, float* buffer32, Channel *chan,
		       unsigned numChannels, int function) {

  Port* port = plc->port;
  VUINT8* buffer8 = (VUINT8*) buffer32;
  float floaty, *fltptr;
  unsigned long longy;
  int bit1 = 0x1B, bit2 = 0x58;
  int bit3 = 0x27, bit4 = 0x88;
  char hldr[1000];
  int dtype, status = 0;
  char charfloat[4];
/* READ_AI Offsets for Reading Calibration Values */
#define ADC_A_ZERO_CAL  16  /* ADC A - Zero Raw Calibration */
#define ADC_A_POS_CAL   17  /* ADC A - Pos  Raw Calibration */
#define ADC_A_NEG_CAL   18  /* ADC A - Neg  Raw Calibration */
#define ADC_B_ZERO_CAL  19  /* ADC B - Zero Raw Calibration */
#define ADC_B_POS_CAL   20  /* ADC B - Pos  Raw Calibration */
#define ADC_B_NEG_CAL   21  /* ADC B - Neg  Raw Calibration */
    short           iChannelCount;
    unsigned long   ADC_Value;

	/* PT-100 / Cold Junction Temperature Variables */
    double			Vpt100;		/* PT100 Voltage */
    double			Ipt100;		/* PT100 Current */
    double			Rpt100;		/* PT100 Resistance */
    double			Tpt100;		/* PT100 Temperature */
    double           Rref = 100.0; /* Reference/Vish Resistor is 100R */
    double           A = 0.00390802;
    double           B = -0.000000580195;
    double Gain;
    double           Voltage;
    double           Temperature;
    double           TV;
 
    if(data.debug)
      vsys_msg_printf(0, "Getting float value for %s", mydbChannelName (&chan->pub.chix));
    
    status = hytecTransact (plc, (void*) buffer32, chan, READ);
    
    // *buffer32 = (float)longy;

    if (status != 0) {
      hytec_error(status);
      return -1;
    }
    if(data.debug)
      vsys_msg_printf(0, "got value: %f", (float)*buffer32);    

  return 0;
}


/*
 *********************************************************************
 */

static int mymbI64get (Plc* plc, VUINT64* buffer, Channel* chan,
		       unsigned numChannels, int function) {

 int status;
  Port* port = plc->port;

 if(data.debug)
      vsys_msg_printf(0, "Getting 64-bit value for %s", mydbChannelName (&chan->pub.chix));

  status = hytecTransact (plc, (void*) buffer, chan, READ);
  if (status != 0) {
    hytec_error(status);
    return -1;
  }

  return 0;
}

/*
 *********************************************************************
 */

static int mymbBput (Plc* plc, Channel *chan, unsigned value) {

  Port* port = plc->port;
  VUINT8* request = port->request;
  VUINT8* response = port->response; 
  int status;
 if(data.debug)
      vsys_msg_printf(0, "Putting binary value %d for %s", value, mydbChannelName (&chan->pub.chix));

  status = hytecTransact (plc, (void*) &value, chan, WRITE);
  if (status != 0) {
    hytec_error(status);
    return -1;
  }

  return 0;
}

/*
 *********************************************************************
 */

static int mymbI16put (Plc* plc, Channel *chan, unsigned value) {

  Port* port = plc->port;
  VUINT8* request = port->request;
  VUINT8* response = port->response;
  int num = 1;
  int bytecount = 2;
  int status;
if(data.debug)
      vsys_msg_printf(0, "Putting int16 value %d for %s", value, mydbChannelName (&chan->pub.chix));

status = hytecTransact (plc, (void*) &value, chan, WRITE);
  if (status != 0) {
    hytec_error(status);
    return -1;
  }

  return 0;
}


static int mymbStringput (Plc* plc, Channel *chan, char *value) {
 Port* port = plc->port;
int status = 0;

if(data.debug)
      vsys_msg_printf(0, "Putting string value %s for %s", value, mydbChannelName (&chan->pub.chix));

switch(port->type) {
  case (VMB_TYPE_85156): {
    //  COM = (CHy85156 *)port->hytec_card;
    status = ((CHy85156*)port->hytec_card)->Hy85156_Write(chan->pub.addr, (UCHAR*)value, strlen(value));
    vsys_msg_printf(0, "status %d", status);
    break; }
  default: {
    wrongcard(port->type, "string");
  }
  }
 if (status != 0) {
    hytec_error(status);
    return -1;
  }
}

static int mymbI16Holdingput (Plc* plc, unsigned addr, unsigned value) {

  Port* port = plc->port;
  VUINT8* request = port->request;
  VUINT8* response = port->response;
  int num = 1;
  int bytecount = 2;

   //Disabled
  vsys_msg_printf(0, "Holding Register type not supported");
  return 0;

  *request++ = 16;
  *request++ = (addr >> 8) & 0xff;
  *request++ = addr & 0xff;
  *request++ = (num >> 8) & 0xff;
  *request++ = num & 0xff;
  *request++ = bytecount;
  *request++ = (value >> 8) & 0xff;
  *request++ = value & 0xff;
  //  if (mymbTransact (plc, 8, 5)) return -1;
  /* if (memcmp (port->request + 1, response + 1, 4) != 0)
    return mberrorPROTOCOL (port); */	/*NYCV*/
  return 0;
}

/*
 *********************************************************************
 */

static int mymbI32put (Plc* plc, Channel *chan, long value) { 

  int status = 0;
  Port* port = plc->port;
  VUINT8* request = port->request;
  VUINT8* response = port->response;
  char val_array[4];
if(data.debug)
      vsys_msg_printf(0, "Putting 32-bit integer value %li for %s", value, mydbChannelName (&chan->pub.chix));
 
 status = hytecTransact (plc, (void*) &value, chan, WRITE);
   if (status != 0) {
    hytec_error(status);
    return -1;
  }

  return 0;
}


/* CML added the following function 6-20-05 */

static int mymbI32put_float (Plc* plc, Channel *chan, float value) { 

  Port* port = plc->port;
  VUINT8* request = port->request;
  VUINT8* response = port->response;
  char charfloat[4];
  int status;
if(data.debug)
      vsys_msg_printf(0, "Putting float value %f for %s", value, mydbChannelName (&chan->pub.chix));

  status = hytecTransact (plc, (void*) &value, chan, WRITE);
  if (status != 0) {
    hytec_error(status);
    return -1;
  }

  return 0;
}

/*
 *********************************************************************
 */

static int mymbI64put (Plc* plc, Channel *chan, VUINT64 value) {

  Port* port = plc->port;
  VUINT8* request = port->request;
  VUINT8* response = port->response;
  union { VUINT8 b[8]; VUINT64 i; } u;
  int status = 0;

  if(data.debug)
    vsys_msg_printf(0, "Putting 64-bit value %li for %s", value, mydbChannelName (&chan->pub.chix));
  
  status = hytecTransact (plc, (void*) &value, chan, WRITE);
  if (status != 0) {
    hytec_error(status);
    return -1;
  }
 return 0;
}

/*
 *********************************************************************
 */

static void disconnectPort (Port* port) {
  Vmbport* vp = port->vp;
  logMsg ("disconnect");
  //  if (vmbDisconnect (vp) != VMB_OK)
    (void) mberrorDISCONNECT (vp);
}

/*
 *********************************************************************
 */

static void initPortIO (void) {

  Port* port;

  for (port = data.portList; port != NULL; port = port->next) {
    if (port->numInputChannels != 0 || port->numOutputChannels != 0) {
      Vmbport* vp = vmbAlloc (&port->request, &port->requestBufferSize,
			      &port->response, &port->responseBufferSize);
      if (vp == NULL) fatalINTERR ();

#define M(f,F) vmbSet (vp, VMB_O_##F, (void*) port->f) != VMB_OK
      if (vmbSet (vp, VMB_O_DEBUG, (void*) data.debug) != VMB_OK ||
	  M (type, TYPE) ||
	  (port->driver != NULL && M (driver, DRIVER)) ||
	  (port->rtsDelay && M (rtsDelay, RTS)) ||
	  (port->turnaroundDelay && M (turnaroundDelay, TURNAROUND)) ||
	  (port->timeout && M (timeout, TIMEOUT)) ||
	  (port->baud && M (baud, BAUD)) ||
	  (port->tcpPort && M (tcpPort, PORT)) ||
	  (port->stop && M (stop, STOP)) ||
	  (port->parity && M (parity, PARITY)))
	fatalMB (vp);	/*NCV*/
#undef M
      if (port->device == NULL)
	vp->name[0] = 0;
      else
	(void) vsys_strncpy (vp->name, port->device, sizeof vp->name);
      
      port->vp = vp;
      if (data.disconnect)
	disconnectPort (port);
    }
  }
  port = data.portList;
    if (vmbConnect (port, port->device) != VMB_OK){
      vsys_msg_printf(0,"Initialization Error");
    }else {
if(data.debug)
      vsys_msg_printf(0,"Now Initialized");
    }
}

/*
 *********************************************************************
 */

static void initAddrOffsetStructure (PlcAddrOffset* o) {
  unsigned n = sizeof (*o) / sizeof (int);
  int* p = (int*) o;
  unsigned i;
  for (i = 0; i < n; i++)
    p[i] = -1;
}

/*
 *********************************************************************
 */

static void mergeAddrOffsetStructures (PlcAddrOffset* ds,
				       PlcAddrOffset* ss) {
  unsigned n = sizeof (*ds) / sizeof (int);
  int* dp = (int*) ds;
  int* sp = (int*) ss;
  unsigned i;
  for (i = 0; i < n; i++)
    if (dp[i] == -1)
      dp[i] = sp[i] == -1 ? 0 : sp[i];
}

/*
 *********************************************************************
 */

static Plc* fatalMallocPlc (void) {
  Plc* o = (Plc*)fatalMalloc0 (sizeof *o);
  /*   o->class = CLASS_UNDEFINED;*/
  o->endian = ENDIAN_UNDEFINED;
  o->a32 = 2;
  initAddrOffsetStructure (&o->addrOffset);
  return o;
}

/*
 *********************************************************************
 */

static Port* fatalMallocPort(void) {
  Port* o = (Port*)fatalMalloc0 (sizeof *o);
 /*  o->class = CLASS_UNDEFINED; */
  o->endian = ENDIAN_UNDEFINED;
  o->a32 = 2;
  initAddrOffsetStructure (&o->addrOffset);
  return o;
}

/*
 *********************************************************************
 */

#define APPEND_TO_CHANGED_LIST(port, plc, list, offset) do {	\
  if (! list->o.changed [offset]) {				\
    list->o.changed [offset] = 1;				\
    if (list->o.numChanged == 0) {				\
      list->o.first = &list->o.list[offset];			\
      list->o.last = list->o.first;				\
      list->o.numChanged = 1;					\
    }								\
    else {							\
      *list->o.last = &list->o.list[offset];			\
      list->o.last = &list->o.list[offset];			\
      list->o.numChanged++;					\
    }								\
    list->o.list[offset] = NULL;				\
    plc->numChannelsChanged++;					\
  }								\
} while (0)

/*
 *********************************************************************
 * Make sure that vmbARCH_BIG_ENDIAN defined correctly and set the
 * endianness flag.
 *********************************************************************
 */

static void setupEndianness (void) {
  short s = 1;
  bool big = *(char*) &s == 0;
#ifdef vmbARCH_BIG_ENDIAN
  if (! big) fatalINTERR ();
#else
  if (big) fatalINTERR ();
#endif
  data.endian = big ? ENDIAN_BIG : ENDIAN_LITTLE;
}

/*
 *********************************************************************
 */

static char* itoaParity (int parity) {
  char* string = NULL;
  switch (parity) {
#define M(n,s) case n: string = s; break
    M (SL_PARITY_NONE, "NONE");
    M (SL_PARITY_EVEN, "EVEN");
    M (SL_PARITY_ODD, "ODD");
#undef M
  }
  return string;
}

/*
 *********************************************************************
 */

static char* itoaType (int type) {
  char* string = NULL;
  switch (type) {
/* #define M(n,s) case n: string = s; break */
/*     M (VMB_TYPE_TCP, "TCP"); */
/*     M (VMB_TYPE_ASCII, "ASCII"); */
/*     M (VMB_TYPE_RTU, "RTU"); */
/*     M (VMB_TYPE_USER, "USER"); */
/*     M (VMB_TYPE_TCP_RTU, "TCP/RTU"); */
/*     M (VMB_TYPE_TCP_ASCII, "TCP/ASCII"); */
/* #undef M */
  default: fatalINTERR ();
  }
  return string;
}

/*
 *********************************************************************
 * Converts to internal hardware format.
 *
 * FIXME: check return status of intToExt and extToInt
 * FIXME: set IO_ERROR if conversion routine fails
 *********************************************************************
 */

static int extToInt (Channel* chan, Value* value) {

  if (chan->conversionRoutine != NULL) {
#ifdef VAXFLOAT
    int status =
#else
    return
#endif
      chan->conversionRoutine (&data.env, &chan->pub, (VhytecValue*) value);
#ifdef VAXFLOAT
    if (status != VHYTEC_OK) return status;
#endif
  }
  else if (chan->pub.hwDtype == VDB_DTYPE_BINARY) {
    switch (chan->pub.dbDtype) {
    case VDB_DTYPE_INT8: value->binary = value->int8 != 0; break;
    case VDB_DTYPE_UINT8: value->binary = value->uint8 != 0; break;
    case VDB_DTYPE_INT16: value->binary = value->int16 != 0; break;
    case VDB_DTYPE_UINT16: value->binary = value->uint16 != 0; break;
    case VDB_DTYPE_INT: value->binary = value->int32 != 0; break;
    case VDB_DTYPE_UINT: value->binary = value->uint32 != 0; break;
    case VDB_DTYPE_FLOAT: value->binary = value->float32 != 0; break;
    case VDB_DTYPE_DOUBLE: value->binary = value->float64 != 0; break;
    default: fatalNOTIMPL ();	/*NCV*/
    }				/*NCV*/
  }
  else {
#define M(field)							\
  switch (chan->pub.hwDtype) {						\
  case VDB_DTYPE_BINARY: value->binary = (Binary) value->field; break;	\
  case VDB_DTYPE_INT16: value->int16 = (VINT16) value->field; break;	\
  case VDB_DTYPE_UINT16: value->uint16 = (VUINT16) value->field; break;	\
  case VDB_DTYPE_INT: value->int32 = (VINT) value->field; break;	\
  case VDB_DTYPE_UINT: value->uint32 = (VUINT) value->field; break;	\
  case VDB_DTYPE_FLOAT: value->float32 = (float) value->field; break;	\
  case VDB_DTYPE_DOUBLE: value->float64 = (double) value->field; break;	\
  default: fatalNOTIMPL ();						\
  }

    switch (chan->pub.dbDtype) {
    case VDB_DTYPE_BINARY: M (binary); break;
    case VDB_DTYPE_INT8: M (int8); break;
    case VDB_DTYPE_UINT8: M (uint8); break;
    case VDB_DTYPE_INT16: M (int16); break;
    case VDB_DTYPE_UINT16: M (uint16); break;
    case VDB_DTYPE_INT: M (int32); break;
    case VDB_DTYPE_UINT: M (uint32); break;
    case VDB_DTYPE_FLOAT: M (float32); break;
    case VDB_DTYPE_DOUBLE: M (float64); break;
    default: fatalNOTIMPL ();	/*NCV*/
    }
#undef M
  }

#ifdef VAXFLOAT
  /* FIXME: check return status and do something on failure */
  if (chan->pub.hwDtype == VDB_DTYPE_FLOAT)
    vax_to_ieee_float (&value->float32, &value->float32);
  else if (chan->pub.hwDtype == VDB_DTYPE_DOUBLE)
    vax_to_ieee_double (&value->float64, &value->float64);
#endif

  return VHYTEC_OK;
}

/*
 *********************************************************************
 * Converts from internal hardware format
 *
 * FIXME: check return status of intToExt and extToInt
 * FIXME: set IO_ERROR if conversion routine fails
 *********************************************************************
 */

static int intToExt (Channel* chan, Value* value) {

#ifdef VAXFLOAT
  /* FIXME: check return status and do something on failure */
  if (chan->pub.hwDtype == VDB_DTYPE_FLOAT)
    ieee_to_vax_float (&value->float32, &value->float32);
  else if (chan->pub.hwDtype == VDB_DTYPE_DOUBLE)
    ieee_to_vax_double (&value->float64, &value->float64);
#endif

  if (chan->conversionRoutine != NULL)
    return chan->conversionRoutine (&data.env, &chan->pub,
				    (VhytecValue*) value);

#define M(field, cast)							\
  switch (chan->pub.hwDtype) {						\
  case VDB_DTYPE_BINARY: value->field = (cast) value->binary; break;	\
  case VDB_DTYPE_INT16: value->field = (cast) value->int16; break;	\
  case VDB_DTYPE_UINT16: value->field = (cast) value->uint16; break;	\
  case VDB_DTYPE_INT: value->field = (cast) value->int32; break;	\
  case VDB_DTYPE_UINT: value->field = (cast) value->uint32; break;	\
  case VDB_DTYPE_FLOAT: value->field = (cast) value->float32; break;	\
  case VDB_DTYPE_DOUBLE: value->field = (cast) value->float64; break;	\
  default: fatalNOTIMPL ();						\
  }

  switch (chan->pub.dbDtype) {
  case VDB_DTYPE_BINARY:
    switch (chan->pub.hwDtype) {
    case VDB_DTYPE_INT16: value->binary = value->int16 != 0; break;
    case VDB_DTYPE_UINT16: value->binary = value->uint16 != 0; break;
    case VDB_DTYPE_INT: value->binary = value->int32 != 0; break;	/*NCV*/
    case VDB_DTYPE_UINT: value->binary = value->uint32 != 0; break;	/*NCV*/
    case VDB_DTYPE_FLOAT: value->binary = value->float32 != 0; break;
    case VDB_DTYPE_DOUBLE: value->binary = value->float64 != 0; break;
   default: fatalNOTIMPL ();	/*NCV*/
    }				/*NCV*/
    break;
  case VDB_DTYPE_INT8: M (int8, VINT8); break;
  case VDB_DTYPE_UINT8: M (uint8, VUINT8); break;
  case VDB_DTYPE_INT16: M (int16, VINT16); break;
  case VDB_DTYPE_UINT16: M (uint16, VUINT16); break;
  case VDB_DTYPE_INT: M (int32, VINT); break;
  case VDB_DTYPE_UINT: M (uint32, VUINT); break;
  case VDB_DTYPE_FLOAT: M (float32, float); break;
  case VDB_DTYPE_DOUBLE: M (float64, double); break;
  default: fatalNOTIMPL ();	/*NCV*/
  }

#undef M

  return VHYTEC_OK;
}

/*
 *********************************************************************
 */

static Plc* findPlc (char* name) {
  Plc* plc;
  for (plc = data.plcList; plc != NULL; plc = plc->next) {
    if (vsys_stricmp (name, plc->name) == 0)
      return plc;
  }
  return NULL;
}

/*
 *********************************************************************
 */

static Port* findPort (char* name) {
  Port* port;
  for (port = data.portList; port != NULL; port = port->next) {
    if (vsys_stricmp (name, port->name) == 0)
      return port;
  }
  return NULL;
}

/*
 *********************************************************************
 * Insert new PLC into sorted list
 *********************************************************************
 */

static void insertPlc (Plc* newplc) {

  Plc* plc;
  Plc* prev = NULL;

  for (plc = data.plcList; plc != NULL; plc = plc->next) {
    if (vsys_stricmp (newplc->name, plc->name) < 0)
      break;
    prev = plc;
  }

  if (prev == NULL) {
    newplc->next = data.plcList;
    data.plcList = newplc;
  }
  else {
    newplc->next = prev->next;
    prev->next = newplc;
  }
}

/*
 *********************************************************************
 * Insert new port into sorted list.
 *********************************************************************
 */

static void insertPort (Port* newport) {

  Port* port;
  Port* prev = NULL;

  for (port = data.portList; port != NULL; port = port->next) {
    if (vsys_stricmp (newport->name, port->name) < 0)
      break;
    prev = port;
  }

  if (prev == NULL) {
    newport->next = data.portList;
    data.portList = newport;
  }
  else {
    newport->next = prev->next;
    prev->next = newport;
  }
}

/*
 *********************************************************************
 * Get initial value of an output channel.
 *********************************************************************
 */

static void getInitialValue (AddrList* list, unsigned i) {

  Channel* chan = &list->chan[i];
  vista_chix* chix = &chan->pub.chix;
  int dtype = chan->pub.dbDtype;
  Value value;

  if (dtype == VDB_DTYPE_BINARY)
    dtype = VDB_DTYPE_INT;

  if (vdbc_vget (chix, &value, NULL, dtype, VDB_INTERNAL) != VDB_ERR_OK)
    vscannerFatalVdbChixError (chix); /*NCV*/

  if (chan->pub.dbDtype == VDB_DTYPE_BINARY)
    value.binary = value.int32;

  if (chan->pub.hwDtype != chan->pub.dbDtype ||
#ifdef VAXFLOAT
      chan->pub.hwDtype == VDB_DTYPE_FLOAT ||
      chan->pub.hwDtype == VDB_DTYPE_DOUBLE ||
#endif
      chan->conversionRoutine != NULL)
    (void) extToInt (chan, &value);

  switch (chan->pub.hwDtype) {
  case VDB_DTYPE_BINARY: list->value.binary[i] = value.binary; break;
  case VDB_DTYPE_INT16:
  case VDB_DTYPE_UINT16: list->value.int16[i] = value.int16; break;
  case VDB_DTYPE_INT:
  case VDB_DTYPE_UINT: list->value.int32[i] = value.int32; break;	/*NCV*/
  case VDB_DTYPE_FLOAT: list->value.float32[i] = value.float32; break;
  case VDB_DTYPE_DOUBLE: list->value.float64[i] = value.float64; break;
  default: fatalNOTIMPL ();	/*NCV*/
  }
}

/*
 *********************************************************************
 */

static void initWdChannel (void) {
  if (data.wd.channelName != NULL) {
    vista_chix* chix = &data.wd.chix;
    if (vdbc_channel_index (data.wd.channelName, chix) != VDB_ERR_OK)
      fatalNOWDCHAN (); /*NCV*/
    if (vscannerDbChannelType (chix) != VDB_TYPE_INTEGER)
      fatalINVWDTYPE ();
    data.wd.valueMin = 1; /* fixme: eventually may change this to LD & UD */
    data.wd.valueMax = 100;
    data.wd.value = data.wd.valueMin;
    data.wd.valid = 1;
  }
}

/*
 *********************************************************************
 */

static int reconfig (char *name) {
  Port* port;
  Plc *plc;
  Plc *foundplc = NULL;
  int status = 0;
  int finalstat = 0;
  for (plc = data.plcList; plc != NULL; plc = plc->next) {
    if (vsys_stricmp(plc->name, name) == 0) {
      foundplc = plc;
      break;
    }
  }

  if (foundplc != NULL)
    plc = foundplc;
  else 
    return -1;

  switch(plc->port->type) { 
    /************************************************************/
    /* Hytec 85156 (COM1)                                       */
    /************************************************************/
  case(VMB_TYPE_85156): {
    plc->port->hytec_card = (CHy85156 *)new CHy85156;
    int i = 0;
    //  COMPRT = (CHy85156 *)fatalMalloc (sizeof(CHy85156));
    if(data.debug)
      vsys_msg_printf(0,"Initializing 85156 \"%s\"", plc->port->name);
    status = 0;
    if (plc->port->ipslot != NULL) {
      char ipslot;
      ipslot = plc->port->ipslot[0];
      if(data.debug)
	vsys_msg_printf(0, "vmeslot %d, IPslot %c, vector %d, delay %d, halfduplex %d, delay485 %d", plc->port->vmeslot, ipslot, plc->port->vectornum, plc->port->delay, plc->port->halfduplex, plc->port->delay485);
      
      status = ((CHy85156 *)plc->port->hytec_card)->Hy85156_Configure(plc->port->vmeslot, ipslot, plc->port->vectornum, plc->port->delay, plc->port->halfduplex, plc->port->delay485);
      if (status != 0) {
	/* 	  plc->port->hytec_card = (void *)&COMPRT; */
	/* 	else { */
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "85156: \"%s\", code %d", plc->port->name, status);
	hytec_error(status);
	(void) fatalCARDINIT(buff);
      }
    }      else
      fatalNOSLOT();
    //  for(i=0; i<8; i++) {	
    status = ((CHy85156 *)plc->port->hytec_card)->Hy85156_ComsSetup(plc->port->portnum, plc->port->baud, plc->port->parity, plc->port->stop, plc->port->databits, plc->port->flowctl);
    if (status != 0) {
      char buff[2000];
      plc->port->hytec_card = NULL;
      sprintf(buff, "85156: \"%s\", port %d, code %d", plc->port->name, i, status);	  
      hytec_error(status);
      (void) fatalPORTINIT(buff);
    }
    //  }
    break;
  }
    /************************************************************/
    /* Hytec 8505 (DIO)                                       */
    /************************************************************/
  case(VMB_TYPE_8505): {
    plc->port->hytec_card = (CHy85056 *)new CHy85056;
    int i = 0;
    // DIO = (CHy85056 *)fatalMalloc (sizeof(CHy85056));
    if(data.debug)
      vsys_msg_printf(0,"Initializing 8505 \"%s\"", plc->port->name);
    status = 0;
    if (plc->port->ipslot != NULL) {
      char ipslot;
      ipslot = plc->port->ipslot[0];
      if(data.debug)
	vsys_msg_printf(0, "ipslot %c, %x, finally %d", ipslot, ipslot, ipslot - 0x41);
      if(data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c,debrate %d, debmask %d, scanrate %d, dir %d, vectornum %d, intmask %d, pwidth %d, pmask %d, clock %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->debrate, 
			plc->port->debmask, 
			plc->port->scanrate, 
			plc->port->dir, 
			plc->port->vectornum, 
			plc->port->intmask, 
			plc->port->pwidth,
			plc->port->pmask,
			plc->port->clock);
      status = ((CHy85056 *)plc->port->hytec_card)->Hy85056_Configure(plc->port->vmeslot, ipslot, plc->port->debrate, plc->port->debmask, 
								      plc->port->scanrate, plc->port->dir, plc->port->vectornum, plc->port->intmask, 
								      plc->port->pwidth, plc->port->pmask, plc->port->clock);	
    }      else
      fatalNOSLOT();
    if(data.debug)
      vsys_msg_printf(0, "status: %d", status);
    if (status != 0) {
      /* 	plc->port->hytec_card = (void *)&DIO; */
      /*       else { */
      char buff[2000];
      plc->port->hytec_card = NULL;
      sprintf(buff, "8505: \"%s\", code %d", plc->port->name, status);
      hytec_error(status);
      (void) fatalCARDINIT(buff);
    }
    break;
  }
    
    /************************************************************/
    /* Hytec 8506 (DIO)                                       */
    /************************************************************/
  case(VMB_TYPE_8506): {
    plc->port->hytec_card = (CHy85056 *)new CHy85056;
    int i = 0;
    // DIO = (CHy85056 *)fatalMalloc (sizeof(CHy85056));
    if(data.debug)
      vsys_msg_printf(0,"Initializing 8506 \"%s\"", plc->port->name);
    status = 0;
    if (plc->port->ipslot != NULL) {
      char ipslot;
      ipslot = plc->port->ipslot[0];
      if(data.debug)
	vsys_msg_printf(0, "ipslot %c, %x, finally %d", ipslot, ipslot, ipslot - 0x41);
      if(data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c,debrate %d, debmask %d, scanrate %d, dir %d, vectornum %d, intmask %d, pwidth %d, pmask %d, clock %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->debrate, 
			plc->port->debmask, 
			plc->port->scanrate, 
			plc->port->dir, 
			plc->port->vectornum, 
			plc->port->intmask, 
			plc->port->pwidth,
			plc->port->pmask,
			plc->port->clock);
      status = ((CHy85056 *)plc->port->hytec_card)->Hy85056_Configure(plc->port->vmeslot, ipslot, plc->port->portnum, plc->port->debrate, plc->port->debmask, 
								      plc->port->scanrate, plc->port->dir, plc->port->vectornum, plc->port->intmask, 
								      plc->port->pwidth, plc->port->pmask, plc->port->clock);	
    }      else
      fatalNOSLOT();
    if(data.debug)
      vsys_msg_printf(0, "status: %d", status);
    if (status != 0) {
      /* 	plc->port->hytec_card = (void *)&DIO; */
      /*       else { */
      char buff[2000];
      plc->port->hytec_card = NULL;
      sprintf(buff, "8506: \"%s\", code %d", plc->port->name, status);
      hytec_error(status);
      (void) fatalCARDINIT(buff);
    }
    break;
  }
    
    /************************************************************/
    /* Hytec 8601 (Motor)                                       */
    /************************************************************/
  case(VMB_TYPE_8601): {
    plc->port->hytec_card = (CHy8601 *)new CHy8601;
    status = 0;
    int i = 0;
    int axis = 0;
    // motor1 = (CHy8601 *)fatalMalloc (sizeof(CHy8601));
    if(data.debug)
      vsys_msg_printf(0,"Initializing 8601 \"%s\"", plc->port->name);
    if (plc->port->ipslot != NULL) {
      char ipslot;
      int cardstat = 0;
      ipslot = plc->port->ipslot[0];
      if(data.debug)
	vsys_msg_printf(0, "ipslot %c\n", ipslot);
      //	for(i=0; i<4; i++)  {                  
      //initialise 4 motor channels
      
      //  axis = i;
      status = ((CHy8601 *)plc->port->hytec_card)->Hy8601_Configure(plc->port->vmeslot, ipslot);
      if (status != 0) {
	cardstat = 1;
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8601: \"%s\", code %d", plc->port->name, i, status);
	hytec_error(status);
	(void) fatalCARDINIT(buff);
      }
      
      status = ((CHy8601 *)plc->port->hytec_card)->Hy8601_MotorSetup(plc->port->axis, plc->port->startspeed, plc->port->maxspeed, plc->port->ramp);
      if (status != 0) {
	char buff[2000];
	plc->port->hytec_card = NULL;
	sprintf(buff, "8601: \"%s\", axis %d, code %d", plc->port->name, i, status);
	hytec_error(status);
	(void) mberrorAXIS(buff);
	//  }
      }
      /* 	if (cardstat != 0) */
      /* 	  plc->port->hytec_card = (void *)&motor1; */
      
    }      else 
      fatalNOSLOT();
    break;
  }
    /************************************************************/
    /* Hytec 8402 (DAC)                                       */
    /************************************************************/
  case(VMB_TYPE_8402): {
    plc->port->hytec_card = (CHy8402 *)new CHy8402;
    
    status = 0;
    if (data.debug)
      vsys_msg_printf(0,"Initializing 8402 \"%s\"", plc->port->name);
    if (plc->port->ipslot != NULL) {
      char ipslot;
      ipslot = plc->port->ipslot[0];
      if (data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c, vectornum %d, doram %d, clocksource %d, clockrate %d, inhibit %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->vectornum, 
			plc->port->doram, 
			plc->port->clocksource, 
			plc->port->clockrate, 
			plc->port->inhibit);
      
      status = ((CHy8402 *)plc->port->hytec_card)->Hy8402_Configure(plc->port->vmeslot, 
								    plc->port->ipslot[0], 
								    plc->port->vectornum, 
								    plc->port->doram, 
								    plc->port->clocksource, 
								    plc->port->clockrate, 
								    plc->port->inhibit); 
      
      
      /* 	status = (DAC).Hy8402_Configure(vmeslot, dacIPslot1, vectornum2, doram1, clockSource1, dacClockRate1, inhibit1); */
      
    }      else
      fatalNOSLOT();
    if (data.debug)
      vsys_msg_printf(0, "status: %d", status);
    if (status != 0) {
      /* 	plc->port->hytec_card = (void *)&DAC; */
      /* 	vsys_msg_printf(0, "8402: %x\n", &DAC); */
      /*       } else { */
      char buff[2000];
      plc->port->hytec_card = NULL;
      sprintf(buff, "8402: \"%s\", code %d", plc->port->name, status);
      hytec_error(status);
      (void) fatalCARDINIT(buff);
    }
    break;
  }
    /************************************************************/
    /* Hytec 8403 (DAC)                                       */
    /************************************************************/
  case(VMB_TYPE_8403): {
    //CHy8403 *DAC03 = new CHy8403;
    plc->port->hytec_card = (CHy8403 *)new CHy8403;
    
    status = 0;   
    if (data.debug)
      vsys_msg_printf(0,"Initializing 8403 \"%s\"", plc->port->name);
    
    //  DAC03 = (CHy8403 *)fatalMalloc (sizeof(CHy8403));
    
    setup_dac((CHy8403 *)plc->port->hytec_card);
    
    if (plc->port->ipslot) {
      char ipslot;
      ipslot = plc->port->ipslot[0];
      if (data.debug)
	vsys_msg_printf(0, "vmeslot %d, ipslot %c, inputtype %d, gain %d", plc->port->vmeslot, plc->port->ipslot[0], plc->port->inputtype, plc->port->gain );	
      status = ((CHy8403 *)plc->port->hytec_card)->Hy8403_Configure(plc->port->vmeslot, plc->port->ipslot[0], plc->port->inputtype, plc->port->gain);
    } else
      fatalNOSLOT();
    
    if (status != 0) {
      char buff[2000];
      plc->port->hytec_card = NULL;
      sprintf(buff, "8403: \"%s\", code %d", plc->port->name, status);
      hytec_error(status);
      (void) fatalCARDINIT(buff);
    } else {
      /* Set up auto-calibration for 6 hour interval */
      status = ((CHy8403 *)plc->port->hytec_card)->Hy8403_EnableAutoCal(12);
      if (status != 0) {
	char buff[2000];
	sprintf(buff, "8403: \"%s\", code %d", plc->port->name, status);
	hytec_error(status);
      }
    }
    break; }
    /************************************************************/
    /* Hytec 8408 ()                                       */
    /************************************************************/
  case(VMB_TYPE_8408): {
    plc->port->hytec_card = (CHy8408 *)new CHy8408;
    
    status = 0;
    if (data.debug)
      vsys_msg_printf(0,"Initializing 8408 \"%s\"", plc->port->name);
    if (plc->port->ipslot != NULL) {
      char ipslot;
      ipslot = plc->port->ipslot[0];
      if (data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c, clockrate %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->clockrate);
      
      status = ((CHy8408 *)plc->port->hytec_card)->Hy8408_Configure(plc->port->vmeslot, 
								    plc->port->ipslot[0], 
								    plc->port->clockrate); 
      
      
      /* 	status = (DAC).Hy8402_Configure(vmeslot, dacIPslot1, vectornum2, doram1, clockSource1, dacClockRate1, inhibit1); */
      
    }      else
      fatalNOSLOT();
    if (data.debug)
      vsys_msg_printf(0, "status: %d", status);
    if (status != 0) {
      /* 	plc->port->hytec_card = (void *)&DAC; */
      /* 	vsys_msg_printf(0, "8402: %x\n", &DAC); */
      /*       } else { */
      char buff[2000];
      plc->port->hytec_card = NULL;
      sprintf(buff, "8408: \"%s\", code %d", plc->port->name, status);
      hytec_error(status);
      (void) fatalCARDINIT(buff);
    }
    break;
  }
    
    /************************************************************/
    /* Hytec 8404 ()                                       */
    /************************************************************/
    
  case(VMB_TYPE_8404): {
    plc->port->hytec_card = (CHy8404 *)new CHy8404;
    
    status = 0;
    if (data.debug)
      vsys_msg_printf(0,"Initializing 8404 \"%s\"", plc->port->name);
    if (plc->port->ipslot != NULL) {
      char ipslot;
      ipslot = plc->port->ipslot[0];
      if (data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c, vectornum %d, doram %d, clocksource %d, clockrate %d, inhibit %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->vectornum, 
			plc->port->doram, 
			plc->port->clocksource, 
			plc->port->clockrate, 
			plc->port->inhibit);
      
      status = ((CHy8404 *)plc->port->hytec_card)->Hy8404_Configure(plc->port->vmeslot, 
								    plc->port->ipslot[0], 
								    plc->port->vectornum, 
								    plc->port->doram, 
								    plc->port->clocksource, 
								    plc->port->clockrate, 
								    plc->port->inhibit); 
      
      
      /* 	status = (DAC).Hy8402_Configure(vmeslot, dacIPslot1, vectornum2, doram1, clockSource1, dacClockRate1, inhibit1); */
      
    }      else
      fatalNOSLOT();
    if (data.debug)
      vsys_msg_printf(0, "status: %d", status);
    if (status != 0) {
      /* 	plc->port->hytec_card = (void *)&DAC; */
      /* 	vsys_msg_printf(0, "8402: %x\n", &DAC); */
      /*       } else { */
      char buff[2000];
      plc->port->hytec_card = NULL;
      sprintf(buff, "8404: \"%s\", code %d", plc->port->name, status);
      hytec_error(status);
      (void) fatalCARDINIT(buff);
    }
    break;
  }
    
    /************************************************************/
    /* Hytec 8413 ()                                       */
    /************************************************************/
    
  case(VMB_TYPE_8413): {
    plc->port->hytec_card = (CHy8413 *)new CHy8413;
    status = 0;
    if (data.debug)
      vsys_msg_printf(0,"Initializing 8413 \"%s\"", plc->port->name);
    //   DAC13 = (CHy8413 *)fatalMalloc (sizeof(CHy8413));
    if (plc->port->ipslot) {
      if (data.debug)
	vsys_msg_printf(0, "vmeslot %d, ipslot %c, vectornum %d itrstate %d aitype %d clocksource %d clockrate %d inhibit %d samples %d spacing %d trigger %d", plc->port->vmeslot, plc->port->ipslot[0], plc->port->vectornum, plc->port->itrstate, plc->port->aitype, plc->port->clocksource, plc->port->clockrate, plc->port->inhibit, plc->port->samples, plc->port->spacing, plc->port->trigger);
      
      status = ((CHy8413 *)plc->port->hytec_card)->Hy8413_Configure(plc->port->vmeslot, plc->port->ipslot[0], plc->port->vectornum, plc->port->itrstate, plc->port->aitype, plc->port->clocksource, plc->port->clockrate, plc->port->inhibit, plc->port->samples, plc->port->spacing, plc->port->trigger);
		 //status = hytec_8413_card.Hy8413_Configure(plc->port->vmeslot, plc->port->ipslot[0], plc->port->vectornum, plc->port->itrstate, plc->port->aitype, plc->port->clocksource, plc->port->clockrate, plc->port->inhibit, plc->port->samples, plc->port->spacing, plc->port->trigger);
    } else
      fatalNOSLOT();
    
    if (status != 0) {
      /* 	plc->port->hytec_card = (void *)&DAC13; */
      /*        else { */
      char buff[2000];
      plc->port->hytec_card = NULL;
      sprintf(buff, "8413: \"%s\", code %d", plc->port->name, status);
      hytec_error(status);
      (void) fatalCARDINIT(buff);
    }
    break; }
    
    /* case(VMB_TYPE_8413): { */
/*   vsys_msg_printf(0,"******* 8413 not yet implemented ******"); */
/* break; } */
    /************************************************************/
    /* Hytec 8005 ()                                       */
    /************************************************************/
  case(VMB_TYPE_8005): {
    plc->port->hytec_card = (CHy8005 *)new CHy8005;
    status = 0;
    if (data.debug)
      vsys_msg_printf(0,"Initializing 8005 \"%s\"", plc->port->name);
    //  HY = (CHy8005 *)fatalMalloc (sizeof(CHy8005));
    if (plc->port->ipslot) {
      char ipslot;
      ipslot = plc->port->ipslot[0];
      if (data.debug)
	vsys_msg_printf(0, "vmeslot %d, ipslot %c, ", plc->port->vmeslot, plc->port->ipslot[0]);
      if (data.debug)
	vsys_msg_printf(0, "portnum %d, debrate %d, debmask %d, scanrate %d, dir %d, vectornum %d, intmask %d, pwidth %d, pmask %d, inhibit %d", plc->port->portnum, plc->port->debrate, 
			plc->port->debmask, plc->port->scanrate,
			plc->port->dir, plc->port->vectornum, 
			plc->port->intmask, plc->port->pwidth, 
			plc->port->pmask, plc->port->inhibit);
      status = ((CHy8005 *)plc->port->hytec_card)->Hy8005_Configure(plc->port->vmeslot, ipslot, 
								    plc->port->portnum, plc->port->debrate, 
								    plc->port->debmask, plc->port->scanrate,
								    plc->port->dir, plc->port->vectornum, 
								    plc->port->intmask, plc->port->pwidth, 
								    plc->port->pmask, plc->port->inhibit);
      
    }      else
      fatalNOSLOT();
    if (data.debug)
      vsys_msg_printf(0, "status: %d", status);
    if (status != 0) {
      /* 	plc->port->hytec_card = (void *)&HY; */
      /*       else { */
      char buff[2000];
      plc->port->hytec_card = NULL;
      sprintf(buff, "8005: \"%s\", code %d", plc->port->name, status);
      hytec_error(status);
      (void) fatalCARDINIT(buff);
    }
    break;
  }
    /************************************************************/
    /* Hytec 8411 ()                                       */
    /************************************************************/
  case(VMB_TYPE_8411): {
    plc->port->hytec_card = (CHy8411 *)new CHy8411;
    status = 0;
    //  HY = (CHy8411 *)fatalMalloc (sizeof(CHy8411));
    if (data.debug)
      vsys_msg_printf(0,"Initializing 8411 \"%s\"", plc->port->name);
    if (plc->port->ipslot != NULL) {
      char ipslot;
      ipslot = plc->port->ipslot[0];
      if (data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c, clockrate %d,",
			plc->port->vmeslot, 
			ipslot,
			plc->port->clockrate);
      status = ((CHy8411 *)plc->port->hytec_card)->Hy8411_Configure(plc->port->vmeslot, ipslot, plc->port->clockrate);
    }      else
      fatalNOSLOT();
    if (data.debug)
      vsys_msg_printf(0, "status: %d", status);
    if (status != 0) {
      /* 	plc->port->hytec_card = (void *)&HY; */
      /*       else { */
      char buff[2000];
      plc->port->hytec_card = NULL;
      sprintf(buff, "8411: \"%s\", code %d", plc->port->name, status);
      hytec_error(status);
      (void) fatalCARDINIT(buff);
    }
    break;
  }
    /************************************************************/
    /* Hytec 8512()                                       */
    /************************************************************/
  case(VMB_TYPE_8512): {
    plc->port->hytec_card = (CHy8512 *)new CHy8512;
    status = 0;
    //   HY = (CHy8512 *)fatalMalloc (sizeof(CHy8512));
    if (data.debug)
      vsys_msg_printf(0,"Initializing 8512 \"%s\"", plc->port->name);
    if (plc->port->ipslot != NULL) {
      char ipslot;
      ipslot = plc->port->ipslot[0];
      if (data.debug)
	vsys_msg_printf(0, "vme slot %d, ipslot %c, scaler %d, clock %d, vectornum %d, overflow %d",
			plc->port->vmeslot, 
			ipslot,
			plc->port->scaler,
			plc->port->clock,
			plc->port->vectornum, 
			plc->port->overflow);
      status = ((CHy8512 *)plc->port->hytec_card)->Hy8512_Configure(plc->port->vmeslot, ipslot, plc->port->scaler, plc->port->clock, plc->port->vectornum, plc->port->overflow);
    }      else
      fatalNOSLOT();
    if (data.debug)
      vsys_msg_printf(0, "status: %d", status);
    if (status != 0) {
      /* 	plc->port->hytec_card = (void *)&HY; */
      /*       else { */
      char buff[2000];
      plc->port->hytec_card = NULL;
      sprintf(buff, "8512: \"%s\", code %d", plc->port->name, status);
      hytec_error(status);
      (void) fatalCARDINIT(buff);
    }
    break;
  }
  default: {
    fatalINVTYPE ();
    break;
  }
  }
  
}
static void forceScan (int id) {
  Port* port;
  for (port = data.portList; port != NULL; port = port->next) {
    if (port->numInputChannels != 0) {
      lockPort (port);
      if (port->timerBank != NULL) {
	if (id != 0) {
	  int x;
	  for (x = 0; x < 100; x++) {
	    if (port->PLCs_to_scan[x] == 666) {
	      port->PLCs_to_scan[x] = id;
	      port->PLCs_to_scan[x+1] = 666;
	      break;
	    }
	    /*	    vsys_msg_printf(0, "IDs are set:");
		    for (x = 0; x < 100; x++) 
		    vsys_msg_printf(0, "%d", port->PLCs_to_scan[x]);*/
	  }

	}
	port->scanFlag = true;
	if (vtimerAlarm (port->timerBank) != VTIMER_OK)
	  fatalVTIMER (port->timerBank);
      }
      unlockPort (port);
    }
  }
}

/*
 *********************************************************************
 */

static int controlChannelAst (struct db_ast_args* args) {

  char* cmd = args->value.string;

  if (vsys_stricmp (cmd, "exit") == 0) {
    if (data.traceControlChannel)
      logMsg ("Got an 'exit' command on the control channel");
    lockGlobal ();
    setExitFlagAndWakeupScanners_LCK ();
    unlockGlobal ();
  }
  else if (vsys_strnicmp (cmd, "scan", 4) == 0) {
    int id = 0, matches;
/*     char		svalue[2000];  */
    vista_chix chix = args->chix;
    if (data.traceControlChannel)
      logMsg ("Got '%s' command on the control channel", cmd);
    matches = sscanf(cmd, "%*s now %d", &id);
    vsys_msg_printf(0, "matches %d, id %d", matches, id);
    if (matches) {
      forceScan (id);
    } else 
      forceScan (0);
    if (vdbc_sput (&chix, " ") != VDB_SUCCESS)
      logMsg ("Error clearing control channel");
  }
  else if (vsys_strnicmp (cmd, "configure", 9) == 0) {
    int id = 0, matches, status = 0;
    char		svalue[2000];  
    vista_chix chix = args->chix;
    if (data.traceControlChannel)
      logMsg ("Got '%s' command on the control channel", cmd);
    matches = sscanf(cmd, "%*s now %s", svalue);
    vsys_msg_printf(0, "Now re-configuring %s", svalue);
    if (matches) {
      status = reconfig(svalue);
      if (status == -1)
	vsys_msg_printf(0,"Warning: re-configure requested, but %s not found\n", svalue);
      //      forceScan (id);
    } else 
      vsys_msg_printf(0,"Warning: re-configure requested, but %s not recognized\n", svalue);
      //      forceScan (0);
    if (vdbc_sput (&chix, " ") != VDB_SUCCESS)
      logMsg ("Error clearing control channel");
  }
  else {
    if (data.traceControlChannel)
      if (strcmp (cmd, " ") != 0)
	logMsg ("Got an event on the control channel");
  }

  return 1;
}

/*
 *********************************************************************
 */

static void initControlChannel (void) {
  if (data.controlChannelName != NULL) {
    vista_chix* chix = &data.controlChannelChix;
    if (vdbc_channel_index (data.controlChannelName, chix) != VDB_ERR_OK)
      fatalNOCTRCHAN (); /*NCV*/
    if (vdbc_sput (chix, " ") != VDB_ERR_OK)
      vscannerFatalVdbChixError (chix);
    if (vdb_add_ast (chix, controlChannelAst, 0) != VDB_ERR_OK)
      vscannerFatalVdbChixError (chix); /*NCV*/
    if (data.traceControlChannel)
      logMsg ("Control channel: %s", data.controlChannelName);
  }
}

/*
 *********************************************************************
 */

static int eventRoutine (VDB_EVENT_ARGS* args) {

  vista_chix chix = args->data_item_list[0].chix;
  long listLong = *(long*) &args->user;
  long offsetLong = *(long*) &args->user_2;
  AddrList* list = (AddrList*) listLong;
  unsigned offset = (unsigned) offsetLong;

  vsys_msg_printf(0, "In here!\n");

  if (offset < list->numChannels) {

    Channel* chan = &list->chan[offset];

    if (vdb_compare_chix (&chix, &chan->pub.chix)) {

      Value* dataAddr = (Value* )args->data_item_list[0].data_addr;
      Plc* plc = list->plc;
      Port* port = plc->port;
      Value value;

      switch (chan->pub.dbDtype) {
      case VDB_DTYPE_BINARY: value.binary = dataAddr->int32; break;
      case VDB_DTYPE_INT8:
      case VDB_DTYPE_UINT8: value.int8 = dataAddr->int8; break;
      case VDB_DTYPE_INT16:
      case VDB_DTYPE_UINT16: value.int16 = dataAddr->int16; break;
      case VDB_DTYPE_INT:
      case VDB_DTYPE_UINT: value.int32 = dataAddr->int32; break;
      case VDB_DTYPE_FLOAT: value.float32 = dataAddr->float32; break;
      case VDB_DTYPE_DOUBLE: value.float64 = dataAddr->float64; break;
      default: fatalNOTIMPL ();	/*NCV*/
      }

      lockPort (port);

      if (chan->pub.hwDtype != chan->pub.dbDtype ||
#ifdef VAXFLOAT
	  chan->pub.hwDtype == VDB_DTYPE_FLOAT ||
	  chan->pub.hwDtype == VDB_DTYPE_DOUBLE ||
#endif
	  chan->conversionRoutine != NULL)
	(void) extToInt (chan, &value);

#define M(x) list->value.x[offset] = value.x
      switch (chan->pub.hwDtype) {
      case VDB_DTYPE_BINARY: M (binary); break;
      case VDB_DTYPE_INT16:
      case VDB_DTYPE_UINT16: M (int16); break;
      case VDB_DTYPE_INT:
      case VDB_DTYPE_UINT: M (int32); break;	/*NYCV*/
      case VDB_DTYPE_FLOAT: M (float32); break;
      case VDB_DTYPE_DOUBLE: M (float64); break;
      default: fatalNOTIMPL ();	/*NCV*/
      }
#undef M
      vsys_msg_printf(0, "Append!\n");
      APPEND_TO_CHANGED_LIST (port, plc, list, offset);
      if (! plc->outputTimer.timer->isSet)
	if (vtimerSetVtime0 (plc->outputTimer.timer) != VTIMER_OK)
	  fatalVTIMER (port->timerBank);

      unlockPort (port);
    }
  }
  
  return 1;
}

/*
 *********************************************************************
 */

static void bookEventsForAddrList (AddrList* list) {

  Channel* chan = list->chan;
  Plc* plc = list->plc;
  VDB_EVENT e;
  long id = 1;
  unsigned i;

  e.event_class_mask = VCLS_M_VALUE;
  e.event_field_mask = VFLD_M_INT_VALUE;
  e.event_source = 0;
  e.index = 0;

  for (i = 0; i < list->numChannels; i++) {
    if (vdb_add_event (&chan[i].pub.chix, &id, &e, &eventRoutine,
		       (long) list, (long) i) != VDB_ERR_OK)
      vscannerFatalVdbChixError (&chan[i].pub.chix); /*NCV*/

    lockGlobal ();

    getInitialValue (list, i);
    APPEND_TO_CHANGED_LIST (port, plc, list, i);

    unlockGlobal ();
  }
}

/*
 *********************************************************************
 */

static void bookDbEvents (void) {
  Plc* plc;
  for (plc = data.plcList; plc != NULL; plc = plc->next)
    DO_OUTPUT_LISTS (bookEventsForAddrList);
  initControlChannel ();
}

/*
 *********************************************************************
 *********************************************************************
 ***
 ***  Varchive stuff
 ***
 *********************************************************************
 *********************************************************************
 */

static void fatalVarcError (char* line) {
  char buff[32];
  if (data.noline)
    buff[0] = 0;
  else
    (void) vsys_snprintf (buff, sizeof buff, " (%s)", line);	/*NCV*/
  logFatal ("Vlogger API error%s\n%s", buff, vlogc_strerror (data.varc.api));
}	/*NCV*/

/*
 *********************************************************************
 */

static void once (Channel* chan, char* cname, vtime* time) {

  unsigned i;

  if (data.varc.onceField == NULL) {
    data.varc.onceField =
      (FunctionField*)fatalMalloc0 (data.varc.onceListSize * sizeof *data.varc.onceField);
    for (i = 0; i < data.varc.onceListSize; i++) {
      FunctionField* field = &data.varc.onceField[i];
      field->name = data.varc.onceList[i];
      mydbFindField (field);
      if (field->dtype != VDB_DTYPE_STRING)
	fatalONCESTRING (field->name);
    }
  }

  for (i = 0; i < data.varc.onceListSize; i++) {
    FunctionField* field = &data.varc.onceField[i];
    vista_chix* chix = &chan->pub.chix;
    char value[512];
    if (mydbGetStringField (chix, field, value, sizeof value) == NULL) {
      logMsg ("Field %s@%s is not allocated", cname, field->name);
    }
    else {
      if (vlogc_snap_field (data.varc.log, time, chan->varcChannel,
			    field->name, value) != 1)
	fatalVARC ();
    }
  }
}

/*
 *********************************************************************
 * Adds to varchive channels from all PLCs from the addr list type
 * specified by its offset relative to the beginning of the Plc
 * data structure.
 *
 * fixme1.pf: consider an option to control the order in which channels
 * are added to varchive (so that channels which change the most often
 * are added first and assigned a one-byte channel ID)
 *********************************************************************
 */

static void addChannelsToVarchive (unsigned listOffset) {

  Plc* plc;
  vtime time;
  unsigned i;

  (void) vsys_get_time (&time);

  for (plc = data.plcList; plc != NULL; plc = plc->next) {
    AddrList* list = (AddrList*) ((VUINT8*) plc + listOffset);
    for (i = 0; i < list->numChannels; i++) {
      Channel* chan = &list->chan[i];
      char* cname = mydbChannelName (&chan->pub.chix);
      char* dbname = data.databaseName [chan->databaseIndex];
      char* name = data.fullChannelName;
      if (strlen (cname) + strlen (dbname) > sizeof data.fullChannelName - 5)
	fatalINTERR ();	/*NCV*/
      (void) strcpy (name, dbname);
      vsys_strupr (name);
      (void) strcat (name, "::");
      (void) strcat (name, cname);
      if (chan->pub.dbDtype != VDB_DTYPE_BINARY) {
	int dbType = vscannerDbChannelType (&chan->pub.chix);
	chan->varcChannel = vlogc_add_channel (data.varc.log, &time, name,
					       dbType, 1);
      }
      else {
	vista_chix* chix = &chan->pub.chix;
	char b0[BINARY_STRING_SIZE];
	char b1[BINARY_STRING_SIZE];
	struct vdb_descrip_v desc;

#define M(n)								\
  VSYS_STRING_FILL (desc, b##n, sizeof b##n);				\
  if (vdb_get (chix, VCLS_M_TEXT, VFLD_M_B##n, &desc) != VDB_ERR_OK)	\
    vscannerFatalVdbChixError (chix);

	M (0);
	M (1);

#undef M

	chan->varcChannel =
	  vlogc_add_binary_channel (data.varc.log, &time, name, b0, b1);
      }
      if (chan->varcChannel == NULL) fatalVARC ();
      chan->varcIOErrorField =
	vlogc_add_field (data.varc.log, &time, chan->varcChannel, "IO_ERROR");
      if (chan->varcIOErrorField < 0) fatalVARC ();
      if (data.varc.onceListSize)
	once (chan, name, &time);
    }
  }
}

/*
 *********************************************************************
 */

#if vmbCONF_LIST_PUTS
static void addChannelListsToVarchive (void) {

  vtime currentTime;
  Port* port;

  vsys_get_time (&currentTime);

  for (port = data.portList; port != NULL; port = port->next) {
    ScanList* sl;
    for (sl = port->scanList; sl != NULL; sl = sl->next) {
      ScanBlock* sb;
      for (sb = sl->block; sb != NULL; sb = sb->next) {

	AddrList* addrList = sb->addrList;
	unsigned numChannels = sb->numChannels;
	Channel* chan = &addrList->chan[sb->offset];
	bool useList = 1;

	/* Does not make sense to use a list for just one channel. */

	if (numChannels < 2) useList = 0;

	/* Blocks of digital channels must be contiguous and start
	   on int16 boundary. */

	switch (addrList->type) {
	case LIST_TYPE_INPUT_R1:
	case LIST_TYPE_OUTPUT_R1: {
	  unsigned firstAddr = chan[0].pub.addr;
	  unsigned lastAddr = chan[numChannels - 1].pub.addr;
	  if ((firstAddr & 15) != 0 ||
	      lastAddr != firstAddr + numChannels - 1)
	    useList = 0;
	  break;
	}
	default:
	  break;
	}

	/* Channels must be of the same database type. */

	if (useList) {
	  int dbDtype0 = chan[0].pub.dbDtype;
	  unsigned i;
	  for (i = 1; i < numChannels; i++) {
	    if (chan[i].pub.dbDtype != dbDtype0) {
	      useList = 0;
	      break;
	    }
	  }
	}

	/* Create vlogger's list. */

	if (useList) {
	  VlogChannel** c = fatalMalloc (numChannels * sizeof *c);
	  unsigned i;
	  for (i = 0; i < numChannels; i++)
	    c[i] = chan[i].varcChannel;
	  sb->varcList = vlogAddList (data.varc.log, &currentTime,
				      c, numChannels);
	  myFree (c);
	  if (sb->varcList == NULL) fatalVARC ();
	}
      }
    }
  }
}
#endif

/*
 *********************************************************************
 * This routine is called every time a new interval is created.
 *********************************************************************
 */

static void newVarchiveInterval (VlogLog* log, void* arg) {
  if (data.varc.iv) {
    Port* port;
    for (port = data.portList; port != NULL; port = port->next) {
      if (port->numInputChannels != 0) {
	lockPort (port);
	port->newInterval = 1;
	unlockPort (port);
      }
    }
  }
}

/*
 *********************************************************************
 */

static void openVarchive (void) {

  struct varc_s *a = (struct varc_s *)&data.varc;

  if (a->name != NULL) {

    VlogLogAttr* attr;
    VlogApi* api;
    Port* port;
    Plc* plc;
    vtime time;

    if ((api = a->api = vlogc_init ()) == NULL) fatalNOMEM ();

    if (data.numPortsWithChannels > 1 || a->shared || a->circular) {
      a->mutexNeeded = 1;
      if (vlogc_global_mutex_init (api) != 1) fatalVARC ();
    }
#ifdef I64
    if (vlogc_set (api, VLOG_O_LOG_BINARY_STATE, (int) 1) != 1) fatalVARC ();
#else
    if (vlogc_set (api, VLOG_O_LOG_BINARY_STATE, (void *) 1) != 1) fatalVARC ();
#endif
    if ((attr = vlogc_attr_create (api)) == NULL) fatalVARC ();
    if (! a->restart)
      if (vlogc_attr_set_replace (attr, 1) != 1) fatalVARC ();

    if (! a->shared && ! a->circular)
      if (vlogc_attr_set_sequential (attr, 1) != 1) fatalVARC ();

    if (a->fsync)
      if (vlogc_attr_set_fsync (attr, 1) != 1) fatalVARC ();

    if (a->shared) {
      if (vlogc_attr_set_shared (attr, 1) != 1) fatalVARC ();
      if (a->restart)
	if (vlogc_attr_set_restart (attr, 1) != 1) fatalVARC ();
    }

    if (data.varc.bufferSize != 0)
      if (vlogc_attr_set_buffer_size (attr, data.varc.bufferSize) != 1)
	fatalVARC ();	/*NCV*/
    if ((a->log = vlogc_open (api, a->name, attr)) == NULL) fatalVARC ();
    (void) vlogc_attr_delete (attr);

    VARC_LOCK ();

    if (a->shared || a->circular) {
      if (a->latency != 0 && vlogc_set_latency (a->log, a->latency) != 1)
	fatalVARC ();	/*NCV*/

      if (a->circular) {
	if (vlogc_set_log_length (a->log, 1,
				  a->lengthSec, a->intervalSec,
				  a->lengthKb, a->intervalKb) != 1 ||
	    vlogc_set_new_interval_routine (a->log, newVarchiveInterval,
					    NULL) != 1)
	  fatalVARC ();	/*NCV*/
      }
    }

    /* add start and end scan channels */

    (void) vsys_get_time (&time);
    for (port = data.portList; port != NULL; port = port->next) {
      if (port->numInputChannels != 0 || port->numOutputChannels != 0) {
	if (port->startScanChannelName != NULL) {
	  port->startScanChannel =
	    vlogc_add_channel (a->log, &time, port->startScanChannelName,
			       VDB_TYPE_INT8, 1);
	  if (port->startScanChannel == NULL) fatalVARC ();
	}
	if (port->endScanChannelName != NULL) {
	  port->endScanChannel =
	    vlogc_add_channel (a->log, &time, port->endScanChannelName,
			       VDB_TYPE_TIME, 1);
	  if (port->endScanChannel == NULL) fatalVARC ();
	}
	if (port->maxChannelName != NULL) {
	  port->maxChannel =						/*NCV*/
	    vlogc_add_channel (a->log, &time, port->maxChannelName,	/*NCV*/
			       VDB_TYPE_INTEGER, 1);			/*NCV*/
	  if (port->maxChannel == NULL) fatalVARC ();			/*NCV*/
	}
      }
    }

    /* add input channels */

#define M(list) addChannelsToVarchive ((VINT8*) (list) - (VINT8*) plc)
    plc = NULL;
    DO_INPUT_LISTS_REG_FIRST (M);
#undef M

    /* add channel lists */
#if vmbCONF_LIST_PUTS
    if (data.varc.useChannelLists)
      addChannelListsToVarchive ();
#endif

    if (vlogc_delete_old_channels (data.varc.log, &time) != 1)
      fatalVARC ();	/*NCV*/

    VARC_UNLOCK ();
  }
}

/*
 *********************************************************************
 */

static void closeVarchive (void) {
  if (data.varc.log != NULL) {
    vtime time = data.wakeupScannersTime;
    if (time.t_sec == 0)
      (void) vsys_get_time (&time);	/*NYCV*/
    VARC_LOCK ();
    if (vlogc_mark (data.varc.log, &time, vlogMARK_STOP_USER) != 1)
      fatalVARC ();	/*NCV*/
    if (vlogc_close (data.varc.log) != 1) fatalVARC ();
    VARC_UNLOCK ();
    vlogc_free (data.varc.api);
  }
}

/*
 *********************************************************************
 */

static void startVarchive (void) {
  vtime time;
  (void) vsys_get_time (&time);
  if (data.varc.log != NULL) {
    VARC_LOCK ();
    if (vlogc_mark (data.varc.log, &time, vlogMARK_START_USER) != 1)
      fatalVARC ();	/*NCV*/
    VARC_UNLOCK ();
  }
}

/*
 *********************************************************************
 *********************************************************************
 ***
 ***  Build scan lists
 ***
 *********************************************************************
 *********************************************************************
 */

static void showPorts (void) {

  Port* port;

  for (port = data.portList; port != NULL; port = port->next) {
    if (port->name[0] == 0 &&
	port->numInputChannels == 0 &&
	port->numOutputChannels == 0)
      continue;
    (void) printf ("Port %s, %d input channels, %d output channels\n",
		   port->name[0] ? port->name : "<noname>",
		   port->numInputChannels,
		   port->numOutputChannels);
    if (port->type)
      (void) printf ("  Type = %s\n", itoaType (port->type));
/*     if (port->type == VMB_TYPE_RTU || port->type == VMB_TYPE_ASCII) { */
      if (port->parity)
	(void) printf ("  Parity = %s\n", itoaParity (port->parity));
      if (port->baud)
	(void) printf ("  Baud Rate = %u\n", port->baud);
      if (port->stop)
	(void) printf ("  Stop Bits = %u\n", port->stop);
      if (port->rtsDelay)
	(void) printf ("  RTS delay = %u\n", port->rtsDelay);
      if (port->timeout)
	(void) printf ("  Timeout = %u\n", port->timeout);
      if (port->turnaroundDelay)
	(void) printf ("  Turnaround delay = %u\n", port->turnaroundDelay);
      if (port->device != NULL && port->device[0] != 0)
	(void) printf ("  Device = %s\n", port->device);
      if (port->driver != NULL && port->driver[0] != 0)
	(void) printf ("  Driver = %s\n", port->driver);
/*     } */

    if (port->numPlcs != 0) {
      unsigned i;
      (void) printf ("  Number of PLCs = %d\n", port->numPlcs);
      for (i = 0; i < port->numPlcs; i++) {
	Plc* plc = port->plc[i];
	(void) printf ("    PLC %s, unit %d, %d input channels, "
		       "%d output channels\n",
		       plc->name,
		       plc->unit ? plc->unit : 1,
		       plc->numInputChannels,
		       plc->numOutputChannels);
      }
    }
  }
}

/*
 *********************************************************************
 */

static void showScanLists (Port* port) {

  ScanList* sl = port->scanList;
  unsigned listCount = 1;

  for (; sl != NULL; sl = sl->next, listCount++) {

    ScanBlock* b = sl->block;
    unsigned blockCount = 1;

    (void) printf ("List %d of scan blocks with scan rate %d, sync %d\n",
		   listCount, sl->rate, sl->clocksync);

    for (; b != NULL; b = b->next, blockCount++) {

      unsigned offset = b->offset;
      AddrList* list = b->addrList;
      Channel* chan = &list->chan[offset];
      unsigned addr0 = chan->pub.addr;
      Plc* plc = list->plc;
      bool b16 = list->dsizeBits == 1 && (plc->d16);
      unsigned i;

      if (list->dsizeBits == 1)
	addr0 &= ~15;

      (void) printf ("  Scan block %d.%d of %s starting at address ",
		     listCount, blockCount, list->descr);
      if (b16) {
	(void) printf (data.showDecimal ? "%05d\n" : "0x%04X\n",
		       addr0 / 16 + 1);
      }
      else {
	unsigned printAddr = addr0 + 1;
	if (plc->enron && list->dsizeBits == 32)
	  printAddr--;
	(void) printf (data.showDecimal ? "%05d\n" : "0x%04X\n", printAddr);
      }

      for (i = 0; i < b->numChannels; i++, chan++) {
	(void) printf ("    ");
	if (b16)
	  (void) printf (data.showDecimal ? "%05d:%02d " : "%04X:%X ",
			 chan->pub.addr / 16 + 1, chan->pub.addr & 15);
	else {
	  unsigned printAddr = chan->pub.addr + 1;
	  if (plc->enron && list->dsizeBits == 32)
	    printAddr--;
	  (void) printf (data.showDecimal ? "%05d " : "%04X ", printAddr);
	}
	if (data.numDatabases > 1)
	  (void) printf ("%s::", data.databaseName[chan->pub.chix.dbix - 1]);
	(void) printf ("%s\n", mydbChannelName (&chan->pub.chix));
      }
    }
  }
}

/*
 *********************************************************************
 */

static void addScanBlockToScanList (Port* port, ScanBlock* newB,
				    unsigned rate, unsigned clocksync) {

  ScanList* sl = port->scanList;
  ScanList* prev = NULL;

  newB->rate = rate;

/*   vsys_msg_printf(0, "Port scanlist %x", port->scanList); */

  while (sl != NULL) {
/*     vsys_msg_printf(0, "THROUGH sl rate %d rate sl %d sync %d sync %d", sl->rate, rate, sl->clocksync, clocksync);  */
    if (sl->rate == rate && sl->clocksync == clocksync) 
      break;
    prev = sl;
    sl = sl->next;
  }

  if (sl == NULL || sl->rate != rate || sl->clocksync != clocksync) {
    ScanList* newsl = (ScanList*)fatalMalloc0 (sizeof (newsl[0]));
    /*    if (sl)
      vsys_msg_printf(0, "sl rate %d rate %d sl sync %d sync %d", sl->rate, rate, sl->clocksync, clocksync); 
    else 
    vsys_msg_printf(0, "rate %d sync %d", rate, clocksync); */
    newsl->rate = rate;
    newsl->clocksync = clocksync;
    newsl->port = port;
    if (prev == NULL) {
      newsl->next = port->scanList;
      port->scanList = newsl;
    }
    else {
      newsl->next = prev->next;
      prev->next = newsl;
    }
    sl = newsl;
  }
    /*  } else {
    if (sl)
      vsys_msg_printf(0, "NO sl rate %d rate %d sl sync %d sync %d", sl->rate, rate, sl->clocksync, clocksync); 
    else 
      vsys_msg_printf(0, "NO rate %d sync %d", rate, clocksync); 
  }
    */

  if (sl->block == NULL)
    sl->block = newB;
  else {
    ScanBlock* b = sl->block;
    while (b->next != NULL)
      b = b->next;
    b->next = newB;
  }
}

/*
 *********************************************************************
 * Set block flags if there is at least one channels with
 * the corresponding flags set.
 *********************************************************************
 */

static void setBlockConversionAndIOErrorFlags (ScanBlock* b) {

  AddrList* addrList = b->addrList;
  Channel* chan = &addrList->chan[b->offset];
  unsigned i;

  for (i = 0; i < b->numChannels; i++) {
    Channel* c = chan + i;
    if (c->conversionRoutine != NULL)
      b->conversionRequired = 1;
    if (c->flags.ioError)
      b->ioError = 1;
  }
}

/*
 *********************************************************************
 * Returns 1 if channels belong to the same scan block.
 *********************************************************************
 */

static int channelsBelongToSameBlock (AddrList* list, Channel* curr,
				      Channel* prev) {
  int retval = 0;

  if (curr->rate == prev->rate && curr->clocksync == prev->clocksync) {

    Plc* plc = list->plc;

    if (plc->map.listSize != 0) {
      if (curr->mapEntry == prev->mapEntry)
	retval = 1;
    }
    else {
      if (list->dsizeBits == 1) {
	unsigned prevAddr16 = prev->pub.addr & ~15;
	unsigned currAddr16 = curr->pub.addr & ~15;
	if (prev->pub.addr != curr->pub.addr &&
	    (currAddr16 == prevAddr16 ||
	     currAddr16 == prevAddr16 + 16))
	  retval = 1;
      }
      else {
	unsigned nextAddr = (list->dsizeBits == 16 ? 1 :
			     list->dsizeBits == 32 ? 2 : 4);
	if (plc->enron && list->dsizeBits == 32)
	  nextAddr = 1;
	/* CML changed this */
	if (data.control_design) {
	  if ((curr->pub.addr <= prev->pub.addr + (nextAddr * 10)) && (curr->pub.addr != prev->pub.addr)) 
	    retval = 1;
	} else {
	  if (curr->pub.addr == prev->pub.addr + nextAddr)
	    retval = 1;
	}
      }
    }
  }

  return retval;
}

/*
 *********************************************************************
 */

static void addrToScanLists (AddrList* list) {

  Plc* plc = list->plc;
  Port* port = plc->port;
  
  if (list->numChannels) {

    Channel* chan = list->chan;
    unsigned i = 0;
    unsigned total_block_diff = 0;
  
    list->numRegisters = 0;

    while (i < list->numChannels) {

      ScanBlock* b = (ScanBlock *)fatalMalloc0 (sizeof (b[0]));
      Channel* chan0 = chan + i;
      Channel* prev = chan0;
      unsigned j;
      unsigned prev_block_addr = 0;
      unsigned prev_block_diff = 0;

/*       vsys_msg_printf(0, "New scan block: starts with %s\n", mydbChannelName(&chan0->pub.chix));  */

      if (list->dsizeBits == 1) {
	unsigned addr0 = chan0->pub.addr & ~15;
	chan0->portBufferOffset = chan0->pub.addr - addr0;

	for (j = i + 1; j < list->numChannels; j++) {
	  Channel* curr = chan + j;
	  if (! channelsBelongToSameBlock (list, curr, prev)) {
	    break;
	 } else {
	    curr->portBufferOffset = curr->pub.addr - addr0;
	    /* CML added this next line */
	    curr->blockListOffset = curr->pub.addr - prev->pub.addr;
/* 	    vsys_msg_printf(0, "%s: addr is %d and blockListOffset is %d\n", mydbChannelName(&curr->pub.chix), curr->pub.addr, curr->blockListOffset); */
	    prev = curr;
	  }
	}
	b->count = prev->pub.addr - addr0 + 1;
      }
      else {
	unsigned dsize16 = (list->dsizeBits == 16 ? 1 :
			    list->dsizeBits == 32 ? 2 : 4);
	for (j = i + 1; j < list->numChannels; j++) {
	  Channel* curr = chan + j;
	  if (! channelsBelongToSameBlock (list, curr, prev)) {
	    /* CML added this */
	    prev_block_diff = curr->pub.addr - prev_block_addr;
	    total_block_diff += prev_block_diff;
	    break;
	  } else {
	    curr->portBufferOffset = plc->enron && list->dsizeBits == 32 ?
	      curr->pub.addr - chan0->pub.addr :
	      (curr->pub.addr - chan0->pub.addr) / dsize16;
	    /* CML added this */
	    curr->blockListOffset = plc->enron && list->dsizeBits == 32 ?
	      curr->pub.addr - prev->pub.addr :
	      (curr->pub.addr - prev->pub.addr) / dsize16;
/* 	    vsys_msg_printf(0, "%s: addr is %d and blockListOffset is %d\n", mydbChannelName(&curr->pub.chix), curr->pub.addr, curr->blockListOffset); */
	    prev_block_addr = curr->pub.addr;
	    prev = curr;
	  }
	}
	b->count = prev->portBufferOffset + 1;
/* 	vsys_msg_printf(0, "The count is %d\n", b->count); */
      }

      /* CML added this */
      list->numRegisters += b->count;

      b->plc = plc;
      b->addrList = list;
      b->numChannels = j - i;
      b->offset = i;
      b->cur_retries = 0;

      setBlockConversionAndIOErrorFlags (b);
      /* CML added the clocksync stuff, to allow for a delay before scanning. 
	 Note that the scanner only checks the first channel's clocksync. 
	 All channels in a scan block should have the same clocksync,
	 or subsequent ones will be ignored. */
      addScanBlockToScanList (port, b, chan0->rate, chan0->clocksync);
      i = j;
    }
  }
}

/*
 *********************************************************************
 */

static void printAddrList (AddrList* list) {

  if (list->numChannels) {

    Plc* plc = list->plc;
    bool b16 = list->dsizeBits == 1 && (plc->d16);
    unsigned nextOffset = (list->dsizeBits == 64 ? 4 :
			   list->dsizeBits == 32 ? 2 : 1);
    unsigned i;

    (void) printf ("    %s", list->descr);

    for (i = 0; i < list->numChannels; i++) {

      unsigned addr = list->chan[i].pub.addr;
      vista_chix chix = list->chan[i].pub.chix;
      bool same = 0;

      if (i) {
	unsigned prev = list->chan[i - 1].pub.addr;
	if (addr != prev + nextOffset) {
	  if (addr == prev)
	    same = 1;
	}
      }

      if (same)
	(void) printf (", ");
      else {
	(void) printf ("\n      ");
	if (b16)
	  (void) printf (data.showDecimal ? "%05d:%02d " : "%04X:%X ",
			 addr / 16 + 1, addr & 15);
	else {
	  unsigned printAddr = addr + 1;
	  if (plc->enron && list->dsizeBits == 32)
	    printAddr--;
	  (void) printf (data.showDecimal ? "%05d " : "%04X ", printAddr);
	}
      }

      if (data.numDatabases > 1)
	(void) printf ("%s::", data.databaseName[chix.dbix - 1]);
      (void) printf ("%s", mydbChannelName (&chix));

      if (list->isInput)
	(void) printf (" (%d)", list->chan[i].rate);
    }

    (void) printf ("\n");
  }
}

/*
 *********************************************************************
 */

static void showOffsets (PlcAddrOffset* o) {

  static char title[] = "  Address offsets\n";
  bool showTitle = 1;

#define M1(ioTitle, ioField, name, field) do {				 \
  if (o->ioField.field != 0) {						 \
    if (showTitle) {							 \
      (void) printf (title);						 \
      showTitle = 0;							 \
    }									 \
    (void) printf ("    " ioTitle " " name " = %u\n", o->ioField.field); \
  }									 \
} while (0)

#define M2(ioTitle, ioField) do {		\
  M1 (ioTitle, ioField, "binary", binary);	\
  M1 (ioTitle, ioField, "float", float32);	\
  M1 (ioTitle, ioField, "double", float64);	\
  M1 (ioTitle, ioField, "int16", int16);	\
  M1 (ioTitle, ioField, "int32", int32);	\
} while (0)

  M2 ("Input", input);
  M2 ("Output", output);

#undef M1
#undef M2
}

/*
 *********************************************************************
 */

static void showPlc (Plc* plc) {
  (void) printf ("PLC %s", plc->name);
  if (plc->endian != ENDIAN_LITTLE)
    (void) printf (" -endian=BIG");
  if (plc->a32 == 1)
    (void) printf (" -a32");
  if (plc->d16)
    (void) printf (" -d16");
  if (plc->enron)
    (void) printf (" -enron");
  (void) printf ("\n");
  showOffsets (&plc->addrOffset);
  if (plc->numInputChannels != 0) {
    (void) printf ("  Input channels\n");
    DO_INPUT_LISTS (printAddrList);
  }
  if (plc->numOutputChannels != 0) {
    (void) printf ("  Output channels\n");
    DO_OUTPUT_LISTS (printAddrList);
  }
}

/*
 *********************************************************************
 */

static void getScanRates (AddrList* list) {
  if (list->numChannels) {
    Channel* chan = list->chan;
    unsigned i;
    for (i = 0; i < list->numChannels; i++, chan++) {
      vista_chix* chix = &chan->pub.chix;
      (void) mydbGetUintField (chix, &data.fields.scanRate, &chan->rate);
      if (chan->rate == 0)
	chan->rate = 1;
      if (data.schedule != NULL) {
	if (chan->rate > data.scheduleSize)
	  fatalNOSCHED (mydbChannelName (chix));
      }
      /* CML added this to allow delay before scanning */
      if (data.fields.clockSync.name != NULL)
	(void) mydbGetUintField (chix, &data.fields.clockSync, &chan->clocksync);
      /* CML added this field to allow SDI points to pass their addresses */
      if (data.fields.SDIaddress.name != NULL)
      (void) mydbGetUintField (chix, &data.fields.SDIaddress, &chan->SDIaddress);

      if (data.fields.refreshRate.name == NULL || ! VARC_CHAN (chan))
	chan->refreshVarcTime = MAX_REFRESH_VARC_TIME;
      else
	(void) mydbGetUintField (chix, &data.fields.refreshRate,
				 &chan->refreshRate);
    }
  }
}

/*
 *********************************************************************
 * This routine allocates a buffer for values of all channels of a
 * given type. Also, for input channels, it calculates the size of
 * a buffer large enough to hold values of all channels of any type.
 *********************************************************************
 */

static void allocValueBuffer (AddrList* list) {

  if (list->numChannels) {

/*     vsys_msg_printf(0, "Allocing %d values, could alloc %d\n", list->numChannels, list->numRegisters); */
    
//#define M(ptr, n) ptr = fatalMalloc0 ((n) * sizeof ((ptr)[0]))

    switch (list->type) {
    case LIST_TYPE_INPUT_R1:
    case LIST_TYPE_OUTPUT_R1:
    case LIST_TYPE_INPUT_COILS:
      
      /*   case VHYTEC_TYPE_INPUT_DO:
    case VHYTEC_TYPE_INPUT_DI:
    case VHYTEC_TYPE_OUTPUT_DO:
    case VHYTEC_TYPE_OUTPUT_DI:*/
      list->value.binary = (VUINT8* ) fatalMalloc0 (list->numChannels * sizeof ((list->value.binary)[0]));
      break;
    case LIST_TYPE_INPUT_R16:
    case LIST_TYPE_OUTPUT_R16:
    case LIST_TYPE_INPUT_HOLDING:
    case LIST_TYPE_OUTPUT_HOLDING:
      list->value.int16 = (VUINT16* )fatalMalloc0 (list->numChannels * sizeof ((list->value.int16)[0]));
      break;
    case LIST_TYPE_INPUT_R32:
    case LIST_TYPE_OUTPUT_R32:
    case LIST_TYPE_INPUT_HOLDING_R32:
    case LIST_TYPE_OUTPUT_HOLDING_R32:
      /*    case VHYTEC_TYPE_OUTPUT_ADC:
    case VHYTEC_TYPE_INPUT_DAC:
    case VHYTEC_TYPE_INPUT_MOTOR:
    case VHYTEC_TYPE_OUTPUT_MOTOR:
    case VHYTEC_TYPE_INPUT_ADC:
    case VHYTEC_TYPE_OUTPUT_DAC:*/
 list->value.int32 = (VUINT32* )fatalMalloc0 (list->numChannels * sizeof ((list->value.int32)[0]));
 list->value.float32 = (float*)fatalMalloc0 (list->numChannels * sizeof ((list->value.float32)[0]));
    /*   M (list->value.int32, list->numChannels); */
/*       M (list->value.float32, list->numChannels); */
      break;
          case LIST_TYPE_INPUT_R64:
    case LIST_TYPE_OUTPUT_R64:
    case LIST_TYPE_INPUT_HOLDING_R64:
    case LIST_TYPE_OUTPUT_HOLDING_R64:
 list->value.int64 = (VUINT64* )fatalMalloc0 (list->numChannels * sizeof ((list->value.int64)[0]));
 list->value.float64 = (double* )fatalMalloc0 (list->numChannels * sizeof ((list->value.float64)[0]));
      
   /*    M (list->value.int64, list->numChannels); */
/*       M (list->value.float64, list->numChannels); */
      break;
        case LIST_TYPE_INPUT_STRING:
	  case LIST_TYPE_OUTPUT_STRING:
	    /*  case VHYTEC_TYPE_INPUT_COM:
		case VHYTEC_TYPE_OUTPUT_COM: */
/*       if (!data.control_design) */
/* 	fatalNOTIMPL (); */
      break;
    case LIST_TYPE_INPUT_SDI:
    case LIST_TYPE_OUTPUT_SDI:
      if (!data.control_design)
	fatalNOTIMPL ();
	break;
    default:
     fatalNOTIMPL ();	/*NCV*/
    }

    if (! list->isInput) {
      list->o.changed = (bool*)fatalMalloc0 (list->numChannels * sizeof ((list->o.changed)[0]));
      list->o.list = (void**)fatalMalloc0 (list->numChannels * sizeof ((list->o.list)[0]));
    /*   M (list->o.changed, list->numChannels); */
/*       M (list->o.list, list->numChannels); */
    }

    //#undef M
  }
}

/*
 *********************************************************************
 */

static void allocPortBuffer (Port* port) {

  ScanBlock* block;
  ScanList* list;
  unsigned maxSize = 0;

  for (list = port->scanList; list != NULL; list = list->next) {
    for (block = list->block; block != NULL; block = block->next) {
      AddrList* addrList = block->addrList;
      unsigned sizeBits = addrList->dsizeBits * block->count;
      unsigned size = (sizeBits + 7) / 8;
      if ((size & 1) != 0)
	size++;
      if (size > maxSize)
	maxSize = size;
    }
  }

  port->buffer.u8 = (VUINT8*)fatalMalloc (maxSize + 4); /* Extra 4 bytes for safety */
}

/*
 *********************************************************************
 */

static void setChannelDisposition (Channel* chan, unsigned disposition,
				   int in) {
  if (! in) {
    chan->flags.dbPut = 1; /* we may need to set I/O bit */
  }
  else {
    if (disposition == 0) {
      if (data.disposition != 0)
	disposition = data.disposition;
      else if (data.varc.name != NULL)
	disposition = VHYTEC_DISPOSITION_VARCHIVE;
      else
	disposition = VHYTEC_DISPOSITION_DATABASE;
    }
    switch (disposition) {
    case VHYTEC_DISPOSITION_VARCHIVE:
      if (data.varc.name != NULL)
	chan->flags.varc = 1;
      break;
    case VHYTEC_DISPOSITION_DATABASE:
      chan->flags.dbPut = 1;
      break;
    case VHYTEC_DISPOSITION_BOTH:
      chan->flags.dbPut = 1;
      if (data.varc.name != NULL)
	chan->flags.varc = 1;
      break;
    default:
      fatalDISPOSITION (mydbChannelName (&chan->pub.chix));
    }	/*NCV*/
  }
}

/*
 *********************************************************************
 * This function is called from buildAddrList for every channel
 * to be scanned.
 *********************************************************************
 */

static void addChannelToAddrList (Plc* plc, vista_chix* chix,
				  unsigned databaseIndex) {

  VhytecRoutine conversionRoutine = NULL;
  AddrList* list = NULL;
  Channel* chan;
  unsigned addr;
  unsigned disposition = 0;
  unsigned hwType = 0;
  Port* port = plc->port;
  struct mapEntry_s* mapEntry = NULL;
  struct map_s* map = NULL;
  int dbType = vscannerDbChannelType (chix);
  int in = vscannerDbIn (chix);
  int dbDtype NIL0;
  int hwDtype NIL0;
  int plcOffset NIL0;
  unsigned thermType = 0;
  unsigned int appregType = 0;
  unsigned int appreg = 0;

  /* get address */

  (void) mydbGetUintField (chix, &data.fields.address, &addr);
  // if (addr == 0) fatalADDR0 (mydbChannelName (chix));
      
  //  addr--;
  /* CML moved this below, so that binary types can use large addresses */
  /*   if (addr > 0xffff) fatalINVADDR (mydbChannelName (chix)); */

  /* get conversion routine */

  if (data.fields.conversionRoutine.name != NULL) {
    char name[512];
    if (mydbGetStringField (chix, &data.fields.conversionRoutine,
			    name, sizeof name) != NULL &&
	name[0] != 0) {
      void* addr = NULL;
      if (data.userFiles != NULL)
	addr = vsys_lib_find_routine (name, data.userFiles);
      if (addr == NULL) fatalNOROUTINE (name);
      (void) memcpy (&conversionRoutine, &addr, sizeof conversionRoutine);
    }
  }

  /* get channel disposition */

  if (data.fields.disposition.name != NULL)
    (void) mydbGetUintField (chix, &data.fields.disposition, &disposition);

  /* get data point type */

  (void) mydbGetUintField (chix, &data.fields.dataPointType, &hwType);

  (void) mydbGetUintField (chix, &data.fields.appRegBit, &appreg);

  (void) mydbGetUintField (chix, &data.fields.thermocoupleType, &thermType);

   if (data.debug) 
     vsys_msg_printf(0,"Data type %d, dbType %d and thermocouple type for %s is %d (1 = K, 2 = T, 3 = J)",  
		     hwType, dbType, mydbChannelName (chix), thermType); 

  /* If data point type is not specified explicitly, determine it
     based on the type of the channel */

   if (hwType == VHYTEC_TYPE_OUTPUT_APPREG ||
       hwType == VHYTEC_TYPE_INPUT_APPREG) {
     appregType = hwType;
     hwType = 0;
   }

  if (hwType == 0) {
    switch (dbType) {
    case VDB_TYPE_BINARY:
      hwType = in ? VHYTEC_TYPE_INPUT_DO : VHYTEC_TYPE_OUTPUT_DO;
      break;
    case VDB_TYPE_INT16:
      hwType = in ? VHYTEC_TYPE_INPUT_I16 : VHYTEC_TYPE_OUTPUT_I16;
      break;
    case VDB_TYPE_UINT16:
      hwType = in ? VHYTEC_TYPE_INPUT_U16 : VHYTEC_TYPE_OUTPUT_U16;
      break;
    case VDB_TYPE_INTEGER:
      hwType = in ? VHYTEC_TYPE_INPUT_I32 : VHYTEC_TYPE_OUTPUT_I32;
      break;
    case VDB_TYPE_UINTEGER:
      hwType = in ? VHYTEC_TYPE_INPUT_U32 : VHYTEC_TYPE_OUTPUT_U32;
      break;
    case VDB_TYPE_REAL:
      hwType = in ? VHYTEC_TYPE_INPUT_ADC : VHYTEC_TYPE_OUTPUT_DAC;
      break;
    case VDB_TYPE_DOUBLE:
      hwType = in ? VHYTEC_TYPE_INPUT_F64: VHYTEC_TYPE_OUTPUT_F64;
      break;
    case VDB_TYPE_STRING:
       hwType = in ? VHYTEC_TYPE_INPUT_COM : VHYTEC_TYPE_OUTPUT_COM;
      break;
    default:
      vsys_msg_printf(0, "Could not match data type");
      fatalINVHWTYPE(mydbChannelName (chix));
      break;
    }
  }

  /* if (hwType == 0) {
#define M(vdb, vhytec)				\
  case VDB_TYPE_##vdb:				\
    hwType = in ?				\
      VHYTEC_TYPE_INPUT_##vhytec :		\
	VHYTEC_TYPE_OUTPUT_##vhytec;		\
    break

    switch (dbType) {
      M (BINARY, I1);
      M (INT16, I16);
      M (UINT16, U16);
      M (REAL, F32);
      M (DOUBLE, F64);
      M (STRING, STRING);
    default: fatalINVHWTYPE (mydbChannelName (chix));
#undef M
    }
    }*/

  /* Determine VDB_DTYPE by VDB_TYPE. */

  switch (dbType) {
  case VDB_TYPE_BINARY: dbDtype = VDB_DTYPE_BINARY; break;
  case VDB_TYPE_INT8: dbDtype = VDB_DTYPE_INT8; break;
  case VDB_TYPE_INT16: dbDtype = VDB_DTYPE_INT16; break;
  case VDB_TYPE_INTEGER: dbDtype = VDB_DTYPE_INT; break;
  case VDB_TYPE_UINT8: dbDtype = VDB_DTYPE_UINT8; break;
  case VDB_TYPE_UINTEGER: dbDtype = VDB_DTYPE_UINT; break;
  case VDB_TYPE_UINT16: dbDtype = VDB_DTYPE_UINT16; break;
  case VDB_TYPE_REAL: dbDtype = VDB_DTYPE_FLOAT; break;
  case VDB_TYPE_DOUBLE: dbDtype = VDB_DTYPE_DOUBLE; break;
  case VDB_TYPE_STRING: dbDtype = VDB_DTYPE_STRING; break;	/*NYCV*/
  case VDB_TYPE_TIME: dbDtype = VDB_DTYPE_TIME; break;		/*NYCV*/
  default: fatalNOTIMPL ();	/*NCV*/
  }

  /* Determine hwDtype and plcOffset. */

  vsys_msg_printf(0, "dbDtype %d, hwType %d",dbDtype, hwType);

  if ((appregType == VHYTEC_TYPE_OUTPUT_APPREG) ||
      (appregType == VHYTEC_TYPE_INPUT_APPREG)) {
    if (dbDtype != VDB_DTYPE_INT16 &&
	dbDtype != VDB_DTYPE_UINT16 &&
	dbDtype != VDB_DTYPE_INT &&
	dbDtype != VDB_DTYPE_BINARY) {
      vsys_msg_printf(0, "Error: Application Register channel must be of 16-bit integer or binary channel type.");
      fatalINVHWTYPE (mydbChannelName (chix));
    }
  }

  switch (hwType) {

#define M(x,y,z)				\
  case VHYTEC_TYPE_##x:			\
    hwDtype = VDB_DTYPE_##y;			\
    plcOffset = plc->addrOffset.z;		\
    map = (struct map_s*)&plc->map.z;			\
    break

    M (INPUT_DO, BINARY, input.binary);
    M (INPUT_DI, BINARY, input.binary);
    M (INPUT_ADC, FLOAT, input.float32);
    M (INPUT_DAC, FLOAT, input.float32);
    M (INPUT_MOTOR, FLOAT, input.float32);
    M (INPUT_COM, STRING, input.string);
    M (OUTPUT_DO, BINARY, output.binary);
    M (OUTPUT_DI, BINARY, output.binary);
    M (OUTPUT_ADC, FLOAT, output.float32);
    M (OUTPUT_DAC, FLOAT, output.float32);
    M (OUTPUT_MOTOR, FLOAT, output.float32);
    M (OUTPUT_COM, STRING, output.string);
    /* Modbus types */
    M (INPUT_I1, BINARY, input.binary);
    M (INPUT_I16, INT16, input.int16);
    M (INPUT_U16, UINT16, input.int16);
    M (INPUT_F32, FLOAT, input.float32);
    M (INPUT_U32, UINT, input.int32);
    M (INPUT_I32, INT, input.int32);
    M (INPUT_F64, DOUBLE, input.float64);
    M (INPUT_U64, ULONG, input.holding64);
    M (INPUT_I64, LONG, input.holding64);
    M (INPUT_STRING, STRING, input.string);
    M (INPUT_SDI, FLOAT, input.sdi);
    M (INPUT_HOLDING, INT16, input.holding);
    M (INPUT_HOLDING_32, FLOAT, input.holding32);
    M (INPUT_HOLDING_64, DOUBLE, input.holding64);
    M (INPUT_COILS, BINARY, input.coils);
    M (OUTPUT_I1, BINARY, output.binary);
    M (OUTPUT_I16, INT16, output.int16);
    M (OUTPUT_U16, UINT16, output.int16);
    M (OUTPUT_F32, FLOAT, output.float32);
    M (OUTPUT_U32, UINT, output.int32);    
    M (OUTPUT_I32, INT, output.int32);    
    M (OUTPUT_U64, ULONG, output.holding64);
    M (OUTPUT_F64, DOUBLE, output.float64);
    M (OUTPUT_I64, LONG, output.holding64);
    M (OUTPUT_STRING, STRING, output.string);
    M (OUTPUT_SDI, FLOAT, output.sdi);
    M (OUTPUT_HOLDING, INT16, output.holding);
    M (OUTPUT_HOLDING_32, FLOAT, output.holding32);
    M (OUTPUT_HOLDING_64, DOUBLE, output.holding64);
#undef M

    /*  case (VHYTEC_TYPE_INPUT_APPREG): {
    if (dbDtype == VDB_DTYPE_BINARY) {
      hwDtype = hwType;
      plcOffset = plc->addrOffset.input.binary;		
      map = (struct map_s*)&plc->map.input.binary;	
    } else {
      hwDtype = hwType;
      plcOffset = plc->addrOffset.input.int16;		
      map = (struct map_s*)&plc->map.input.int16;
    }		
    break;
  }
  case (VHYTEC_TYPE_OUTPUT_APPREG): {
    if (dbDtype == VDB_DTYPE_BINARY) {
      hwDtype = VDB_DTYPE_BINARY;			
      plcOffset = plc->addrOffset.output.binary;		
      map = (struct map_s*)&plc->map.output.binary;	
    } else {
      hwDtype = VDB_DTYPE_INT16;			
      plcOffset = plc->addrOffset.output.int16;		
      map = (struct map_s*)&plc->map.output.int16;
    }		
    break;
  }
    */
  default: 
    vsys_msg_printf(0, "Could not match type to i/o list");
    fatalINVHWTYPE (mydbChannelName (chix));
    break;
  }


    vsys_msg_printf(0, "%s: hwType %d, hwDtype %d", mydbChannelName (chix), hwType, hwDtype);

/*   if ((addr > 0xffff) && (hwDtype != VDB_DTYPE_BINARY)) */
/*     fatalINVADDR (mydbChannelName (chix)); */
  
  /* Determine the address list this channel belongs to. */

  if (in) {
    switch (hwType) {
    case VHYTEC_TYPE_INPUT_DO: list = &plc->addrList.in.i1; break;
    case VHYTEC_TYPE_INPUT_DI: list = &plc->addrList.in.i1; break;
    case VHYTEC_TYPE_INPUT_ADC: list = &plc->addrList.in.i32; break;
    case VHYTEC_TYPE_INPUT_DAC: list = &plc->addrList.in.i32; break;
    case VHYTEC_TYPE_INPUT_MOTOR: list = &plc->addrList.in.i32; break;
    case VHYTEC_TYPE_INPUT_COM: list = &plc->addrList.in.string; break;
      /*    case VHYTEC_TYPE_INPUT_APPREG: 
      if (dbDtype == VDB_DTYPE_BINARY) 
	list = &plc->addrList.in.i1;
      else
      list = &plc->addrList.in.i16; break;*/
    case VHYTEC_TYPE_OUTPUT_DO: list = &plc->addrList.in.o1; break;
    case VHYTEC_TYPE_OUTPUT_DI: list = &plc->addrList.in.o1; break;
    case VHYTEC_TYPE_OUTPUT_ADC: list = &plc->addrList.in.o32; break;
    case VHYTEC_TYPE_OUTPUT_DAC: list = &plc->addrList.in.o32; break;
    case VHYTEC_TYPE_OUTPUT_MOTOR: list = &plc->addrList.in.o32; break;
    case VHYTEC_TYPE_OUTPUT_COM: list = &plc->addrList.in.string; break;
      /*    case VHYTEC_TYPE_OUTPUT_APPREG: 
      if (dbDtype == VDB_DTYPE_BINARY) 
      list = &plc->addrList.in.o1; 
      else
	list = &plc->addrList.in.o16;
	break;*/
    case VHYTEC_TYPE_INPUT_I1: list = &plc->addrList.in.i1; break;
    case VHYTEC_TYPE_INPUT_I16: list = &plc->addrList.in.i16; break;
    case VHYTEC_TYPE_INPUT_U16: list = &plc->addrList.in.i16; break;
    case VHYTEC_TYPE_INPUT_U32: list = &plc->addrList.in.i32; break;
    case VHYTEC_TYPE_INPUT_I32: list = &plc->addrList.in.i32; break;
    case VHYTEC_TYPE_INPUT_F32: list = &plc->addrList.in.i32; break;
    case VHYTEC_TYPE_INPUT_F64: list = &plc->addrList.in.i64; break;
    case VHYTEC_TYPE_INPUT_U64: list = &plc->addrList.in.i64; break;
    case VHYTEC_TYPE_INPUT_I64: list = &plc->addrList.in.i64; break;
    case VHYTEC_TYPE_INPUT_STRING: list = &plc->addrList.in.string; break;
    case VHYTEC_TYPE_INPUT_SDI: list = &plc->addrList.in.sdi; break;
    case VHYTEC_TYPE_INPUT_HOLDING: list = &plc->addrList.in.iholding; break;
    case VHYTEC_TYPE_INPUT_HOLDING_32: list = &plc->addrList.in.iholding_i32; break;
    case VHYTEC_TYPE_INPUT_HOLDING_64: list = &plc->addrList.in.iholding_i64; break;
    case VHYTEC_TYPE_INPUT_COILS: list = &plc->addrList.in.icoils; break;
    case VHYTEC_TYPE_OUTPUT_I1: list = &plc->addrList.in.o1; break;
    case VHYTEC_TYPE_OUTPUT_I16: list = &plc->addrList.in.o16; break;
    case VHYTEC_TYPE_OUTPUT_U16: list = &plc->addrList.in.o16; break;
    case VHYTEC_TYPE_OUTPUT_F32: list = &plc->addrList.in.o32; break;
    case VHYTEC_TYPE_OUTPUT_F64: list = &plc->addrList.in.o64; break;
    case VHYTEC_TYPE_OUTPUT_U32: list = &plc->addrList.in.o32; break;
    case VHYTEC_TYPE_OUTPUT_U64: list = &plc->addrList.in.o64; break;
    case VHYTEC_TYPE_OUTPUT_I32: list = &plc->addrList.in.o32; break;
    case VHYTEC_TYPE_OUTPUT_I64: list = &plc->addrList.in.o64; break;
    case VHYTEC_TYPE_OUTPUT_SDI: list = &plc->addrList.in.osdi; break;
    case VHYTEC_TYPE_OUTPUT_HOLDING: list = &plc->addrList.in.oholding; break;
    case VHYTEC_TYPE_OUTPUT_HOLDING_32: list = &plc->addrList.in.oholding_o32; break;
    case VHYTEC_TYPE_OUTPUT_HOLDING_64: list = &plc->addrList.in.oholding_o64; break;
    default: fatalNOTIMPL ();	/*NCV*/
    }				/*NCV*/
  }
  else {
    switch (hwType) {
    case VHYTEC_TYPE_OUTPUT_DO: list = &plc->addrList.out.o1; break;
    case VHYTEC_TYPE_INPUT_DO: list = &plc->addrList.out.o1; break;
    case VHYTEC_TYPE_OUTPUT_DI: list = &plc->addrList.out.o1; break;
    case VHYTEC_TYPE_OUTPUT_ADC: list = &plc->addrList.out.o32; break;
    case VHYTEC_TYPE_OUTPUT_DAC: list = &plc->addrList.out.o32; break;
    case VHYTEC_TYPE_OUTPUT_MOTOR: list = &plc->addrList.out.o32; break;
    case VHYTEC_TYPE_OUTPUT_COM: list = &plc->addrList.out.ostring; break;
      /*    case VHYTEC_TYPE_OUTPUT_APPREG: 
      if(dbDtype == VDB_DTYPE_BINARY) 
	list = &plc->addrList.out.o1; 
      else
	list = &plc->addrList.out.o16;
	break;*/
    case VHYTEC_TYPE_OUTPUT_I1: list = &plc->addrList.out.o1; break;
    case VHYTEC_TYPE_OUTPUT_I16: list = &plc->addrList.out.o16; break;
    case VHYTEC_TYPE_OUTPUT_U16: list = &plc->addrList.out.o16; break; /*NYCV*/
    case VHYTEC_TYPE_OUTPUT_F32: list = &plc->addrList.out.o32; break;
    case VHYTEC_TYPE_OUTPUT_U32: list = &plc->addrList.out.o32; break;
    case VHYTEC_TYPE_OUTPUT_I32: list = &plc->addrList.out.o32; break;
    case VHYTEC_TYPE_OUTPUT_F64: list = &plc->addrList.out.o64; break;
    case VHYTEC_TYPE_OUTPUT_U64: list = &plc->addrList.out.o64; break;
    case VHYTEC_TYPE_OUTPUT_I64: list = &plc->addrList.out.o64; break;
    case VHYTEC_TYPE_OUTPUT_STRING: list = &plc->addrList.out.ostring; break;
    case VHYTEC_TYPE_OUTPUT_SDI: list = &plc->addrList.out.osdi; break;
    case VHYTEC_TYPE_OUTPUT_HOLDING: list = &plc->addrList.out.oholding; break;
    case VHYTEC_TYPE_OUTPUT_HOLDING_32: list = &plc->addrList.out.oholding_o32; break;
    case VHYTEC_TYPE_OUTPUT_HOLDING_64: list = &plc->addrList.out.oholding_o64; break;
    default: fatalINVINOUT (mydbChannelName (chix));	/*NYCV*/
    }
  }

  /* Adjust address based on -a32 option */

  /*  if (list->dsizeBits >= 32 && plc->a32 == 1 && ! plc->enron) {
    switch (list->dsizeBits) {
    case 32: addr *= 2; break;
    case 64: addr *= 4; break;
    default: fatalNOTIMPL ();
    }				
    }*/

  /*
  if (plc->map.listSize == 0) {
    if (plcOffset > 0) {
      if (list->dsizeBits == 1 && (plc->d16))
	addr += plcOffset * 16;
      else
	addr += plcOffset;
    }
  }
  else if (map->totalCount <= addr)
    fatalADDRMAP (mydbChannelName (chix));
  else {				
    struct mapEntry_s* e;
    for (e = (struct mapEntry_s*)map->entry; ; e = e->next) {
      if (e->count <= addr)
	addr -= e->count;
      else {
	unsigned offset = e->offset + plc->map.offset;
	if (list->dsizeBits == 1)
	  offset *= 16;
	mapEntry = e;
	addr += offset;
	break;
      }
    }
  }
*/
  /* Enron 32-bit floating points */

  /*  if (plc->enron) {
    if (hwType == VHYTEC_TYPE_INPUT_F32 ||
	hwType == VHYTEC_TYPE_OUTPUT_F32) {
      if (addr < 6999 || addr > 7998) { 
	logMsg ("%s addr = %d, but float addresses must be between 7000 and 7999", mydbChannelName (chix), addr);
	fatalINTERR ();
      }
      else {
	addr++;
      }
    }
    }*/

  /* Find the position of the new channel in the list. */

  {
    unsigned x = 0;
    int l = 0;
    int r = (int) list->numChannels - 1;

    chan = list->chan;

    if (r >= 0) {
      while (r >= l) {
	x = (l + r) / 2;
	if (addr < chan[x].pub.addr)
	  r = x - 1;
	else if (addr > chan[x].pub.addr)
	  l = ++x;
	else
	  break;
      }
    }

    if (list->numChannels >= list->numChannelsAllocated) {
      unsigned n = list->numChannelsAllocated + 100;
      list->numChannelsAllocated = n;
      list->chan = (Channel* )fatalRealloc (list->chan, sizeof (list->chan[0]) * n);
    }

    if (x != list->numChannels) {
      unsigned n = list->numChannels - x;
      (void) memmove (&list->chan[x + 1], &list->chan[x],
		      sizeof (list->chan[0]) * n);
    }

    chan = list->chan + x;
  }

  (void) memset (chan, 0, sizeof *chan);
  chan->pub.addr = addr;
  chan->pub.chix = *chix;
  chan->pub.thermType = thermType;
  chan->pub.hwDtype = hwDtype;
  chan->pub.dbDtype = dbDtype;
  chan->pub.appreg = appreg;
  chan->pub.appregType = appregType;
  chan->pub.id = data.env.numChannels++;
  chan->mapEntry = mapEntry;
  chan->databaseIndex = databaseIndex;
  chan->conversionRoutine = conversionRoutine;
  if (in)
    chan->flags.putInitialValue = 1;
  chan->flags.ioError = mydbGetIOError (chix);
  setChannelDisposition (chan, disposition, in);
  list->numChannels++;

  if (list->isInput) {
    plc->numInputChannels++;
    port->numInputChannels++;
  }
  else {
    plc->numOutputChannels++;
    port->numOutputChannels++;
  }
}

/*
 *********************************************************************
 */

static void buildAddrLists (void) {

  vista_chix chix;
  unsigned i;
  Plc* plc;
  struct vdb_descrip_s desc; 

  for (plc = data.plcList; plc != NULL; plc = plc->next) {

#define M(name, _type, _isInput, _func1, _func0, _dsizeBits, _descr)	\
  plc->addrList.name.descr = _descr;					\
  plc->addrList.name.dsizeBits = _dsizeBits;				\
  plc->addrList.name.isInput = _isInput;				\
  plc->addrList.name.plc = plc;						\
  plc->addrList.name.type = _type;					\
  plc->addrList.name.func = _func0 

    /* List of functions; function list; function codes */
    /*    M (in.i1,	VHYTEC_TYPE_INPUT_DI,   1, 2, 3, 1, "1-bit input discretes");
    M (in.o1,	VHYTEC_TYPE_OUTPUT_DO, 1, 1, 3, 1, "1-bit output discretes");
    M (in.i32,	VHYTEC_TYPE_INPUT_ADC,  1, 4, 3, 32, "32-bit input words");
    M (in.string, VHYTEC_TYPE_INPUT_COM,  1, 3, 3, sizeof(char*), "input string");
    M (out.o32,	VHYTEC_TYPE_OUTPUT_DAC, 1, 3, 3, 32, "32-bit output words");
#undef M
}*/
    M (in.i1,	LIST_TYPE_INPUT_R1,   1, 2, 3, 1, "1-bit input discretes");
    M (in.i16,	LIST_TYPE_INPUT_R16,  1, 4, 3, 16, "16-bit input registers");
    M (in.i32,	LIST_TYPE_INPUT_R32,  1, 4, 3, 32, "32-bit input words");
    M (in.i64,	LIST_TYPE_INPUT_R64,  1, 4, 3, 64, "64-bit input words");
    M (in.string, LIST_TYPE_INPUT_STRING,  1, 3, 3, sizeof(char*), "input string");
    M (in.sdi, LIST_TYPE_INPUT_SDI,  1, 3, 3, sizeof(char*), "input SDI");
    M (in.iholding, LIST_TYPE_INPUT_HOLDING, 1, 3, 3, 16, "input Holding");
    M (in.iholding_i32, LIST_TYPE_INPUT_HOLDING_R32, 1, 3, 3, 32, "input Holding 32");
    M (in.iholding_i64, LIST_TYPE_INPUT_HOLDING_R32, 1, 3, 3, 64, "input Holding 64");
    M (in.icoils, LIST_TYPE_INPUT_COILS, 1, 1, 1, 1, "1-bit input Coils");
    M (in.o1,	LIST_TYPE_OUTPUT_R1, 1, 1, 3, 1, "1-bit output discretes");
    M (in.o16,	LIST_TYPE_OUTPUT_R16, 1, 3, 3, 16, "16-bit output registers");
    M (in.o32,	LIST_TYPE_OUTPUT_R32, 1, 3, 3, 32, "32-bit output words");
    M (in.o64,	LIST_TYPE_OUTPUT_R64, 1, 3, 3, 64, "64-bit output words");
    M (in.ostring, LIST_TYPE_OUTPUT_STRING, 1, 3, 3, sizeof(char*), "output string");
    M (in.osdi, LIST_TYPE_OUTPUT_SDI, 1, 3, 3, sizeof(char*), "output SDI");
    M (in.oholding, LIST_TYPE_OUTPUT_HOLDING, 1, 16, 16, 16, "output Holding");
    M (in.oholding_o32, LIST_TYPE_OUTPUT_HOLDING_R32, 1, 16, 16, 32, "output Holding 32");
    M (in.oholding_o64, LIST_TYPE_OUTPUT_HOLDING_R64, 1, 16, 16, 64, "output Holding 64");
    M (out.o1,	LIST_TYPE_OUTPUT_R1,  0, 0, 0, 1, "1-bit output discretes");
    M (out.o16,	LIST_TYPE_OUTPUT_R16, 0, 0, 0, 16, "16-bit output registers");
    M (out.o32,	LIST_TYPE_OUTPUT_R32, 0, 0, 0, 32, "32-bit output words");
    M (out.o64,	LIST_TYPE_OUTPUT_R64, 0, 0, 0, 64, "64-bit output words");
    M (out.ostring, LIST_TYPE_OUTPUT_STRING, 0, 0, 0, sizeof(char *), "output string");
    M (out.osdi, LIST_TYPE_OUTPUT_SDI, 0, 0, 0, sizeof(char *), "output SDI");
    M (out.oholding, LIST_TYPE_OUTPUT_HOLDING, 0, 0, 0, 16, "output Holding");
    M (out.oholding_o32, LIST_TYPE_OUTPUT_HOLDING_R32, 0, 0, 0, 32, "output Holding 32");
    M (out.oholding_o64, LIST_TYPE_OUTPUT_HOLDING_R64, 0, 0, 0, 64, "output Holding 64");
    if (plc->d16) {
      plc->addrList.in.i1.func = 3; 
      plc->addrList.in.o1.func = 3;
    }
#undef M
  }
    
  for (i = 0; i < data.numDatabases; i++) {
    char* templatey = (char *)fatalMalloc (strlen (data.databaseName[i]) + 5);
    (void) sprintf (templatey, "%s::*", data.databaseName[i]);
    (void) vdb_zero_chix (&chix);
    while (vdbc_search (templatey, &chix) == VDB_ERR_OK) {
      char plcName[128];
      if (chix.dbix != i + 1) fatalINTERR ();
      if (mydbGetStringField (&chix, &data.fields.plcName,
			      plcName, sizeof plcName) != NULL) {
	Plc* plc = findPlc (plcName);
	if (plc != NULL)
	  addChannelToAddrList (plc, &chix, i);
      }
    }
    myFree (templatey);
  }

  /* Get chixes for the throughput channels */

  for (plc = data.plcList; plc != NULL; plc = plc->next) {
    if (plc->throughput_channel) {
      VSYS_STRING_FILL(desc, plc->throughput_channel, strlen(plc->throughput_channel) + 1); 
      /*       vsys_msg_printf(0, "Getting first chix"); */
      if (!vdb_channel_index(&desc, &plc->throughput_chix)) 
	{
	  vsys_msg_printf(0, "Warning: could not get channel index for throughput channel %s",
			  plc->throughput_channel);
	  vdb_zero_chix(&plc->throughput_chix);
	}
    }

    if (plc->staledata_channel) {
      VSYS_STRING_FILL(desc, plc->staledata_channel, strlen(plc->staledata_channel) + 1); 
      /*       vsys_msg_printf(0, "Getting first chix"); */
      if (!vdb_channel_index(&desc, &plc->staledata_chix)) 
	{
	  vsys_msg_printf(0, "Warning: could not get channel index for staledata channel %s",
			  plc->staledata_channel);
	  vdb_zero_chix(&plc->staledata_chix);
	}
    }
  }
  
  /*
   * Free extra memory.
   */

#define M(l) do {							  \
  if ((l)->numChannels && (l)->numChannels < (l)->numChannelsAllocated) { \
    (l)->chan = (Channel*)fatalRealloc ((l)->chan, (sizeof ((l)->chan[0]) * \
				       (l)->numChannels));		  \
    (l)->numChannelsAllocated = (l)->numChannels;			  \
  }									  \
} while (0)

  for (plc = data.plcList; plc != NULL; plc = plc->next)
    DO_ALL_LISTS (M);

#undef M

  /* Allocate value buffers. */

  for (plc = data.plcList; plc != NULL; plc = plc->next)
    DO_ALL_LISTS (allocValueBuffer);

  /* Allocate and fill VhytecEnv->channel list. */

  if (data.env.numChannels != 0) {
    data.env.channel = (VhytecChannel**)fatalMalloc0 (data.env.numChannels *
				     sizeof data.env.channel[0]);
#define M(l) do {							\
  if ((l)->numChannels != 0) {						\
    unsigned i;								\
    for (i = 0; i < (l)->numChannels; i++)				\
      data.env.channel [(l)->chan[i].pub.id] = &(l)->chan[i].pub;	\
  }									\
} while (0)

    for (plc = data.plcList; plc != NULL; plc = plc->next)
      DO_ALL_LISTS (M);
#undef M
  }
}

/*
 *********************************************************************
 * fixme1.pf: multiple class=0 data types can be combined into a single scan
 *********************************************************************
 */

static void buildScanLists (void) {

  Port* port;
  Plc* plc;

  buildAddrLists ();

  for (plc = data.plcList; plc != NULL; plc = plc->next) {
    DO_INPUT_LISTS (getScanRates);
    if (data.showPlcs)
      showPlc (plc);
    DO_INPUT_LISTS (addrToScanLists);
  }

  for (port = data.portList; port != NULL; port = port->next) {
    int x;
    for (x = 0; x < 100; x++) 
      port->PLCs_to_scan[x] = 666;
    if (port->numInputChannels != 0 || port->numOutputChannels != 0)
      data.numPortsWithChannels++;
    if (data.showScanLists)
      showScanLists (port);
    allocPortBuffer (port);
  }

  if (data.showPorts)
    showPorts ();

  if (data.numDatabases == 0) fatalNODB ();
  if (data.numPortsWithChannels == 0) fatalNOCHANNELS ();
}

/*
 *********************************************************************
 * This function is invoked from user conversion routines to compare
 * the version of the user module against the version of the scanner
 * itself.
 *********************************************************************
 */

static int checkVersion (VhytecEnv* env, int version) {
  if (version != VHYTEC_H_VERSION)
    fatalUSERVERSION ();		/*NCV*/
  return VHYTEC_OK;
}

/*
 *********************************************************************
 * If specified, called the user initialization routine.
 *********************************************************************
 */

static void callUserInitRoutine (void) {

  data.env.checkVersion = checkVersion;

  if (data.userInitRoutine != NULL) {
    int (*routine) (VhytecEnv* env) = NULL;
    int status;
    if (data.userFiles != NULL) {
      routine =
	(int(*)(VhytecEnv*))
	vsys_lib_find_routine (data.userInitRoutine, data.userFiles);
    }
    if (routine == NULL) fatalNOROUTINE (data.userInitRoutine);
    status = routine (&data.env);
    if (status != VHYTEC_OK) fatalUSERSTATUS (status);
  }
}

/*
 *********************************************************************
 *********************************************************************
 ***
 ***  Scanner
 ***
 *********************************************************************
 *********************************************************************
 */

static void setInitialValueFlag_PLCK (AddrList* list) {
  if (list->numChannels) {
    Channel* chan = list->chan;
    unsigned i;
    for (i = 0; i < list->numChannels; i++, chan++)
      chan->flags.putInitialValue = 1;
  }
}

/*
 *********************************************************************
 * Clear I/O error field of the specified channel.
 *********************************************************************
 */

static void clearIOError (Channel* chan, vtime* currentTime, unsigned count) {

  while (1) {
    if (chan->flags.ioError != 0) {
      if (VARC_CHAN (chan)) {
	int zero = 0;						/*NYCV*/
	VARC_LOCK ();						/*NYCV*/
	(void) vlogc_log_field (data.varc.log, currentTime,	/*NYCV*/
			 chan->varcChannel,			/*NYCV*/
			 chan->varcIOErrorField, &zero);	/*NYCV*/
	VARC_UNLOCK ();						/*NYCV*/
      }
      if (DBPUT_CHAN (chan)) {
	vista_chix* chix = &chan->pub.chix;
	struct vdb_descrip_v desc;
	int value = 0;
	VSYS_INTEGER_FILL (desc, value, sizeof value);
	if (vdb_put (chix, VFLD_M_ENTLVE, VFLD_M_IO_ERROR,
		     &desc) != VDB_ERR_OK)
	  errorVDBCHIX (chix);	/*NYCV*/
      }
      chan->flags.ioError = 0;
    }

    if (--count == 0)
      break;

    chan++;	/*NYCV*/
  }		/*NYCV*/
}

/*
 *********************************************************************
 * Set I/O error field of the specified channels.
 *********************************************************************
 */

static void setIOError (Port* port, Channel* chan, unsigned count) {

  vtime currentTime;
  bool exitFlag;

  (void) vsys_get_time (&currentTime);

  /* FIXME: I added these locks, but there is a setIOError call from
     locked state. I either need to fix that place of do something else
     about these locks. */
  /*FIXME*/ /* lockGlobal (); */
  exitFlag = data.exitFlag;
  /*FIXME*/ /* unlockGlobal (); */

  while (1) {
    if (chan->flags.ioError == 0) {

      /* We may come here after a communication timeout, when the exitFlag
	 has long been set, in which case the current timestamp may be
	 much beyond the stop mark (the wakeupScanners time). In that
	 case keeping the log file's max_delta in better shape seems
	 to be better idea than logging the IO_ERROR field. */

      /* fixme2: Archiving IO_ERROR needs work. There are several problems:
	 - Why do I archive IO_ERROR in the first place ? Does Mike Castle
	 use it. Vtrend does not seem to use IO_ERROR any more.
	 - If IO_ERROR is archived, then all IO_ERROR fields should probably
	 be archive at the beginning of each interval.
	 - What to do upon restart ? How do we know what was the last IO_ERROR
	 archive prior to restart ? */

      if (! exitFlag && VARC_CHAN (chan)) {
	int one = 1;						/*NYCV*/
	VARC_LOCK ();						/*NYCV*/
	(void) vlogc_log_field (data.varc.log, &currentTime,	/*NYCV*/
				chan->varcChannel,		/*NYCV*/
				chan->varcIOErrorField, &one);	/*NYCV*/
	VARC_UNLOCK ();						/*NYCV*/
      }

      if (DBPUT_CHAN (chan)) {
	vista_chix* chix = &chan->pub.chix;
	if (vdb_hardware_error (chix) != VDB_ERR_OK)
	  errorVDBCHIX (chix);	/*NCV*/
      }
      chan->flags.ioError = 1;
    }

    if (--count == 0)
      break;

    chan++;
  }
}

/*
 *********************************************************************
 */

static void setRetryTimer_PLCK (Plc* plc, struct PlcRetryTimer* rt) {

  Vtimer* timer = rt->timer;
  Port* port = plc->port;
  VtimerBank* bank = port->timerBank;

  if (! timer->isSet) {
    unsigned index = rt->index;
    unsigned delaySec = data.retry.delay[index];
    if (index < data.retry.size - 1)
      rt->index++;
    if (vtimerSet (timer, delaySec) != VTIMER_OK)
      fatalVTIMER (bank);
#if vmbCONF_TRACE_SCAN
    if (strcmp (port->name, "port1") == 0)
      logMsg ("set retry %s sec=%u timer %u,%u",
	      plc->name,
	      delaySec,
	      timer->nextTime.t_sec,
	      timer->nextTime.t_nsec);
      
#endif    
  }
}

/*
 *********************************************************************
 */

static void setInputRetryTimer_PLCK (Plc* plc) {

  setRetryTimer_PLCK (plc, (struct PlcRetryTimer *)(&plc->inputRetryTimer));
}

/*
*********************************************************************
*/

static void setOutputRetryTimer_PLCK (Plc* plc) {
  setRetryTimer_PLCK (plc, (struct PlcRetryTimer *)(&plc->outputTimer));
}

/*
 *********************************************************************
 */

static void incNumPlcFailedPorts (Port* port) {
  lockGlobal ();
  data.numPlcFailedPorts++;
  unlockGlobal ();
}

/*
 *********************************************************************
 */

static void setAllPortsAreOK (Port* port) {
  unsigned count;
  lockGlobal ();
  count = --data.numPlcFailedPorts;
  unlockGlobal ();
  if (count == 0) {
    /* Generally there should be no commitments to the format and
       the contents of error messages. But in this case we did make
       a commitment - Mike Castle expects that this message does
       not have the word 'error' in it. */
//    logMsg ("All ports are OK");
  }
}

/*
 *********************************************************************
 */

static void setBlockInputFailed (ScanBlock* b) {

  Plc* plc = b->plc;
  Port* port = plc->port;
  float num = 0;
  vsys_msg_printf(0, "Block input failed"); 

  if (b->inputFailed) {
    /* CML added the following -- normally we would just return here, but to keep 
       the throughput count correct we need to count retries as well. */
/*     vsys_msg_printf(0, "Incrementing numFailedScans #1!");  */
    plc->numFailedScans++;
     plc->runningFailedScans++; 
    if (plc->numScans > 0) {
      num = ((plc->numScans - plc->numFailedScans) / plc->numScans) * 100;
    }
 /*         vsys_msg_printf(0, "Now figuring throughput: total %d failed %d throughput %f",   */
/*     		    (int)plc->numScans, (int)plc->numFailedScans, num);  */
    if (vdb_test_chix(&plc->throughput_chix)) {
      if (plc->numScans > 0) {
	num = ((plc->numScans - plc->numFailedScans) / plc->numScans) * 100;
      } else 
	num = 0;
      if (vdb_rput (&plc->throughput_chix, &num) != VDB_ERR_OK)
	(void) errorWDPUT (&plc->throughput_chix);	       
    }
/*      vsys_msg_printf(0, "Now figuring staledata: total %d\n", plc->runningFailedScans);  */
    if (vdb_test_chix(&plc->staledata_chix)) { 
      if (vdb_iput (&plc->staledata_chix, &(plc->runningFailedScans)) != VDB_ERR_OK) 
 	(void) errorWDPUT (&plc->staledata_chix);	        
     } 
    /* End of addition */
    return; 
  }

  do {
    if (b->plc == plc) {
      /* fixme0: PLCSBLIST */
      AddrList* addrList = b->addrList;
      unsigned numChannels = b->numChannels;
      unsigned offset = b->offset;
      Channel* chan = &addrList->chan[offset];
      b->inputFailed = true;
      if (plc->numInputFailedBlocks++ == 0)
	if (! plc->outputFailed)
	  if (port->numFailedPlcs++ == 0)
	    incNumPlcFailedPorts (port);
      /*       vsys_msg_printf(0, "Setting IO error"); */
/*       vsys_msg_printf(0, "Incrementing numFailedScans #2!");  */
      plc->numFailedScans++;
      plc->runningFailedScans++;
      if (plc->numScans > 0) {
	num = ((float)(plc->numScans - plc->numFailedScans) / plc->numScans) * 100;
      }

/*       		      (int)plc->numScans, (int)plc->numFailedScans, num); */
      if (vdb_test_chix(&plc->throughput_chix)) {
	if (plc->numScans > 0) {
	  num = ((plc->numScans - plc->numFailedScans) / plc->numScans) * 100;
	} else 
	  num = 0;
	if (vdb_rput (&plc->throughput_chix, &num) != VDB_ERR_OK)
	  (void) errorWDPUT (&plc->throughput_chix);	       
      }
/*       vsys_msg_printf(0, "Now figuring staledata: %d\n",  plc->runningFailedScans); */
      if (vdb_test_chix(&plc->staledata_chix)) {
	if (vdb_iput (&plc->staledata_chix, &(plc->runningFailedScans)) != VDB_ERR_OK)
	  (void) errorWDPUT (&plc->staledata_chix);	       
      }
      setIOError (port, chan, numChannels);
#if vmbCONF_LIST_PUTS
      if (b->varcList != NULL)
	sb->ioError = 1;
#endif
    }
  } while ((b = b->next) != NULL);

  lockPort (port);
  setInputRetryTimer_PLCK (plc);
  unlockPort (port);
}

/*
 *********************************************************************
 * This should be called only if block->inputFailed is set.
 *********************************************************************
 */

static void clearBlockInputFailed (ScanBlock* b) {
  Plc* plc = b->plc;
  Port* port = plc->port;
  b->inputFailed = false;
  if (--plc->numInputFailedBlocks == 0)
    if (! plc->outputFailed)
      if (--port->numFailedPlcs == 0)
	setAllPortsAreOK (port);
}

/*
 *********************************************************************
 */

static void scanOneBlock (ScanBlock* b, vtime *currentListTime) {

  AddrList* addrList = b->addrList;
  unsigned numChannels = b->numChannels;
  unsigned offset = b->offset;
  Channel* chan = &addrList->chan[offset];
  int type = addrList->type;
  Plc* plc = addrList->plc;
  Port* port = plc->port;
  vtime currentTime;
  unsigned i;
  char *logbuffer = NULL;
  int status = 0;
  float num = 0;
  char name[800];
  /* End of addition */
/*  vsys_msg_printf(0, "Incrementing numScans!"); */
  plc->numScans++;

  switch (type) {

    /* case VHYTEC_TYPE_INPUT_DI:
       case VHYTEC_TYPE_OUTPUT_DI: {*/
  case LIST_TYPE_INPUT_R1:
  case LIST_TYPE_OUTPUT_R1: 
  case LIST_TYPE_INPUT_COILS: {
    VUINT8* value = addrList->value.binary + b->offset;
    for (i = 0; i < numChannels; i++, chan++) {	
      unsigned addr0 = chan[0].pub.addr;// & ~15;
      unsigned addrDiff = chan->portBufferOffset;
      /*   unsigned byteNumber = addrDiff / 8; */
      /*       unsigned bitNumber = addrDiff & 7; */
      unsigned byteNumber = 0;
      unsigned bitNumber = 0;
      VUINT8 mask = 1 << bitNumber;
      VUINT8 newValue;
      bool ioError = chan->flags.ioError != 0;
      bool dbput;	

      //  vsys_msg_printf(0, "Address %d, channel name %s offset %d numchannels %d, valueptr %d, value %d", chan->pub.addr, mydbChannelName (&(chan->pub.chix)), b->offset, numChannels, value, *value);
      /*  vsys_msg_printf(0, "Address %d pub.addr %d count %d function %d\n", addr0, chan[0].pub.addr, b->count, addrList->func); */ 
      
      if (mymbBget (plc, port->buffer.u8, chan, b->count, addrList->func)) {
	/*       vsys_msg_printf(0, "Input failed!\n"); */
	setBlockInputFailed (b); 
	return; 
      } else { /* CML added the else to keep things from getting hung up on binaries */
	plc->runningFailedScans = 0;
	newValue = (port->buffer.u8[byteNumber] & mask) ? 1 : 0;

	/*       vsys_msg_printf(0, "Clearing runningfailedScans: %d\n", plc->runningFailedScans); */
	
	/* We get the timestamp here rather than before reading from hardware
	   because if he have multiple ports and and reading from multiple
	   port takes different time, then timestamps in the archive will
	   jump back and forth. Below we use the same timestamp for multiple
	   puts and do locks/unlocks between puts, so it is still possible
	   that timestamps will just back and forth, but at least they
	   should not jump as much as they could if we used timestamp before
	   the get from hardware. */
	
	(void) vsys_get_time (&currentTime);
	
#if vmbCONF_LIST_PUTS
	if (b->varcList != NULL) {
	  if (b->conversionRequired) {
	  }
	  VARC_LOCK ();
	  if (vlogLogList (data.varc.log, &currentTime,
			   b->varcList, port->buffer.u8) != 0)
	    fatalVARC ();
	  VARC_UNLOCK ();
	  if (b->ioError) {
	    clearIOError (chan, &currentTime, numChannels);
	    b->ioError = 0;
	  }
	  break;
	}
	
#endif
	
	//  for (i = 0;;) {
	dbput = (newValue != *value ||
		 chan->flags.putInitialValue ||
		 ioError);

	if (dbput || chan->refreshVarcTime < currentTime.t_sec) {
	  
	  int dbDtype = chan->pub.dbDtype;
	  Value u;
	  
	  chan->flags.putInitialValue = 0;
	  u.binary = newValue;
	  
	  if (dbDtype != chan->pub.hwDtype || chan->conversionRoutine != NULL)
	    (void) intToExt (chan, &u);
	  
	  if (dbDtype == VDB_DTYPE_BINARY) {
	    dbDtype = VDB_DTYPE_INT8;
	    u.int8 = u.binary;
	  }
	  
	  if (ioError)
	    clearIOError (chan, &currentTime, 1);
	  
	  if (DBPUT_CHAN (chan) && dbput) {
	    /* CML added this to allow timestamps to use initial scan time */
	    if (data.group_timestamps) {
	      if (vdbc_vput (&chan->pub.chix, &u, currentListTime, dbDtype,
			     VDB_INTERNAL) != VDB_ERR_OK)
		errorVDBCHIX (&chan->pub.chix); /*NCV*/
	    } else {
	      if (vdbc_vput (&chan->pub.chix, &u, NULL, dbDtype,
			     VDB_INTERNAL) != VDB_ERR_OK)
		errorVDBCHIX (&chan->pub.chix); /*NCV*/
	    }
	  }
	  
	  if (VARC_CHAN (chan)) {
	    VARC_LOCK ();
	    if (vlogc_log_channel (data.varc.log, &currentTime,
				   chan->varcChannel, &u) != 1)
	      fatalVARC ();	/*NCV*/
	    VARC_UNLOCK ();
	    if (chan->refreshRate == 0)
	      chan->refreshVarcTime = MAX_REFRESH_VARC_TIME;
	    else if (chan->refreshRate == (unsigned) -1)
	      chan->refreshVarcTime = 0;
	    else
	      chan->refreshVarcTime = currentTime.t_sec + chan->refreshRate;
	  }
	  
	  *value = newValue;
	}
	value++;

	/*  if (++i >= numChannels)
	    break;
	    
	    chan++;
	    value++;
	    }*/
      }
    }
      break;
  } /* CML added this */
 
#define C16()					\
  if (dbDtype != chan->pub.hwDtype ||		\
      chan->conversionRoutine != NULL)		\
    (void) intToExt (chan, &u)

#ifdef VAXFLOAT
#define C3264()					\
  intToExt (chan, &u)
#else
#define C3264()					\
  if (dbDtype != chan->pub.hwDtype ||		\
      chan->conversionRoutine != NULL)		\
    (void) intToExt (chan, &u)
#endif

#if vmbCONF_LIST_PUTS
#define ML2(nn)							\
  if (b->varcList != NULL) {					\
    if (b->conversionRequired) {				\
    }								\
    VARC_LOCK ();						\
    FIXME: port->buffer.u##n would not work if there are gaps;	\
    (if -map is defined); 					\
    if (vlogLogList (data.varc.log, &currentTime,		\
		     b->varcList, port->buffer.u##nn) != 0)	\
      fatalVARC ();						\
    VARC_UNLOCK ();						\
    if (b->ioError) {						\
      clearIOError (chan, &currentTime, numChannels);		\
      b->ioError = 0;						\
    }								\
    break;							\
  }
#else
#define ML2(nn)
#endif

    /* CML added this to get the log buffer from Control Design PLCs */
   case LIST_TYPE_INPUT_STRING:						 
   case LIST_TYPE_OUTPUT_STRING: {	 
/*   case VHYTEC_TYPE_OUTPUT_COM: { */
    char *value = NULL;		
    char *buff = NULL;
    char *buffer_copy = NULL;
    char *target_hit = NULL;
    int status = 0;
    int dbput = 1;
    for (i = 0; i < numChannels; i++, chan++) {
      bool ioError = chan->flags.ioError != 0;
      value = (char *)malloc(sizeof(char) * 5000); 
      
      //  vsys_msg_printf(0, "Before string get: value ptr is %x\n", value);
      status = mymbStringget (plc, value, chan,	
			      b->count, addrList->func);
      //   vsys_msg_printf(0, "SCAN:Back from string get\n");
      //  vsys_msg_printf(0, "SCAN:back, value ptr: %x, %s\n", value, value);
      /*   if (status) {
	   if (status != 666 || value == NULL)
	   setBlockInputFailed (b);						
	   return;								
	   }	*/								
      plc->runningFailedScans = 0;
      /*       vsys_msg_printf(0, "Clearing runningfailedScans: %d\n", plc->runningFailedScans); */
      
      (void) vsys_get_time (&currentTime);				
      ML2 (nn);								
      //vsys_msg_printf(0, "Before malloc\n");
      buff = (char *)malloc(sizeof(char) * (strlen(value) + 5));
      /* strcpy (buff, value);
	 buffer_copy = buff;*/
      // vsys_msg_printf(0, "Before for\n");
      /*         int loopy;
		 loopy = 0;
		 strcpy (buff, value);
		 buffer_copy = buff;	
		 vsys_msg_printf(0, "SCAN:Before strtok: %s\n", buffer_copy);
		 target_hit = strtok (buffer_copy, "\r\n");
		 if (target_hit == NULL) {
		 buffer_copy = buff;	
		 vsys_msg_printf(0, "SCAN:Before strtok #2: %s\n", buffer_copy);
		 target_hit = strtok (buffer_copy, "\n");
		 loopy = 1;
		 }
		 while (target_hit != NULL) {
		 bool ioError = chan->flags.ioError != 0;				
		 unsigned j = chan->portBufferOffset;				
	
		 
		 vsys_msg_printf(0, "SCAN:Found a string: %s\n", target_hit); 
      */
      if (dbput || chan->refreshVarcTime < currentTime.t_sec) {		
	
	int dbDtype = chan->pub.dbDtype;				
	
	chan->flags.putInitialValue = 0;				
	
	/* 	conversionMacro ();						 */
	
	if (ioError)							
	  clearIOError (chan, &currentTime, 1);				
	
	if (DBPUT_CHAN (chan) && dbput) {				
	  /* CML added this to allow timestamps to use initial scan time */       
	  //    vsys_msg_printf(0, "Before put\n");
	  if (data.group_timestamps) {    
	    if (vdbc_vput (&chan->pub.chix, target_hit, currentListTime, dbDtype, 
			   VDB_INTERNAL) != VDB_ERR_OK)                 
	      errorVDBCHIX (&chan->pub.chix); /*NCV*/                   
	  } else {                                                      
	    if (vdbc_vput (&chan->pub.chix, target_hit, NULL, dbDtype,		
			   VDB_INTERNAL) != VDB_ERR_OK)		       
	      errorVDBCHIX (&chan->pub.chix);				
	  }                                                             
	}								
	
	if (VARC_CHAN (chan)) {						
	  VARC_LOCK ();							
	  if (vlogc_log_channel (data.varc.log, &currentTime,		
				 chan->varcChannel, target_hit) != 1)		
	    fatalVARC ();	/*NCV*/				       
	  VARC_UNLOCK ();						
	  if (chan->refreshRate == 0)					
	    chan->refreshVarcTime = MAX_REFRESH_VARC_TIME;	       
	  else if (chan->refreshRate == (unsigned) -1)			
	    chan->refreshVarcTime = 0;					
	  else								
	    chan->refreshVarcTime =					
	      currentTime.t_sec + chan->refreshRate;			
	}								
	
      }									
      /*vsys_msg_printf(0, "SCAN:Looping\n");
	if (loopy == 1)
	target_hit = strtok(NULL, "\n");
	else
	target_hit = strtok(NULL, "\r\n");
      */
    }	
    break;	
  }

    /*    vsys_msg_printf(0, "SCAN:Cleaning up\n");
    if (value)
      myFree(value);
    if (buff)
    myFree(buff);*/
  

  /* End of addition */


  case LIST_TYPE_INPUT_R16:						 
  case LIST_TYPE_OUTPUT_R16: {					 

    bool ioError, dbput;                                                
    unsigned j;                                                         
    for (i = 0; i < numChannels; i++, chan++) {				
      
      VUINT16* value = addrList->value.int16 + offset;		
      
      if (mymbI16get (plc, port->buffer.u16, chan,	                
		      b->count, addrList->func)) {			
	setBlockInputFailed (b);						
	return;								
      }		 							

      plc->runningFailedScans = 0;					
      (void) vsys_get_time (&currentTime);				
      ML2 (16);								
      
      ioError = chan->flags.ioError != 0;				
      j = chan->portBufferOffset;				
      dbput = (value[i] != port->buffer.u16[j] ||			
	       chan->flags.putInitialValue ||			
	       ioError);						
      if (dbput || chan->refreshVarcTime < currentTime.t_sec) {		
	
	int dbDtype = chan->pub.dbDtype;				
	Value u;							
	
	chan->flags.putInitialValue = 0;				
	u.int16 = port->buffer.u16[j];				
	C16();
	
	if (dbDtype == VDB_DTYPE_BINARY) {				
	  dbDtype = VDB_DTYPE_INT8;					
	  u.int8 = u.binary == 0 ? 0 : 1;				
	}								
	if (ioError)							
	  clearIOError (chan, &currentTime, 1);				
	
	if (DBPUT_CHAN (chan) && dbput) {				
	       
	  vdbc_channel_name(&chan->pub.chix, name, 800);	

	  if (data.group_timestamps) {                                  
	    if (vdbc_vput (&chan->pub.chix, &u, currentListTime, dbDtype,  
			   VDB_INTERNAL) != VDB_ERR_OK)                 
	      errorVDBCHIX (&chan->pub.chix);                  
	  } else {                                                      
	    if (vdbc_vput (&chan->pub.chix, &u, &currentTime, dbDtype,		
			   VDB_INTERNAL) != VDB_ERR_OK)			
	      errorVDBCHIX (&chan->pub.chix);				
	  }                                                             
	}								
	if (VARC_CHAN (chan)) {						
	  VARC_LOCK ();							
	  if (vlogc_log_channel (data.varc.log, &currentTime,		
				 chan->varcChannel, &u) != 1)		
	    fatalVARC ();      				
	  VARC_UNLOCK ();						
	  if (chan->refreshRate == 0)					
	    chan->refreshVarcTime = MAX_REFRESH_VARC_TIME;		
	  else if (chan->refreshRate == (unsigned) -1)			
	    chan->refreshVarcTime = 0;					
	  else								
	    chan->refreshVarcTime =					
	      currentTime.t_sec + chan->refreshRate;		       
	}								
	
	value[i] = port->buffer.u16[j];				
      }									
    }									
    break;								
    }
  
   case LIST_TYPE_INPUT_R32:						 
   case LIST_TYPE_OUTPUT_R32: {					 
/*   case VHYTEC_TYPE_INPUT_ADC: {	 */					

    bool ioError, dbput;                                                
    unsigned j;       
    //  vsys_msg_printf(0,"Scanning for %d floats\n", numChannels);
    for (i = 0; i < numChannels; i++, chan++) {				
      int dbDtype = chan->pub.dbDtype;
      VUINT32* value = addrList->value.int32 + offset;		
      float* floatval = addrList->value.float32 + offset;

      if (chan->pub.hwDtype == VDB_DTYPE_FLOAT) {
      vsys_msg_printf(0,"About to call mymbI32get_float");
	if (mymbI32get_float (plc, port->buffer.float32, chan,	                
			      b->count, addrList->func)) {			
	  setBlockInputFailed (b);						
	  continue;								
	}
        } else {
	vsys_msg_printf(0,"About to call mymbI32get");
	if (mymbI32get (plc, port->buffer.u32, chan,	                
			b->count, addrList->func)) {			
	  setBlockInputFailed (b);						
	  continue;								
	}
      }				
      //  vsys_msg_printf(0,"Back from mymbI32get_float");
      plc->runningFailedScans = 0;					
      (void) vsys_get_time (&currentTime);				
      ML2 (32);								
      //   vsys_msg_printf(0,"Back from mymbI32get_float, set ioerror");
      ioError = chan->flags.ioError != 0;				
      j = chan->portBufferOffset;	
      if (chan->pub.hwDtype == VDB_DTYPE_FLOAT) {
	dbput = (floatval[i] != port->buffer.float32[j] ||  
		 chan->flags.putInitialValue ||			
		 ioError);		
      } else {
	dbput = (value[i] != port->buffer.u32[j] ||			
		 chan->flags.putInitialValue ||			
		 ioError);
      }				
      // vsys_msg_printf(0,"Back from mymbI32get_float, dbput is %d, offset %d", dbput, j);
      if (dbput || chan->refreshVarcTime < currentTime.t_sec) {		
	
	int dbDtype = chan->pub.dbDtype;				
	Value u;
	float floaty;
	float floaty2;
	VUINT32 inty;
	//	vsys_msg_printf(0,"entering putting value\n");
	chan->flags.putInitialValue = 0;
	if (chan->pub.hwDtype == VDB_DTYPE_FLOAT) {
	    floaty = port->buffer.float32[j];
	    vsys_msg_printf(0, "floaty %li", floaty);
	  } else {
	  u.int32 = port->buffer.u32[j]; 
	  floaty = port->buffer.u32[j]; 
	    //	    vsys_msg_printf(0, "inty %il", inty);
	  vsys_msg_printf(0, "value %li", u.int32);
	  }

	C3264();
	
	if (dbDtype == VDB_DTYPE_BINARY) {				
	  dbDtype = VDB_DTYPE_INT8;					
	  inty = u.binary == 0 ? 0 : 1;				
	}								
	if (ioError)							
	  clearIOError (chan, &currentTime, 1);				
	
	if (DBPUT_CHAN (chan) && dbput) {				
	  vdbc_channel_name(&chan->pub.chix, name, 800);	
	  if (data.group_timestamps) {                                  
	    if (dbDtype == VDB_DTYPE_FLOAT) {
	      vsys_msg_printf(0, "About to put float %f", floaty);
	      if (vdb_internal_rput_ts(&chan->pub.chix, &floaty, &currentTime)!= VDB_ERR_OK) 
		errorVDBCHIX (&chan->pub.chix);      
	    } else {
	      vsys_msg_printf(0, "About to put other %li, type %d", u.int32, dbDtype);
	      if (vdbc_vput (&chan->pub.chix, &u, currentListTime, dbDtype,  
			     VDB_INTERNAL) != VDB_ERR_OK)                 
		errorVDBCHIX (&chan->pub.chix);
	    }            
	  } else {                                                      
	    //	    if (vdbc_vput (&chan->pub.chix, &u, NULL, VDB_DTYPE_FLOAT,  

	    if (dbDtype == VDB_DTYPE_FLOAT) {
	      vsys_msg_printf(0, "About to put float %f", floaty);
	      if (vdb_internal_rput_ts(&chan->pub.chix, &floaty, &currentTime) != VDB_ERR_OK)
		errorVDBCHIX (&chan->pub.chix);
	    } else {
	      vsys_msg_printf(0, "About to put other %li, type %d", u.int32, dbDtype);
	      if (vdbc_vput (&chan->pub.chix, &u, NULL, dbDtype,  
			     VDB_INTERNAL) != VDB_ERR_OK)                 
		errorVDBCHIX (&chan->pub.chix);	      
	    }

	    vdbc_vget (&chan->pub.chix, &inty, NULL, dbDtype, VDB_INTERNAL);
	    vsys_msg_printf(0, "Value Readback: %li", inty);
	    /*	    if (vdbc_vput (&chan->pub.chix, &u, &currentTime, dbDtype, 
		    VDB_INTERNAL) != VDB_ERR_OK)	*/		
	    //  errorVDBCHIX (&chan->pub.chix);				
	  }                                                             
	}	

	
	if (VARC_CHAN (chan)) {						
	  VARC_LOCK ();					
	  if (dbDtype == DSC_K_DTYPE_F) {
	    if (vlogc_log_channel (data.varc.log, &currentTime,		
				   chan->varcChannel, &floaty) != 1)		
	      fatalVARC ();      				
	  } else {
	    if (vlogc_log_channel (data.varc.log, &currentTime,		
				   chan->varcChannel, &inty) != 1)		
	      fatalVARC ();      				
	  }
	  
	  VARC_UNLOCK ();						
	  if (chan->refreshRate == 0)					
	    chan->refreshVarcTime = MAX_REFRESH_VARC_TIME;		
	  else if (chan->refreshRate == (unsigned) -1)			
	    chan->refreshVarcTime = 0;					
	  else								
	    chan->refreshVarcTime =					
	      currentTime.t_sec + chan->refreshRate;		       
	}								
	if (dbDtype == DSC_K_DTYPE_F) {
	  value[i] = port->buffer.u32[j];
	} else {
	  floatval[i] = port->buffer.float32[j];
	}				
      }									
    }									
    break;								
  }
    
  case LIST_TYPE_INPUT_R64:						
  case LIST_TYPE_OUTPUT_R64: {					
    bool ioError, dbput;                                                
    unsigned j;       
    for (i = 0; i < numChannels; i++, chan++) {				
      
      
      VUINT64* value = addrList->value.int64 + offset;		
      
      if (mymbI64get (plc, port->buffer.u64, chan,	                
		      b->count, addrList->func)) {			
	setBlockInputFailed (b);						
	return;								
      }		 							
      plc->runningFailedScans = 0;					
      (void) vsys_get_time (&currentTime);				
      ML2 (64);								
      
      ioError = chan->flags.ioError != 0;				
      j = chan->portBufferOffset;				
      dbput = (value[i] != port->buffer.u64[j] ||			
	       chan->flags.putInitialValue ||			
	       ioError);						
      if (dbput || chan->refreshVarcTime < currentTime.t_sec) {		
	
	int dbDtype = chan->pub.dbDtype;				
	Value u;
	float floaty;
	
	chan->flags.putInitialValue = 0;
	u.int64 = *(port->buffer.u64);
	floaty = (float)*(port->buffer.u64);

	C3264();
	
	if (dbDtype == VDB_DTYPE_BINARY) {				
	  dbDtype = VDB_DTYPE_INT8;					
	  u.int8 = u.binary == 0 ? 0 : 1;				
	}								

	if (ioError)							
	  clearIOError (chan, &currentTime, 1);				
	
	if (DBPUT_CHAN (chan) && dbput) {				

	       
	  vdbc_channel_name(&chan->pub.chix, name, 800);	

	  if (data.group_timestamps) {                                  

	    if (vdbc_vput (&chan->pub.chix, &u, currentListTime, dbDtype,  
			   VDB_INTERNAL) != VDB_ERR_OK)                 
	      errorVDBCHIX (&chan->pub.chix);                  
	  } else {                                                      

	    vdb_rput_ts(&chan->pub.chix, &floaty, &currentTime); 

	  }                                                             
	}								

	if (VARC_CHAN (chan)) {						
	  VARC_LOCK ();							
	  if (vlogc_log_channel (data.varc.log, &currentTime,		
				 chan->varcChannel, &u) != 1)		
	    fatalVARC ();      				
	  VARC_UNLOCK ();						
	  if (chan->refreshRate == 0)					
	    chan->refreshVarcTime = MAX_REFRESH_VARC_TIME;		
	  else if (chan->refreshRate == (unsigned) -1)			
	    chan->refreshVarcTime = 0;					
	  else								
	    chan->refreshVarcTime =					
	      currentTime.t_sec + chan->refreshRate;		       
	}								
	
	value[i] = port->buffer.u64[j];				
      }									
    }									
    break;								
    }
    
    /*
#define M(nn, conversionMacro)						\
  case LIST_TYPE_INPUT_R##nn:						\
  case LIST_TYPE_OUTPUT_R##nn: {					\
									\
    VUINT##nn* value = addrList->value.int##nn + offset;		\
									\
    if (mymbI##nn##get (plc, port->buffer.u##nn, chan->pub.addr,	\
			b->count, addrList->func)) {			\
      setBlockInputFailed (b);						\
      return;								\
    }									\
    plc->runningFailedScans = 0;					\
    (void) vsys_get_time (&currentTime);				\
    ML2 (nn);								\
									\
    for (i = 0; i < numChannels; i++, chan++) {				\
									\
      bool ioError = chan->flags.ioError != 0;				\
      unsigned j = chan->portBufferOffset;				\
      bool dbput = (value[i] != port->buffer.u##nn[j] ||		\
		    chan->flags.putInitialValue ||			\
		    ioError);						\
									\
      if (dbput || chan->refreshVarcTime < currentTime.t_sec) {		\
									\
	int dbDtype = chan->pub.dbDtype;				\
	Value u;							\
									\
	chan->flags.putInitialValue = 0;				\
	u.int##nn = port->buffer.u##nn[j];				\
									\
	conversionMacro ();						\
									\
	if (dbDtype == VDB_DTYPE_BINARY) {				\
	  dbDtype = VDB_DTYPE_INT8;					\
	  u.int8 = u.binary == 0 ? 0 : 1;				\
	}								\
									\
	if (ioError)							\
	  clearIOError (chan, &currentTime, 1);				\
									\
	if (DBPUT_CHAN (chan) && dbput) {				\
          vdbc_channel_name(&chan->pub.chix, name, 800);\
	  if (data.group_timestamps) {                                  \
	    if (vdbc_vput (&chan->pub.chix, &u, currentListTime, dbDtype,  \
			   VDB_INTERNAL) != VDB_ERR_OK)                 \
	      errorVDBCHIX (&chan->pub.chix);                    \
	  } else {                                                      \
	  if (vdbc_vput (&chan->pub.chix, &u, NULL, dbDtype,		\
			 VDB_INTERNAL) != VDB_ERR_OK)			\
	    errorVDBCHIX (&chan->pub.chix);				\
          }                                                             \
	}								\
									\
	if (VARC_CHAN (chan)) {						\
	  VARC_LOCK ();							\
	  if (vlogc_log_channel (data.varc.log, &currentTime,		\
				 chan->varcChannel, &u) != 1)		\
	    fatalVARC ();						\
	  VARC_UNLOCK ();						\
	  if (chan->refreshRate == 0)					\
	    chan->refreshVarcTime = MAX_REFRESH_VARC_TIME;		\
	  else if (chan->refreshRate == (unsigned) -1)			\
	    chan->refreshVarcTime = 0;					\
	  else								\
	    chan->refreshVarcTime =					\
	      currentTime.t_sec + chan->refreshRate;			\
	}								\
									\
	value[i] = port->buffer.u##nn[j];				\
      }									\
    }									\
    break;								\
  }

  M (16, C16);
  M (32, C3264);
  M (64, C3264);
    */

  /* CML added this to handle holding registers */
#undef M

    /*
#define M(nn, conversionMacro)						\
  case LIST_TYPE_INPUT_HOLDING_R##nn:						\
  case LIST_TYPE_OUTPUT_HOLDING_R##nn: {					\
									\
    VUINT##nn* value = addrList->value.int##nn + offset;		\
									\
    if (mymbI##nn##get (plc, port->buffer.u##nn, chan->pub.addr,	\
			b->count, addrList->func)) {			\
      setBlockInputFailed (b);						\
      return;								\
    }									\
    plc->runningFailedScans = 0;					\
    (void) vsys_get_time (&currentTime);				\
    ML2 (nn);								\
									\
    for (i = 0; i < numChannels; i++, chan++) {				\
									\
      bool ioError = chan->flags.ioError != 0;				\
      unsigned j = chan->portBufferOffset;				\
      bool dbput = (value[i] != port->buffer.u##nn[j] ||		\
		    chan->flags.putInitialValue ||			\
		    ioError);						\
									\
      if (dbput || chan->refreshVarcTime < currentTime.t_sec) {		\
									\
	int dbDtype = chan->pub.dbDtype;				\
	Value u;							\
									\
	chan->flags.putInitialValue = 0;				\
	u.int##nn = port->buffer.u##nn[j];				\
									\
	conversionMacro ();						\
									\
	if (dbDtype == VDB_DTYPE_BINARY) {				\
	  dbDtype = VDB_DTYPE_INT8;					\
	  u.int8 = u.binary == 0 ? 0 : 1;				\
	}								\
									\
	if (ioError)							\
	  clearIOError (chan, &currentTime, 1);				\
									\
	if (DBPUT_CHAN (chan) && dbput) {				\
          vdbc_channel_name(&chan->pub.chix, name, 800);\
	  if (data.group_timestamps) {                                  \
	    if (vdbc_vput (&chan->pub.chix, &u, currentListTime, dbDtype,  \
			   VDB_INTERNAL) != VDB_ERR_OK)                 \
	      errorVDBCHIX (&chan->pub.chix);                    \
	  } else {                                                      \
	  if (vdbc_vput (&chan->pub.chix, &u, NULL, dbDtype,		\
			 VDB_INTERNAL) != VDB_ERR_OK)			\
	    errorVDBCHIX (&chan->pub.chix);				\
          }                                                             \
	}								\
									\
	if (VARC_CHAN (chan)) {						\
	  VARC_LOCK ();							\
	  if (vlogc_log_channel (data.varc.log, &currentTime,		\
				 chan->varcChannel, &u) != 1)		\
	    fatalVARC ();					\
	  VARC_UNLOCK ();						\
	  if (chan->refreshRate == 0)					\
	    chan->refreshVarcTime = MAX_REFRESH_VARC_TIME;		\
	  else if (chan->refreshRate == (unsigned) -1)			\
	    chan->refreshVarcTime = 0;					\
	  else								\
	    chan->refreshVarcTime =					\
	      currentTime.t_sec + chan->refreshRate;			\
	}								\
									\
	value[i] = port->buffer.u##nn[j];				\
      }									\
    }									\
    break;								\
  }
    */
    //  M (32, C3264);
    // M (64, C3264);

  /* CML added this to handle holding registers */
    /*  case LIST_TYPE_INPUT_HOLDING:						
  case LIST_TYPE_OUTPUT_HOLDING: {					
									
    VUINT16* value = addrList->value.int16 + offset;		
									
    if (mymbI16get (plc, port->buffer.u16, chan,	
			b->count, addrList->func)) {			
      setBlockInputFailed (b);						
      return;								
    }									
    plc->runningFailedScans = 0;

    (void) vsys_get_time (&currentTime);				
    ML2 (16);								
									
    for (i = 0; i < numChannels; i++, chan++) {				
									
      bool ioError = chan->flags.ioError != 0;				
      unsigned j = chan->portBufferOffset;				
      bool dbput = (value[i] != port->buffer.u16[j] ||		
		    chan->flags.putInitialValue ||			
		    ioError);						
									
      if (dbput || chan->refreshVarcTime < currentTime.t_sec) {		
									
	int dbDtype = chan->pub.dbDtype;				
	Value u;							
									
	chan->flags.putInitialValue = 0;				
	u.int16 = port->buffer.u16[j];				
									
	C16 ();						
									
	if (dbDtype == VDB_DTYPE_BINARY) {				
	  dbDtype = VDB_DTYPE_INT8;					
	  u.int8 = u.binary == 0 ? 0 : 1;				
	}								
									
	if (ioError)							
	  clearIOError (chan, &currentTime, 1);				
									
	if (DBPUT_CHAN (chan) && dbput) {				
          vdbc_channel_name(&chan->pub.chix, name, 800);
	  if (data.group_timestamps) {                                  
	    if (vdbc_vput (&chan->pub.chix, &u, currentListTime, dbDtype,  
			   VDB_INTERNAL) != VDB_ERR_OK)                 
	      errorVDBCHIX (&chan->pub.chix);                    
	  } else {                                                      
	  if (vdbc_vput (&chan->pub.chix, &u, NULL, dbDtype,		
			 VDB_INTERNAL) != VDB_ERR_OK)			
	    errorVDBCHIX (&chan->pub.chix);				
          }                                                             
	}								
									
	if (VARC_CHAN (chan)) {						
	  VARC_LOCK ();							
	  if (vlogc_log_channel (data.varc.log, &currentTime,		
				 chan->varcChannel, &u) != 1)		
	    fatalVARC ();					
	  VARC_UNLOCK ();						
	  if (chan->refreshRate == 0)					
	    chan->refreshVarcTime = MAX_REFRESH_VARC_TIME;		
	  else if (chan->refreshRate == (unsigned) -1)			
	    chan->refreshVarcTime = 0;					
	  else								
	    chan->refreshVarcTime =					
	      currentTime.t_sec + chan->refreshRate;			
	}								
									
	value[i] = port->buffer.u16[j];				
      }									
    }									
    break;								
  }
    */
  default:
    fatalNOTIMPL ();	/*NCV*/
  }

#undef M
#undef ML2
#undef C16
#undef C3264
   
  if (b->inputFailed)
    clearBlockInputFailed (b);  

  if (plc->numScans > 0) {
    num = ((plc->numScans - plc->numFailedScans) / plc->numScans) * 100;
  }
/*   vsys_msg_printf(0, "Now figuring throughput: total %d failed %d throughput %f",  */
/* 		  (int)plc->numScans, (int)plc->numFailedScans, num); */
  if (vdb_test_chix(&plc->throughput_chix)) {
    if (plc->numScans > 0) {
      num = ((plc->numScans - plc->numFailedScans) / plc->numScans) * 100;
    } else 
      num = 0;
    if (vdb_rput (&plc->throughput_chix, &num) != VDB_ERR_OK)
      (void) errorWDPUT (&plc->throughput_chix);	       
  }
/*   vsys_msg_printf(0, "Now figuring staledata: %d\n",  plc->runningFailedScans); */
  if (vdb_test_chix(&plc->staledata_chix)) {
    if (vdb_iput (&plc->staledata_chix, &(plc->runningFailedScans)) != VDB_ERR_OK)
      (void) errorWDPUT (&plc->staledata_chix);	       
  }  
}

/*
 *********************************************************************
 */

static void scan (ScanList* sl) {

  ScanBlock* b = sl->block;
  Port* port = sl->port;
  vtime currentListTime;
  char hard_time[800];
  struct vdb_descrip_s time_d;
  int status;

   vscannerLogMsgNoprint("Scan start"); 
/*    vsys_msg_printf(0, "Scan start"); */

  if (port->startScanChannel != NULL) {
    Vtimer* timer = sl->timer;
    VARC_LOCK ();
    port->varcScanId++;
    if (vlogc_log_channel (data.varc.log, &timer->callbackTime,
			   port->startScanChannel, &port->varcScanId) != 1)
      fatalVARC ();	/*NCV*/
    VARC_UNLOCK ();
  }

  (void) vsys_get_time (&currentListTime);
  VSYS_STRING_FILL(time_d, hard_time, 800);
  vsys_asctime(&time_d, &currentListTime, &status);
  if (data.control_design && data.group_timestamps) {
    //    vsys_msg_printf(0, "Figuring current list time: rate %d time %d sec %d nsec %s time\n", b->rate, currentListTime.t_sec, currentListTime.t_nsec, hard_time);
    currentListTime.t_sec = currentListTime.t_sec - (currentListTime.t_sec % b->rate);
    currentListTime.t_nsec = 0;  
    VSYS_STRING_FILL(time_d, hard_time, 800);
    vsys_asctime(&time_d, &currentListTime, &status);
    //    vsys_msg_printf(0, "Figuring possible time: %d? %s?\n", (int)334798*3600, hard_time);
  }
  currentListTime.t_nsec = 0;

  if (b == NULL) {
    vsys_msg_printf(0, "Called with a NULL scan block!");
    return; 
  }
  do {
    if (! b->inputFailed) {
      scanOneBlock (b, &currentListTime);
    } else 
      vsys_msg_printf(0, "Not scanning due to failure!");  
  } while ((b = b->next) != NULL);

  VSYS_STRING_FILL(time_d, hard_time, 800);
  vsys_asctime(&time_d, &currentListTime, &status);
  vscannerLogMsgNoprint("Scan done %d %d %s", sl->rate, sl->clocksync, hard_time);

  if (port->endScanChannel != NULL) {
    vtime currentTime;
    Vtimer* timer = sl->timer;
    (void) vsys_get_time (&currentTime);
    VARC_LOCK ();
    if (vlogc_log_channel (data.varc.log, &currentTime,
			   port->endScanChannel, &timer->nextTime) != 1)
      fatalVARC ();	/*NCV*/
    VARC_UNLOCK ();
  }
}

/*
 *********************************************************************
 * Output to hardware values of changed channels from the specified list.
 *********************************************************************
 */

static int mbputListChanges_PLCK (AddrList* list) {

  Plc* plc = list->plc;
  Port* port = plc->port;
  unsigned n = list->o.numChanged;
  int type = list->type;
  VUINT64 int64Value;
  VUINT32 int32Value;
  VUINT16 int16Value;
  VUINT8 binary;
   char *string; 
  unsigned i;

  for (i = 0; i < n; i++) {

    void** ptr = list->o.first;
    unsigned offset = ptr - list->o.list;
    Channel* chan = &list->chan[offset];
    unsigned addr = chan->pub.addr;
    bool putFailed;

    list->o.first = (void**)*ptr;
    list->o.numChanged--;
    list->o.changed[offset] = 0;
    plc->numChannelsChanged--;
    
    switch (type) {
    case LIST_TYPE_OUTPUT_R1:
      binary = list->value.binary [offset];
      unlockPort (port);
      putFailed = mymbBput (plc, chan, binary);
      break;
    case LIST_TYPE_OUTPUT_R16:
      int16Value = list->value.int16 [offset];
      unlockPort (port);
      putFailed = mymbI16put (plc, chan, int16Value);
      break;
    case LIST_TYPE_OUTPUT_HOLDING:
      int16Value = list->value.int16 [offset];
      unlockPort (port);
      putFailed = mymbI16Holdingput (plc, addr, int16Value);
      break;
    case LIST_TYPE_OUTPUT_R32:
    case LIST_TYPE_OUTPUT_HOLDING_R32:
      {
	/* CML added the float stuff here */
	float float32Value;
/* 	vsys_msg_printf(0, "Offset for 32-bit is %d, ", offset); */
	switch (chan->pub.hwDtype) {
	case VDB_DTYPE_UINT: 
	case VDB_DTYPE_INT: 
	  int32Value = list->value.int32[offset];
/*        vsys_msg_printf(0, "value %f\n",int32Value); */
	  unlockPort (port);
	  putFailed = mymbI32put (plc, chan, int32Value);
	  break;
	  
	case VDB_DTYPE_FLOAT: 
	  float32Value = list->value.float32[offset];
/* 	  vsys_msg_printf(0, "value %f\n",float32Value);  */
	  unlockPort (port);
	  putFailed = mymbI32put_float (plc, chan, float32Value);
	  break;
	default:
	  putFailed = 0;	/*NCV*/
	  fatalNOTIMPL ();	/*NCV*/
	  break;
	}      
      }
      
      break;

    case LIST_TYPE_OUTPUT_R64:
    case LIST_TYPE_OUTPUT_HOLDING_R64:
      int64Value = list->value.int64 [offset];
      unlockPort (port);
      putFailed = mymbI64put (plc, chan, int64Value);
      break;
    case LIST_TYPE_OUTPUT_STRING:
      string = list->value.string [offset];
      unlockPort (port);
      putFailed = mymbStringput (plc, chan, string);
      break;

    default:
      putFailed = 0;	/*NCV*/
      fatalNOTIMPL ();	/*NCV*/
    }			/*NCV*/

    lockPort (port);

    if (putFailed) {

      vtime currentTime;

      (void) vsys_get_time (&currentTime);

      /* We must re-insert the failed entry into the list. We try to
	 re-insert it at the beginning of the list. But the channel
	 may have changed and may have already been appended to the
	 end of list by the event routine. */

      if (! list->o.changed [offset]) {
	list->o.changed [offset] = 1;
	if (list->o.numChanged == 0) {
	  list->o.last = &list->o.list[offset];
	  list->o.list[offset] = NULL;
	}
	else
	  list->o.list[offset] = list->o.first;
	list->o.first = &list->o.list[offset];
	plc->numChannelsChanged++;
	list->o.numChanged++;
      }

      return FAILURE;
    }
    else {
      vtime currentTime;
      (void) vsys_get_time (&currentTime);
      if (chan->flags.ioError)
	clearIOError (chan, &currentTime, 1);
    }
  }

  return SUCCESS;
}

/*
 *********************************************************************
 * Set IO_ERROR field on all channels in an output list.
 *********************************************************************
 */

static void setIOErrorList_PLCK (AddrList* list) {

  Plc* plc = list->plc;
  Port* port = plc->port;
  void** ptr = list->o.first;
  unsigned i;

  for (i = 0; i < list->o.numChanged; i++) {
    unsigned offset = ptr - list->o.list;
    /* FIXME: setIOError is called with port mutex locked!
       I temporary disabled mutex locking in setioerror.
       I am not sure about the permanent solution. One could
       be to unlock before the call and lock after. But the
       list of changed channels may change in between. So
       I will probably have to start  from the beginning of
       the list. Another solution is that maybe I do not need
       that lock/unlock in setIOError. It seems that it is
       only used to not write a record with an old timestamp,
       maybe there is a better solution for that, without mutexes. */
    setIOError (port, &list->chan[offset], 1);
    ptr = (void **)*ptr;
  }
}

/*
 *********************************************************************
 */

static void scanCallback (void* arg) {

  ScanList* sl = (ScanList*)arg;
  Port* port = sl->port;

#if vmbCONF_TRACE_SCAN
  if (strcmp (port->name, "port1") == 0)
    logMsg ("scan %s start %u,%u %u,%u",
	    port->name,
	    sl->timer->callbackTime.t_sec,
	    sl->timer->callbackTime.t_nsec,
	    sl->timer->nextTime.t_sec,
	    sl->timer->nextTime.t_nsec);
#endif

  unlockPort (port);
  scan (sl);
  if (data.disconnect)
    disconnectPort (port);
  if (port->freeFlag) {
    for (; sl != NULL; sl = sl->next) {
      ScanBlock* sb = sl->block;
      ScanBlock* next;
      /* 	  ScanBlock* newsb;*/
      while (sb != NULL) {
	next = sb->next;
	myFree(sb);
	sb = next;
      }
    }
    if (sl != NULL)
      myFree(sl);
  }
  port->freeFlag = 0;
  lockPort (port);
  
  if (port->newInterval) {
    unsigned i;
    for (i = 0; i < port->numPlcs; i++) {
      Plc* plc = port->plc[i];
      DO_INPUT_LISTS (setInitialValueFlag_PLCK);
    }
    port->newInterval = 0;
  }

#if vmbCONF_TRACE_SCAN
  if (strcmp (port->name, "port1") == 0)
    logMsg ("scan %s end", port->name);
#endif
}

/*
 *********************************************************************
 */

/* CML added this function, to allow for a delay before scanning initial scan */
static int scanwait(void *scanlist) {
  ScanList *sl = ((ScanList *)scanlist);
  vtime current_vtime;
  struct time_word current_time; 
  unsigned delay_min = 0;
  int status = 0;
  int scantime_min = 0;
  unsigned curtime_min = 0; 
  unsigned clocksync_holder = 0;
  Vtimer* timer;

/*   vsys_msg_printf(0, "Scanwait!");    */

  vsys_get_time (&current_vtime);
  vsys_conv_to_vec_time(&current_time, &current_vtime, &status);

  if (status) {

    /* Converts milliseconds to minutes */
    if (sl->rate != 0 && sl->rate / 1000 != 0)
      scantime_min = (sl->rate / 1000) / 60;
    else
      scantime_min = 0;
    
    /* Converts minutes and hours to minutes */
    curtime_min = (current_time.t_hour * 60) + current_time.t_min;
    
    /* If the current minutes are less than the clocksync, no need to 
       adjust, just subtract. */
    if (curtime_min < sl->clocksync) {
/*       vsys_msg_printf(0, "Cur min %d clocksync %d 1", curtime_min, sl->clocksync); */
      delay_min = sl->clocksync - curtime_min;
    } else { 
      /* Set the delay to tomorrow */
/*         vsys_msg_printf(0, "Cur min %d clocksync %d 2", curtime_min, sl->clocksync); */
	  delay_min = sl->clocksync + (24 * 60) - curtime_min;
    }
  /* Otherwise, crank up the clocksync by its own increment until it is 
	 larger than the current time, then subtract. */
      /*      clocksync_holder = sl->clocksync;
	      while (1) {
	      if (curtime_min < clocksync_holder) {
	      delay_min = clocksync_holder - curtime_min;
	      break;
	      }
	      clocksync_holder += sl->clocksync;
	      }
	      }
      */
      /* Wait until this delay */    

    if (delay_min > 840) { /* If greater than 14 hours */
      /* Set the delay to the next increment */
/*       vsys_msg_printf(0, "Defaulting to the next time increment"); */
      if (current_time.t_min > 30)
	delay_min = (60 - current_time.t_min);
      else 	
	delay_min = (30 - current_time.t_min);
    }

/*       vsys_msg_printf(0, "The delay is %d", delay_min); */
    if (delay_min != 0 && delay_min != sl->clocksync) {
      
      if (60 - current_time.t_sec != 0)
	delay_min--;
      vsys_msg_printf(0, "Delay time is %d hours, %d minutes, %d seconds\n", delay_min / 60, delay_min % 60, 60 - current_time.t_sec); 
      vsys_wait((delay_min * 60000)+ (60 - current_time.t_sec) * 1000);
    }
  }

  /* And register the scan timer */
  lockPort (sl->port);
  
  if ((timer = vtimerCreate (sl->port->timerBank)) == NULL ||
      vtimerSetCallback (timer, scanCallback, sl) != VTIMER_OK ||
      vtimerEnableImmediateMode (timer) != VTIMER_OK ||
      vtimerEnableAutomaticMode (timer) != VTIMER_OK)
    fatalVTIMER (sl->port->timerBank);
  if (data.schedule != NULL) {
    unsigned index = sl->rate - 1;
    if (vtimerSetSchedule (timer, data.schedule[index]) != VTIMER_OK)
      fatalVTIMER (sl->port->timerBank); 
  }
  else {
    unsigned rateMs = sl->rate * data.scanRateUnitMs;
    if (vtimerSet (timer, rateMs / 1000.) != VTIMER_OK)
      fatalVTIMER (sl->port->timerBank);
  }
  sl->timer = timer;

  unlockPort (sl->port);

  return VSYS_OK;
}

/*
 *********************************************************************
 */

static void alarmCallback (void* arg) {

  Port* port = (Port*)arg;
  int id = -1;

  if (port == data.timerLoopPort && data.exitFlag)
    if (port->timerBank != NULL)
      vtimerThreadExit (port->timerBank);

  if (port->scanFlag) {
    ScanList* sl;
    if (port->PLCs_to_scan[0] == 666) {
      for (sl = port->scanList; sl != NULL; sl = sl->next)
	scanCallback (sl);
      port->scanFlag = false;
    } else {
      ScanList* newsl;
      int x;
/*      vsys_msg_printf(0, "Got an unscheduled scan!"); */
      for (x = 99; x > 0; x--) {
	if (port->PLCs_to_scan[x] == 666 && port->PLCs_to_scan[x - 1] != 666) {
	  id = port->PLCs_to_scan[x-1];
	  port->PLCs_to_scan[x-1] = 666;
	} else
	  continue;
/* 	vsys_msg_printf(0, "Found id %d!", id); */
	newsl = (ScanList* )fatalMalloc0 (sizeof (ScanList));
	newsl->port = port;
	for (sl = port->scanList; sl != NULL; sl = sl->next) {
	  ScanBlock* sb;
/* 	  ScanBlock* newsb;*/
	  for (sb = sl->block; sb != NULL; sb = sb->next) {
	    AddrList* addrList = sb->addrList;
	    unsigned numChannels = sb->numChannels;
	    Channel* chan = &addrList->chan[sb->offset];
	    Plc* plc = addrList->plc;
	    if (plc->unit == id) {
	      if (newsl->block == NULL) {
		newsl->block = copy_scanblock(sb);
		newsl->block->next = NULL;
	      } else {
		ScanBlock* b = newsl->block;
		while (b->next != NULL)
		  b = b->next;
		b->next = copy_scanblock(sb);
	      }
	    }
	  }
	}	
	port->freeFlag = true;
	port->scanFlag = false;
	scanCallback (newsl);
      }
    }
  }
}



/*
 *********************************************************************
 */

static void inputRetryCallback (void* arg) {

  Plc* plc = (Plc *)arg;
  int reset = 0;
  vtime currentListTime;

#if vmbCONF_TRACE_SCAN
  if (strcmp (plc->port->name, "port1") == 0)
    logMsg ("retry %s start %u,%u %u,%u",
	    plc->name,
	    plc->inputRetryTimer.timer->callbackTime.t_sec,
	    plc->inputRetryTimer.timer->callbackTime.t_nsec,
	    plc->inputRetryTimer.timer->nextTime.t_sec,
	    plc->inputRetryTimer.timer->nextTime.t_nsec);
#endif

  /*    vsys_msg_printf (0, "input retry %s start %u,%u %u,%u",
	  plc->name,
	  plc->inputRetryTimer.timer->callbackTime.t_sec,
	  plc->inputRetryTimer.timer->callbackTime.t_nsec,
	  plc->inputRetryTimer.timer->nextTime.t_sec,
	  plc->inputRetryTimer.timer->nextTime.t_nsec);
  */
  if (plc->numInputFailedBlocks != 0) {
    /* fixme0: PLCSBLIST */
    Port* port = plc->port;
    ScanList* sl;
    for (sl = port->scanList; sl != NULL; sl = sl->next) {
      ScanBlock* sb;
/*       vsys_msg_printf(0, "Retrying for scan list %d %d", sl->rate, sl->clocksync);  */
      (void) vsys_get_time (&currentListTime);
      for (sb = sl->block; sb != NULL; sb = sb->next) {
/* 	vsys_msg_printf(0, "Retrying for scan block starting at %d", sb->offset); */
	if (sb->cur_retries >= data.retry.num_retries) {
/* 	  vsys_msg_printf(0, "RESETTING RETRIES INPUT (%d %x) for sb!\n", sb->cur_retries, sb->cur_retries); */ 
	  sb->cur_retries = 0;
	  reset = 1;
	} else {
/* 	  vsys_msg_printf(0, "Incrementing retry count %d", sb->cur_retries); */
	  sb->cur_retries++;
	  reset = 0;
	}
	if (sb->inputFailed && sb->plc == plc) {
	  if (reset == 1) {
/*  	    vsys_msg_printf(0, "CLEARING BLOCK\n");  */
	    clearBlockInputFailed (sb);
	    break;
	  } 	  
/* 	  vsys_msg_printf(0, "SCANNING BLOCK\n");  */
	  scanOneBlock (sb, &currentListTime);

	  /* If failed, do not retry the rest of the list (for consistency
	     with regular scan. If a regular scan fails, we set inputFailed
	     flag on the remaining blocks belonging to the same PLC. */

	  /*if (sb->inputFailed)
	    break;*/
	}
      }
    }
/*     vsys_msg_printf(0, "OUT OF INPUT LOOP!\n");  */
    if (data.disconnect)
      disconnectPort (port);
  }
  
  if (plc->numInputFailedBlocks != 0) {      
    if (reset == 0) {
/*       vsys_msg_printf(0, "Not resetting, setting another retry timer\n");  */
      setInputRetryTimer_PLCK (plc);
    } else {
      ScanList* sl;
      Port* port = plc->port;
      /*       vsys_msg_printf(0, "Resetting 1, removing retry timer/n");*/ 
      plc->inputRetryTimer.index = 0;
      for (sl = port->scanList; sl != NULL; sl = sl->next) {
	ScanBlock* sb;
	for (sb = sl->block; sb != NULL; sb = sb->next) {
/* 	    vsys_msg_printf(0, "Resetting counter and clearing errors for block starting with %d", sb->offset); */
	    sb->cur_retries = 0;
	    if (sb->plc == plc) {
	      if (sb->inputFailed)
		clearBlockInputFailed(sb);
	  }
	}
      }
    }
  } else {
    ScanList* sl;
    Port* port = plc->port;
/*     vsys_msg_printf(0, "Resetting 2, removing retry timer\n");  */
    plc->inputRetryTimer.index = 0;
    for (sl = port->scanList; sl != NULL; sl = sl->next) {
      ScanBlock* sb;
      for (sb = sl->block; sb != NULL; sb = sb->next) {
/* 	  vsys_msg_printf(0, "Resetting counter for block starting with %d", sb->offset); */
	    sb->cur_retries = 0;
      }
    }
  }

#if vmbCONF_TRACE_SCAN
  if (strcmp (plc->port->name, "port1") == 0)
    logMsg ("retry %s end", plc->name);
#endif
}

/*
 *********************************************************************
 */

static void outputCallback (void* arg) {

  Plc* plc = (Plc *)arg;
  Port* port = plc->port;
  bool failed = false;

#if vmbCONF_TRACE_SCAN
  if (strcmp (plc->port->name, "port1") == 0)
    logMsg ("output %s start", plc->name);
#endif

/*     vsys_msg_printf (0, "output %s start", plc->name); */

#define M(list) do {						\
  if (list.o.numChanged != 0 && ! failed)			\
    failed = mbputListChanges_PLCK (&list) != SUCCESS;		\
  if (failed)							\
    setIOErrorList_PLCK (&list);				\
} while (0)

      M (plc->addrList.out.o1);
      M (plc->addrList.out.o16);
      M (plc->addrList.out.o32);
      M (plc->addrList.out.o64);   
      M (plc->addrList.out.oholding_o32);   
      M (plc->addrList.out.oholding_o64);   
#undef M

  if (failed) {
    if (! plc->outputFailed) {
      plc->outputFailed = true;
      if (plc->numInputFailedBlocks == 0)
	if (port->numFailedPlcs++ == 0)
	  incNumPlcFailedPorts (port);
    }
      setOutputRetryTimer_PLCK (plc);    
  }
  else {
    if (plc->outputFailed) {
      plc->outputFailed = false;
      plc->outputTimer.index = 0;

      if (plc->numInputFailedBlocks == 0)
	if (--port->numFailedPlcs == 0)
	  setAllPortsAreOK (port);
    }
  }

  if (data.disconnect)
    disconnectPort (port);

#if vmbCONF_TRACE_SCAN
  if (strcmp (plc->port->name, "port1") == 0)
    logMsg ("output %s end", plc->name);
#endif
}

/*
 *********************************************************************
 */

static void scanners (void) {

  VtimerBank* bank;
  Port* firstPort = NULL;
  Port* port;

  startVarchive ();

  for (port = data.portList; port != NULL; port = port->next) {
    if (port->numInputChannels != 0 || port->numOutputChannels != 0) {

      ScanList* sl;
      unsigned i;
      Vtimer* timer;

      lockPort (port);

      if ((bank = port->timerBank = vtimerBankCreate (&port->mutex)) == NULL)
	fatalNOMEM ();	/*NCV*/
      if (firstPort == NULL) /* We scan the first port in the main thread. */ {
	if (vtimerDoNotCreateThread (bank) != VTIMER_OK)
	  fatalVTIMER (bank);
	firstPort = data.timerLoopPort = port;
      }
      if (port == firstPort || port->numInputChannels != 0)
	if (vtimerSetAlarmRoutine (bank, alarmCallback, port) != VTIMER_OK)
	  fatalVTIMER (bank);

      for (i = 0; i < port->numPlcs; i++) {
	Plc* plc = port->plc[i];
	if ((timer = vtimerCreate (bank)) == NULL ||
	    vtimerSetCallback (timer, inputRetryCallback, plc) != VTIMER_OK)
	  fatalVTIMER (bank);
	plc->inputRetryTimer.timer = timer;
	if ((timer = vtimerCreate (bank)) == NULL ||
	    vtimerSetCallback (timer, outputCallback, plc) != VTIMER_OK)
	  fatalVTIMER (bank);
	plc->outputTimer.timer = timer;

	if (data.startOutput) {
	  /* Perform the initial (at startup) output for all output channels */
	  if (plc->numOutputChannels)
	    if (vtimerSetVtime0 (timer) != VTIMER_OK)
	      fatalVTIMER (bank);
	}
      }

      for (sl = port->scanList; sl != NULL; sl = sl->next) {
	/* CML added the following, to allow a start delay */
	if (sl->clocksync != 0) {
/* 	  vsys_msg_printf(0, "Clocksync!"); */
	  vsys_create_thread(64000, scanwait, sl, &sl->delay_thread);
	} else {
/* 	  vsys_msg_printf(0, "No clocksync!"); */
	  if ((timer = vtimerCreate (sl->port->timerBank)) == NULL ||
	      vtimerSetCallback (timer, scanCallback, sl) != VTIMER_OK ||
	      vtimerEnableImmediateMode (timer) != VTIMER_OK ||
	      vtimerEnableAutomaticMode (timer) != VTIMER_OK)
	    fatalVTIMER (sl->port->timerBank);
	  if (data.schedule != NULL) {
	    unsigned index = sl->rate - 1;
	    if (vtimerSetSchedule (timer, data.schedule[index]) != VTIMER_OK)
	      fatalVTIMER (sl->port->timerBank); 
	  }
	  else {
	    unsigned rateMs = sl->rate * data.scanRateUnitMs;
	    if (vtimerSet (timer, rateMs / 1000.) != VTIMER_OK)
	      fatalVTIMER (sl->port->timerBank);
	  }
	  sl->timer = timer;
	}
      }
      unlockPort (port);
    }
  }

  port = firstPort;
  if (port == NULL)
    fatalINTERR ();	/*NCV*/
  bank = port->timerBank;
  lockPort (port);
  if (vtimerLoop (bank) != VTIMER_OK) {
    //    vsys_msg_printf(0,"now!\n");
    logMsg ("%s", vtimerStrerror (bank));
  }
  unlockPort (port);

  for (port = data.portList; port != NULL; port = port->next) {
    if (port->numInputChannels != 0 || port->numOutputChannels != 0) {
      lockPort (port);
      if (port->timerBank != NULL) {
	if (vtimerBankDelete (port->timerBank) != VTIMER_OK)
	  fatalVTIMER (port->timerBank);
	port->timerBank = NULL;
      }
      unlockPort (port);
    }
  }
}

/*
 *********************************************************************
 *********************************************************************
 ***
 ***  Parse command line and configuration file
 ***
 *********************************************************************
 *********************************************************************
 */

static void checkCommTypeOptions (Vcli* cli, Port* port, bool cfgFile) {

  char* option1 = NULL;
  char* option2 = NULL;

  if (port->numTypes != 0) {
    if (port->numTypes == 1) 
      port->type = port->typeList[0];
    else if (port->numTypes != 2)
      fatalINVTYPE ();
      
      /*      unsigned i0 = port->typeList[0] == VMB_TYPE_TCP ? 0 : 1;
      unsigned i1 = i0 == 0 ? 1 : 0;
      if (port->typeList[i0] != VMB_TYPE_TCP ||
	  (port->typeList[i1] != VMB_TYPE_RTU &&
	   port->typeList[i1] != VMB_TYPE_ASCII))
	fatalINVTYPE ();
      port->type = port->typeList[i1] == VMB_TYPE_RTU ?
      VMB_TYPE_TCP_RTU : VMB_TYPE_TCP_ASCII;
      }*/
    }

#define M(x)						\
  if (cfgFile) {					\
    if (vcliPresent (cli, "PORT." x) == VCLI_PRESENT)	\
      option2 = x;					\
  }							\
  else {						\
    if (vcliPresent (cli, x) == VCLI_PRESENT)		\
      option2 = x;					\
  }

  /*  if (port->type == VMB_TYPE_TCP ||
      port->type == VMB_TYPE_TCP_RTU ||
      port->type == VMB_TYPE_TCP_ASCII) {
      option1 = "TCP";*/
    M ("BAUDRATE");
    M ("PARITY");
    M ("STOP");
    M ("DELAYS");
/*   } 
  else if (port->type == VMB_TYPE_RTU ||
	   port->type == VMB_TYPE_ASCII) {
	   option1 = port->type == VMB_TYPE_RTU ? "RTU" : "ASCII";*/
    M ("TCPPORT");
    M ("PORT");
  /* } */

#undef M

  if (option1 != NULL && option2 != NULL)
    fatalCONFLICT (option1, option2);
}

/*
 *********************************************************************
 * Common qualifier for command line and config file. If you are adding
 * a new qualifier, do not forget to change configFilePort routine.
 *********************************************************************
 */

#define PORT_QUALIFIERS()						  \
    "define type type",							\
    "keyword HY85156", (void*) VMB_TYPE_85156,				\
    "keyword HY8505", (void*) VMB_TYPE_8505,				\
    "keyword HY8506", (void*) VMB_TYPE_8506,				\
    "keyword HY8601", (void*) VMB_TYPE_8601,				\
    "keyword HY8402", (void*) VMB_TYPE_8402,				\
    "keyword HY8403", (void*) VMB_TYPE_8403,				\
    "keyword HY8413", (void*) VMB_TYPE_8413,				\
    "keyword HY8005", (void*) VMB_TYPE_8005,				\
    "keyword HY8512", (void*) VMB_TYPE_8512,				\
    "keyword HY8411", (void*) VMB_TYPE_8411,				\
    "keyword HY8404", (void*) VMB_TYPE_8404,				\
    "keyword HY8408", (void*) VMB_TYPE_8408,				\
    "define type parity",						\
    "k NONE", (void*) SL_PARITY_NONE,					\
    "k EVEN", (void*) SL_PARITY_EVEN,					\
    "k ODD", (void*) SL_PARITY_ODD,					\
    "define type voltrange",						\
      "k UNIPOLAR_5V", (void*) RANGE_5V_UNIPOLAR_8404,		      	\
      "k UNIPOLAR_10V", (void*) RANGE_10V_UNIPOLAR_8404,		\
      "k BIPOLAR_5V", (void*) RANGE_5V_BIPOLAR_8404,		       	\
      "k BIPOLAR_10V", (void*) RANGE_10V_BIPOLAR_8404,		       	\
      "k BIPOLAR_2V5", (void*) RANGE_2V5_BIPOLAR_8404,		       	\
      "k V2V5_TO_7V5", (void*) RANGE_2V5_TO_7V5_8404,	      		\
    "define type inputtype",						\
      "k DIF", (void*) INPUTTYPE_DIF,	      		\
      "k SE", (void*) INPUTTYPE_SE,	      		\
      "k PT100", (void*) INPUTTYPE_PT100,	      		\
      "k TC", (void*) INPUTTYPE_TC,	      		\
      "k PTB", (void*) INPUTTYPE_PTB,	      		\
      "k TCB", (void*) INPUTTYPE_TCB,	      		\
    "define type delays",						\
    "k RTS $uint", &port->rtsDelay,					\
    "k turnaround $uint", &port->turnaroundDelay,			\
    "q type type type list", &port->numTypes, &port->typeList,		\
    "q delays type delays list", NULL,					\
    "q stop $uint range 1:4 default 1", &port->stop,			\
    "q parity type parity", &port->parity,				\
    "q baudRate $uint default 9600", &port->baud,			\
    "q timeout $uint", &port->timeout,					\
    "q startScanChannel $string nohelp", &port->startScanChannelName,	\
    "q endScanChannel $string nohelp", &port->endScanChannelName,	\
    "q maxTransactTimeChannel $string nohelp", &port->maxChannelName,	\
    "q delay $int range -64:32000 default 0", &port->delay,				\
    "q halfduplex $uint range 0:2 default 0", &port->halfduplex,     	      	\
    "q delay485 $uint range 0:15 default 0", &port->delay485,		      	\
    "q databits $uint range 5:8 default 8", &port->databits,		      	\
    "q flowctl $uint default 0", &port->flowctl,			\
      "q portnum $uint range 0:7 default 0", &port->portnum,			\
    "q ipslot $string", &port->ipslot,	        \
    "q axis $uint default 1", &port->axis,				\
    "q startspeed $uint default 100", &port->startspeed,		\
    "q maxspeed $uint default 400", &port->maxspeed,			\
    "q ramp $uint default 64", &port->ramp,				\
    "q vectornum $uint range 0:255 default 0", &port->vectornum,      	\
    "q doram $uint default 0", &port->doram,				\
    "q clocksource $uint range 0:1 default 0", &port->clocksource,		\
    "q clockrate $uint default 13", &port->clockrate,			\
    "q inhibit $uint range 0:3 default 0", &port->inhibit,    		\
    "q inputtype type inputtype default TCB", &port->inputtype,	    	\
    "q gain $uint range 1:128 default 64", &port->gain,			\
    "q vmeslot $uint default 0", &port->vmeslot,			\
    "q debrate $uint default 0", &port->debrate,			\
    "q debmask $uint default 0", &port->debmask,			\
    "q scanrate $uint default 0", &port->scanrate,			\
    "q dir $uint default 1", &port->dir,				\
    "q intmask $uint default 0", &port->intmask,			\
    "q pwidth $uint range 0:9 default 0", &port->pwidth,				\
    "q pmask $uint default 0", &port->pmask,				\
    "q scaler $uint default 255", &port->scaler,				\
      "q soverflow $uint default 0", &port->overflow,			\
    "q itrstate $uint range 0:1 default 0", &port->itrstate,  		\
    "q aitype $uint range 0:1 default 0", &port->aitype,		\
    "q samples $uint range 0:1 default 1", &port->samples,	       	\
    "q spacing $uint default 1", &port->spacing,			\
    "q trigger $uint default 0", &port->trigger,			\
    "q voltrange type voltrange default UNIPOLAR_5V", &port->range,	\
    "q clock $uint default 0", &port->clock				 

/*
 *********************************************************************
 * Routines to open, read and close a file.
 *********************************************************************
 */

typedef struct {
  char errstr[256];
  FILE* file;
  char* buffer;
  unsigned bufferSize;
  unsigned lineNumber;
} MyFile;

static MyFile* myFileInit (void) {
  return (MyFile*)VMEM_MALLOC0 (sizeof (MyFile));
}

static int myFileOpen (MyFile* file, char* name) {
  file->lineNumber = 0;
  if ((file->file = fopen (name, "r")) == NULL) {
    (void) sprintf (file->errstr, "errno = %d", errno);
    return -1;
  }
  file->errstr[0] = 0;
  return 0;
}

static int myFileClose (MyFile* file) {
  int status = fclose (file->file) ? -1 : 0;
  if (status) (void) sprintf (file->errstr, "errno = %d", errno);
  file->file = NULL;
  return status;
}

static int myFileDestroy (MyFile* file) {
  if (file->buffer != NULL) VMEM_FREE (file->buffer);
  VMEM_FREE (file);
  return 0;
}

/*
 *********************************************************************
 * Combines multiple lines terminated with a backslash into a single.
 * Return -1 on error, 0 on EOF, 1 on success.
 *********************************************************************
 */

static int myFileReadExtendedLine (MyFile* file) {

  unsigned totalLen = 0;
  unsigned remSize = file->bufferSize;	/* size remaining */
  bool prevLineWithoutNewLine = 0;
  bool contFlag = 0;			/* continuation line flag */

  while (1) {

    if (remSize < 512) {
      file->bufferSize += 512;
      file->buffer = (char* )realloc (file->buffer, file->bufferSize);
      //      file->buffer = fatalRealloc (file->buffer, file->bufferSize);
      remSize += 512;
    }

    if (fgets (file->buffer + totalLen, remSize, file->file) == NULL) {
      if (ferror (file->file)) {
	(void) sprintf (file->errstr, "errno = %d", errno);	/*NCV*/
	return -1;					/*NCV*/
      }
      if (! feof (file->file)) {
	(void) sprintf (file->errstr, "! feof");	/*NCV*/
	return -1;					/*NCV*/
      }
      if (contFlag) {
	(void) sprintf (file->errstr, "unexpected eof");
	return -1;
      }
      return totalLen == 0 ? 0 : 1;
    }
    else {
      char* p = file->buffer + totalLen;
      unsigned len = strlen (p);

      if (! prevLineWithoutNewLine) file->lineNumber++;

      /* any line should have a non zero length */
      if (len == 0) {
	(void) sprintf (file->errstr, "len == 0");	/*NCV*/
	return -1;					/*NCV*/
      }

      /* if there is a newline char, replace it with 0; remember
	 that there was a newline */
      if (p[len - 1] != '\n') {
	prevLineWithoutNewLine = 1;
	totalLen += len;
	remSize -= len;
      }
      else {
	p[len - 1] = 0;
	len--;

	/* check if there is a backslash at the end */
	contFlag = 0;
	if (len != 0 && p[len - 1] == '\\') {
	  contFlag = 1;
	  p[len - 1] = 0;
	  len--;
	}

	totalLen += len;
	remSize -= len;
	if (! contFlag)
	  return 1;
      }
    }
  }

#ifdef DUNIX
  return 0; /* We should never come here, but dunix compiler compains anyway */
#endif
}

/*
 *********************************************************************
 */

static void configFilePlc (Plc* plc) {
  /* Make our own copy of vcli strings (the vcli ones will be invalid
     after the next vcli_parse or vcli_destroy. */
  plc->name = fatalStrdup (plc->name);
  plc->portName = fatalStrdup (plc->portName);
  
  if (plc->throughput_channel) {
    plc->throughput_channel = fatalStrdup(plc->throughput_channel);
  } else
    plc->throughput_channel = NULL;

  if (plc->staledata_channel) {
    plc->staledata_channel = fatalStrdup(plc->staledata_channel);
  } else
    plc->staledata_channel = NULL;

  if (plc->map.listSize != 0) {
    char** list = (char**)fatalMalloc (plc->map.listSize * sizeof list[0]);
    unsigned i;
    for (i = 0; i < plc->map.listSize; i++)
      list[i] = fatalStrdup (plc->map.list[i]);
    plc->map.list = list;
  }
  if (findPlc (plc->name) != NULL) fatalDUPPLC (plc->name);
  insertPlc (plc);
}

/*
 *********************************************************************
 */

static void configFilePort (Port* port, Vcli* cli) {

  Port* def = data.defaultPort;

  port->name = fatalStrdup (port->name);
  if (findPort (port->name) != NULL) fatalDUPPORT (port->name);

  checkCommTypeOptions (cli, port, true);

  /* Use command line options for missing values. */

#define M(name, field)					\
  if (vcliPresent (cli, "PORT." name) == VCLI_ABSENT)	\
    port->field = def->field
  M ("type", type);
  M ("a32", a32);
/*   M ("class", class); */
  M ("d16", d16);
  M ("device", device);
  M ("driver", driver);
  M ("endian", endian);
  M ("enron", enron);
  M ("stop", stop);
  M ("parity", parity);
  M ("baudRate", baud);
  M ("timeout", timeout);
  M ("startScanChannel", startScanChannelName);
  M ("endScanChannel", endScanChannelName);
  M ("maxTransactTimeChannel", maxChannelName);
  M ("delays.RTS", rtsDelay);
  M ("delays.turnaround", turnaroundDelay);
  mergeAddrOffsetStructures (&port->addrOffset, &def->addrOffset);
#undef M

  /* Make our own copy of vcli strings (the vcli ones will be invalid
     after the next vcli_parse or vcli_destroy. */

  port->device = fatalStrdup (port->device);
  port->driver = fatalStrdup (port->driver);
  if (!(port->ipslot)) {
      vsys_msg_printf(0, "No slot for card %s, defaulting to 'A'", port->name);
      port->ipslot = strdup("A");
    } else 
    port->ipslot = fatalStrdup (port->ipslot);
  port->startScanChannelName = fatalStrdup (port->startScanChannelName);
  port->endScanChannelName = fatalStrdup (port->endScanChannelName);
  port->maxChannelName = fatalStrdup (port->maxChannelName);

  insertPort (port);
}

/*
 *********************************************************************
 */

static void readConfigFile (char* name) {

  MyFile* file;
  Port* port = NULL;
  Plc* plc = NULL;
  Vcli* cli;

  if ((cli = vcliInit ()) == NULL) fatalNOMEM ();
  if ((file = myFileInit ()) == NULL) fatalNOMEM ();

  if (myFileOpen (file, name)) fatalCONFOPEN (file->errstr);

  while (myFileReadExtendedLine (file) > 0) {

    char* p = file->buffer;
    char buff[5000];
    int option;

    /* ignore comments and blank lines */
    if (p[0] == '#') continue;
    while (isspace (*p)) p++;
    if (*p == 0) continue;

    /* */
    if (plc == NULL) plc = fatalMallocPlc ();
    if (port == NULL) port = fatalMallocPort ();

    plc->throughput_channel = NULL;
    plc->staledata_channel = NULL;
    plc->numScans = 0.0;
    plc->numFailedScans = 0.0;
    plc->runningFailedScans = 0;

    if (vcliVparseCmd
	(cli, file->buffer,
	 "define type option",
	 //	 "k PLC syntax PLC", (void*) 1,
	 "k port syntax port", (void*) 2,
	 "p option type option", &option,
	/*  ENDIAN_TYPE (), */
	 /*	 OFFSET_TYPE ("plcOffset", plc->addrOffset),*/

	 /*
	 "define syntax PLC",
         "q a32 default 2", &plc->a32,
         "q d16 nonneg", &plc->d16,
	
         "q enron nonneg", &plc->enron,
	 "p name $string", &plc->name,
	 "q map $string list", &plc->map.listSize, &plc->map.list,
	 "q offsets type plcOffset list", NULL,
	 "q port $string", &plc->portName,
	 "q unit $uint default 1 range 1:255", &plc->unit,
	 "q throughput_channel $string", &plc->throughput_channel,
	 "q staledata_channel $string", &plc->staledata_channel,
	 "disallow map && offsets",
	 "disallow a32 && enron",*/

	 "define syntax port",
	 "p name $string", &port->name,
	 PORT_QUALIFIERS (),
	 NULL) != VCLI_OK) {
      (void) errorINVCONF (file->lineNumber);
      logFatal (vcliStrerror (cli));
    }


      /* This part essentially skips the PLC setup, as Hytec uses cards, not 
	 separate ports & PLCs */
      sprintf(buff, "%s", port->name);
      plc->name = fatalStrdup (buff);
      plc->portName = fatalStrdup (port->name);
      //  printf("plc->portname %s\n", plc->portName);
      plc->endian = ENDIAN_UNDEFINED;
      plc->next = NULL;
      configFilePlc (plc); 
      plc = NULL; 

    switch (option) {
   /*  case 1: */
/*       configFilePlc (plc); */
/*       plc = NULL; */
/*       break; */
    case 2:
      configFilePort (port, cli);
      port = NULL;
      break;
    }
  }

  myFree (plc);
  myFree (port);
  vcliDestroy (cli);
  if (file->errstr[0] != 0) fatalCONFREAD (file->errstr);
  if (myFileClose (file)) fatalCONFREAD (file->errstr);
  if (myFileDestroy (file)) fatalINTERR ();
}

/*
 *********************************************************************
 */

static void parsePlcList (unsigned listSize, char** list) {

  Plc* plc;
  unsigned i;

  for (i = 0; i < listSize; i++) {
    char name[512];
    char* unit;
    /* ignore too long names (better to ignore than to crash) */
    if (strlen (list[i]) < sizeof name) {
      /* make our own copy so that we replace ':' with '\0' */
      (void) strcpy (name, list[i]);
      unit = strchr (name, ':');
      if (unit != NULL) *unit++ = 0;
      if ((plc = findPlc (name)) == NULL) {
	plc = fatalMallocPlc ();
	plc->name = fatalStrdup (name);
	plc->unit = 1;
	insertPlc (plc);
      }
      plc->specifiedOnCmdline = 1;
      if (unit != NULL) {
	char* p = unit;
	plc->unit = strtol (p, &p, 10);
	if (*p) fatalINVUNIT (unit);
      }
    }
  }

  /* Keep only those specified with the qualifier */
  {
    Plc* prev = NULL;
    plc = data.plcList;
    while (plc != NULL) {
      if (plc->specifiedOnCmdline)
	prev = plc;
      else {
	if (prev == NULL)
	  data.plcList = plc->next;
	else
	  prev->next = plc->next;
      }
      plc = plc->next;
    }
  }
}

/*
 *********************************************************************
 * Set up references and set up class, a32 and offset PLC options.
 *********************************************************************
 */

static void setupPlcPortReferences (void) {

  Port* port;
  Plc* plc;
  char* portName;
  for (plc = data.plcList; plc != NULL; plc = plc->next) {
    if (plc->portName == NULL)
      portName = fatalStrdup("");
    else
      portName = fatalStrdup(plc->portName);
      
    // char* portName = (const char *)(plc->portName == NULL ? "" : (const char *)plc->portName);
    port = findPort (portName);
    if (port == NULL)
      fatalNOSUCHPORT (portName);
    /*    if (port->device == NULL)
	  fatalNODEVICE ();*/
    if (port->type == 0)
      fatalNOTYPE ();
    plc->port = port;
    port->numPlcs++;

    if (! plc->d16 && port->d16)
      plc->d16 = 1;
    if (! plc->enron && port->enron)
      plc->enron = 1;
/*     if (plc->class == CLASS_UNDEFINED) */
/*       plc->class = port->class == CLASS_UNDEFINED ? */
/* 	1 : port->class; */
    if (plc->endian == ENDIAN_UNDEFINED)
      plc->endian = port->endian == ENDIAN_UNDEFINED ?
	ENDIAN_LITTLE : port->endian;
    if (plc->a32 == 2)
      plc->a32 = port->a32 == 2 ? 0 : port->a32;
    mergeAddrOffsetStructures (&plc->addrOffset, &port->addrOffset);
   /*  if(portName) { */
/*       free(portName); */
/*       portName = NULL; */
/*     } */
  }

  for (port = data.portList; port != NULL; port = port->next) {
    unsigned n = port->numPlcs;
    if (n) {
      unsigned i;
      plc = data.plcList;
      port->plc = (Plc**)fatalMalloc (n * sizeof port->plc[0]);
      for (i = 0; i < n; i++) {
	while (plc->port != port) plc = plc->next;
	port->plc[i] = plc;
	plc = plc->next;
      }
    }
  }
}

/*
 *********************************************************************
 * Parse the name part of a -map option. If matches, return 1
 * and set 'value' to point to the beginning of the value.
 * Otherwise return 0.
 *********************************************************************
 */

static int mapOption (char* templatey, char* option, char** value) {

  char* p1 = templatey;
  char* p2 = option;
  bool star = 0;
  int retval = 0;

  while (1) {
    if (*p1 == '*') {
      star = 1;
      p1++;
    }
    if (toupper (*p1) != toupper (*p2))
      break;
    p1++;
    p2++;
  }

  if (*p2 == ':') {
    if (star || *p1 == 0) {
      *value = p2 + 1;
      retval = 1;
    }
  }

  return retval;
}

/*
 *********************************************************************
 * Parse -map offset:x option. Return 1 if option matches template,
 * 0 otherwise.
 *********************************************************************
 */

static int mapOption1 (char* plcName, char* templatey, char* option,
		       unsigned* valuePtr) {
  int retval = 0;
  char* valueStr;
  if (mapOption (templatey, option, &valueStr)) {
    if (isdigit (*valueStr)) {
      char* p;
      unsigned value = strtol (valueStr, &p, 10);
      if (*p == 0) {
	*valuePtr = value;
	retval = 1;
      }
    }
  }
  return retval;
}

/*
 *********************************************************************
 * Parse -map type:x$y option. Return 1 if option matches template,
 * 0 otherwise.
 *********************************************************************
 */

static int mapOption2 (char* plcName, char* templatey, char* option,
		       struct map_s* map, bool binary) {
  int retval = 0;
  char* valueStr;
  /* if (mapOption (templatey, option, &valueStr)) {
    if (isdigit (*valueStr)) {
      char* p;
      unsigned v1 = strtol (valueStr, &p, 10);
      if (*p == '#' || *p == '$' || *p == '@') {
	p++;
	if (isdigit (*p)) {
	  unsigned v2 = strtol (p, &p, 10);
	  if (*p == 0) {
	    struct mapEntry_s* entry = (struct mapEntry_s*)fatalMalloc0 (sizeof *entry);
	    entry->offset = v1;
	    entry->count = v2;
	    if (map->entry == NULL)
	      map->entry = (struct mapEntry_s *)entry;
	    else {
	      struct mapEntry_s* list = map->entry;
	      unsigned count;
	      while (list->next != NULL)
		list = list->next;
	      count = list->count;
	      if (binary)
		count = (count + 15) / 16;
	      if (list->offset + count > entry->offset)
		fatalMAPOPT (plcName, option);
	      list->next = entry;
	    }
	    map->totalCount += entry->count;
	    retval = 1;
	  }
	}
      }
    }
  }
  */
  return retval;
}

/*
 *********************************************************************
 * Parse -map options for all PLC. If -map is not specified for a PLC,
 * use command line -map options.
 *********************************************************************
 */

static void parseMapOptions (char** list, unsigned listSize) {
  Plc* plc;
  for (plc = data.plcList; plc != NULL; plc = plc->next) {
    char* name = plc->name;
    if (plc->map.listSize == 0) {
      plc->map.list = list;
      plc->map.listSize = listSize;
    }
    if (plc->map.listSize != 0) {
      unsigned i;
      plc->a32 = 1;
      for (i = 0; i < plc->map.listSize; i++) {
	char* opt = plc->map.list[i];
	/*	if (mapOption1 (name, "off*set", opt, &plc->map.offset));
#define M(x,y,z) else if (mapOption2 (name, x, opt, &plc->map.y, z))
        M ("bi*nary", input.binary, 1);
	M ("do*uble", input.float64, 0);
	M ("fl*oat", input.float32, 0);
	M ("int32", input.int32, 0);
	M ("int16", input.int16, 0);
	M ("outputbi*nary", output.binary, 1);
	M ("outputdo*uble", output.float64, 0);
	M ("outputfl*oat", output.float32, 0);
	M ("outputint32", output.int32, 0);
	M ("outputint16", output.int16, 0);
#undef M
else {*/
	  fatalMAPOPT (name, opt);
 /*      } */
	  
      }
    }
  }
}

/*
 *********************************************************************
 */

static void parseCommandLine (int argc, char** argv) {

  VcliPriority priority;
  unsigned numUserFiles;
  unsigned mapSize;
  unsigned plcListSize;
  Port* port = fatalMallocPort ();
  Vcli* cli;
  char** plcList;
  char** map;
  char** userFiles;
  char* configFileName;
  char* outputFileName;
  bool outputAppend;
  int ic_tech = 0;

  if ((cli = vcliInit ()) == NULL) fatalNOMEM ();

  if (vcliSetTitle (cli, programName) != VCLI_OK ||
      vcliSetVersion (cli, versionString) != VCLI_OK ||
      vcliSetCommandName (cli, commandName) != VCLI_OK ||
      vcliEnableSortedHelp (cli) != VCLI_OK ||
      vcliSetLibrary (cli, "vlog") != VCLI_OK ||
      vcliEnableCommandFileMode (cli) != VCLI_OK ||
      vscannerDefineStandardVcliGroup (cli) != VCLI_OK ||
      vcliVparse
      (cli,
       argc, argv,
       "define type show",
       "k dummyKeywordToFoolCLI $uint", NULL,
       "k plcs", &data.showPlcs,
       "k ports", &data.showPorts,
       "k scans", &data.showScanLists,
       "k decimal", &data.showDecimal,

       "define type fields",
#define M(func, default) "k " #func " $string " default, &data.fields.func.name
       M (address, "default 'hparam5'"),
       M (conversionRoutine, ""),
       M (dataPointType, "default 'hparam6'"),
       M (appRegBit, "default 'hparam8'"),
       M (thermocoupleType, "default 'hparam7'"),
       M (disposition, ""),
       M (refreshRate, ""),
       M (plcName, "default 'iotype'"),
       M (scanRate, "default 'hparam2'"),
       M (clockSync, ""),
       M (SDIaddress, ""),
#undef M

/*        ENDIAN_TYPE (), */
       "define type length",
       "k seconds $uint", &data.varc.lengthSec,
       "k KBytes $uint", &data.varc.lengthKb,

       "define type intervalLength",
       "k seconds $uint", &data.varc.intervalSec,
       "k KBytes $uint", &data.varc.intervalKb,

       "define type trace",
       "k dummyKeywordToFoolCLI $uint", NULL,
       "k controlChannel", &data.traceControlChannel,
       "k initComplete", &data.traceInitComplete,

	"define type disposition",
	"k BOTH", (void*) VHYTEC_DISPOSITION_BOTH,
	"k DATABASE", (void*) VHYTEC_DISPOSITION_DATABASE,
	"k VARCHIVE", (void*) VHYTEC_DISPOSITION_VARCHIVE,

       "group standard",
       "q bufferSize $uint", &data.varc.bufferSize,
       "q circular", &data.varc.circular,
       "q configFile $file", &configFileName,
#if !defined VMS
       "q startupOutput $uint default 1 range 0:1", &data.startOutput, 
#endif
       "q controlChannel $string", &data.controlChannelName,
       "q logChannel $string", &data.logChannelName,
       "q database $string list", &data.numDatabases, &data.databaseName,
       "q debug $uint", &data.debug,
       "q disconnect", &data.disconnect,
       "q disposition type disposition", &data.disposition,
       "q fields type fields list", NULL,
       "q fsync", &data.varc.fsync,
#if !defined VMS
       "q groupTimestamps $uint default 0 range 0:1", &data.group_timestamps, 
#endif
       "q intervalLength type intervalLength list", NULL,
       "q IV default 1", &data.varc.iv,
       "q latency $uint", &data.varc.latency,
       "q length type length list", NULL,
#if vmbCONF_LIST_PUTS
       "q listPut nohelp", &data.varc.useChannelLists,
#endif
       "q map $string list", &mapSize, &map,
       "q once $string list", &data.varc.onceListSize, &data.varc.onceList,
       "q output $file nohelp", &outputFileName,
       "q _outputAppend nohelp", &outputAppend,
       "q PLC $string list", &plcListSize, &plcList,
       "q priority $priority", &priority,
       "q rate $uint default 1000 range 1:1000000000", &data.scanRateUnitMs,
       "q restart", &data.varc.restart,
       "q numRetries $uint default 1000000000", &data.retry.num_retries,
       "q retryDelay $uint list", &data.retry.size, &data.retry.delay,
       "q schedule $string list", &data.scheduleSize, &data.schedule,
       "q setup", &data.setup,
       "q shared", &data.varc.shared,
       "q show type show list", NULL,
       "q timeSpan $uint nohelp", &data.timeSpan,
       "q trace type trace list nohelp", NULL,
       "q userFiles $file list", &numUserFiles, &userFiles,
       "q userInitRoutine $string", &data.userInitRoutine,
       "q userParameter $string", &data.env.commandLineParameter,
       "q varchive $file", &data.varc.name,
       "q wdChannel $string", &data.wd.channelName,
       "q wdRate $uint default 10 range 1:1000000000", &data.wd.rate,
       "dis latency && ! shared",
       "dis circular && ! (length && intervalLength)",
       "dis rate && schedule",
       PORT_QUALIFIERS (),
       NULL) != VCLI_OK)
    logFatal ("%s", vcliStrerror (cli));	/*NCV*/

  if (data.setup) {
/*     vsys_msg_printf(0, "Setup!"); */
    if (!vsys_create_process("vhytec_setup", NULL, NULL, 0)) {
      vsys_msg_printf(0, "Error spawing process to execute: vhytec_setup");
    }
    exit(0);
  }

  port->name = fatalStrdup ("default");
  insertPort (port);
  data.defaultPort = port;

  data.retry.cur_retries = 0;
  data.retry.cur_retries_output = 0;
/*   if (data.control_design || ic_tech) */
/*     data.control_design = 1; */
/*   else */
/*     data.control_design = 0; */
/*   set_control_design(data.control_design); */
//  set_show_decimal(data.showDecimal);

  if (data.retry.num_retries == 0)
    data.retry.num_retries = 1000000000;

  data.timeSpanPresent = vcliPresent (cli, "TIMESPAN") == VCLI_PRESENT;
  data.noline = vscannerGetNoline ();

  if (outputFileName != NULL) {
    char* mode;
    if (outputAppend) 
      mode = strdup("a");
    else
      mode = strdup("w");
    if (freopen (outputFileName, mode, stdout) == NULL) {
      perror ("unable to redirect stdout");	/*NCV*/
      vscannerExitFailure ();			/*NCV*/
    }
  }

  if (data.retry.size == 0) {
    static unsigned delay[] = {
      /* I do not have a good idea what these should be ... */
      1, 1, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10
    };
    data.retry.delay = delay;
    data.retry.size = VSYS_ARRAY_SIZE (delay);
  }

  checkCommTypeOptions (cli, port, false);

  if (configFileName != NULL)
    readConfigFile (configFileName);
  if (plcListSize != 0)
    parsePlcList (plcListSize, plcList);

  if (numUserFiles != 0) {
#ifndef VMS
    unsigned i;
#endif
    data.userFiles = (char **)fatalMalloc ((numUserFiles + 1) * sizeof *data.userFiles);
    (void) memcpy (data.userFiles, userFiles,
		   numUserFiles * sizeof *data.userFiles);
    data.userFiles[numUserFiles] = NULL;

    /* vsys_lib_find_routine expects a NULL terminated list. Also, some
       platforms do not seem to like file names without an extension
       (dunix for example). In my autotests I cannot easily specify
       file with extension, so I add the default extension. But do not
       add default extension on VMS, because otherwise logical names
       would not work. */

#ifndef VMS
    for (i = 0; i < numUserFiles; i++) {
      char* ext = strrchr (data.userFiles[i], '.');
      if (ext != NULL) {
	/* Check that this is really looks like an extension, i.e., not
	   special characters, just alphanums. It is of course possible
	   that the user has a file with a weird extension. Well, he would
	   have to change his mind. */
	char* p = ext;					/*NCV*/
	for (; *p; p++) if (! isalnum (*p)) break;	/*NCV*/
	if (*p) ext = NULL;				/*NCV*/
      }
      else {
	static char defaultExt[] = "." VSYS_LIB_SHR_EXT;
	unsigned len = strlen (data.userFiles[i]) + sizeof defaultExt;
	char* newy = (char*)fatalMalloc (len);
	(void) sprintf (newy, "%s%s", data.userFiles[i], defaultExt);
	data.userFiles[i] = newy;
      }
    }
#endif
  }

  if (vcliSetPriority (cli, &priority) != VCLI_OK)
    (void) logError ("%s", vcliStrerror (cli));	/*NCV*/
    
  setupPlcPortReferences ();
  parseMapOptions (map, mapSize);
  if (data.plcList == NULL) fatalNOPLC ();
}

/*
 *********************************************************************
 */

int main (int argc, char** argv) {

  vscannerSetNameAndVersion (programName, versionString);
  vscannerInitLogMutex ();
  setupEndianness ();
  parseCommandLine (argc, argv);
  vscannerBanner ();
  vscannerIgnoreSigfpe ();
  vscannerSetEventQueueSize ();
  buildScanLists ();
  callUserInitRoutine ();
  initMutexes ();
  if (! data.timeSpanPresent || data.timeSpan != 0) {
    openVarchive ();
    bookDbEvents ();
    vscannerInitStatusChannel ();
    initWdChannel ();
    initPortIO ();
    initWdAndTimeSpanTimers ();
    if (data.traceInitComplete)
      logMsg ("Initialization complete");
    scanners ();
    deleteTimerBank ();
    closeVarchive ();
  }
  return vscannerExitSuccess ();
}
